<?php

namespace NF\Tickets\Admin\Controller;

use XF\Admin\Controller\AbstractController;
use XF\Mvc\Entity\Entity;
use XF\Mvc\FormAction;
use XF\Mvc\ParameterBag;
use XF\Mvc\Reply\Exception;

class KBCategory extends AbstractController
{
    /**
     * @param              $action
     * @param ParameterBag $params
     * @throws \XF\Mvc\Reply\Exception
     */
    protected function preDispatchController($action, ParameterBag $params)
    {
        $this->assertAdminPermission('nfTickets');
    }

    /**
     * @return \XF\ControllerPlugin\AbstractPlugin|\NF\Tickets\ControllerPlugin\KBCategoryTree
     */
    protected function getCategoryTreePlugin()
    {
        return $this->plugin('NF\Tickets:KBCategoryTree');
    }

    /**
     * @return \XF\Mvc\Reply\View
     */
    public function actionIndex()
    {
        return $this->getCategoryTreePlugin()->actionList();
    }

    /**
     * @param ParameterBag $parameterBag
     *
     * @return \XF\Mvc\Reply\View
     * @throws \XF\Mvc\Reply\Exception
     */
    public function actionEdit(ParameterBag $parameterBag)
    {
        $category = $this->assertKBCategoryExists($parameterBag->kb_category_id);
        return $this->categoryAddEdit($category);
    }

    /**
     * @return \XF\Mvc\Reply\View
     */
    public function actionAdd()
    {
        /** @var \NF\Tickets\Entity\KBCategory $category */
        $category = $this->em()->create('NF\Tickets:KBCategory');
        $category->parent_category_id = $this->filter('parent_category', 'uint');

        return $this->categoryAddEdit($category);
    }

    /**
     * @param \NF\Tickets\Entity\KBCategory $category
     *
     * @return \XF\Mvc\Reply\View
     */
    protected function categoryAddEdit(\NF\Tickets\Entity\KBCategory $category)
    {
        $categoryRepo = $this->getKBCategoryRepo();
        $categories = $categoryRepo->findCategoryList()->fetch();
        $categoryTree = $categoryRepo->createCategoryTree($categories);

        $viewParams = [
            'category' => $category,
            'categoryTree' => $categoryTree,
        ];
        return $this->view('NF\Tickets:KB\Category\Edit', 'nf_tickets_kb_category_edit', $viewParams);
    }

    /**
     * @param \NF\Tickets\Entity\KBCategory $category
     *
     * @return FormAction
     */
    protected function categorySaveProcess(\NF\Tickets\Entity\KBCategory $category)
    {
        $entityInput = $this->filter([
            'title' => 'str',
            'description' => 'str',
            'parent_category_id' => 'uint',
            'display_order' => 'uint',

        ]);

        $form = $this->formAction();
        $form->basicEntitySave($category, $entityInput);

        return $form;
    }

    /**
     * @param ParameterBag $parameterBag
     *
     * @return \XF\Mvc\Reply\Redirect
     * @throws \XF\Mvc\Reply\Exception
     * @throws \XF\PrintableException
     */
    public function actionSave(ParameterBag $parameterBag)
    {
        $this->assertPostOnly();

        if ($parameterBag->kb_category_id)
        {
            $category = $this->assertKBCategoryExists($parameterBag->kb_category_id);
        }
        else
        {
            /** @var \NF\Tickets\Entity\KBCategory $category **/
            $category = $this->em()->create('NF\Tickets:KBCategory');
        }

        $this->categorySaveProcess($category)->run();

        return $this->redirect(
            $this->buildLink('tickets/kb/categories') . $this->buildLinkHash($category->getEntityId())
        );
    }

    /**
     * @param ParameterBag $parameterBag
     * @return \XF\Mvc\Reply\Redirect|\XF\Mvc\Reply\View
     */
    public function actionDelete(ParameterBag $parameterBag)
    {
        return $this->getCategoryTreePlugin()->actionDelete($parameterBag);
    }

    /**
     * @return \XF\Mvc\Reply\Redirect|\XF\Mvc\Reply\View
     */
    public function actionSort()
    {
        return $this->getCategoryTreePlugin()->actionSort();
    }

    /**
     * @param int     $id
     * @param array   $with
     * @param ?string $phraseKey
     * @throws \XF\Mvc\Reply\Exception
     * @return \NF\Tickets\Entity\KBCategory
     */
    protected function assertKBCategoryExists(int $id, array $with = [], string $phraseKey = null): \NF\Tickets\Entity\KBCategory
    {
        /** @noinspection PhpIncompatibleReturnTypeInspection */
        return $this->assertRecordExists('NF\Tickets:KBCategory', $id, $with, $phraseKey);
    }

    protected function getKBCategoryRepo(): \NF\Tickets\Repository\KBCategory
    {
        /** @noinspection PhpIncompatibleReturnTypeInspection */
        return $this->repository('NF\Tickets:KBCategory');
    }
}