<?php

namespace NF\Tickets\Admin\Controller;

use XF\Admin\Controller\AbstractField;
use XF\Entity\AbstractField as AbstractFieldEntity;
use XF\Mvc\FormAction;
use XF\Mvc\ParameterBag;

class TicketField extends AbstractField
{
    /**
     * @param $action
     * @param ParameterBag $params
     *
     * @throws \XF\Mvc\Reply\Exception
     */
    protected function preDispatchController($action, ParameterBag $params)
    {
        $this->assertAdminPermission('nfTickets');
    }

    /**
     * @return string
     */
    protected function getClassIdentifier()
    {
        return 'NF\Tickets:TicketField';
    }

    /**
     * @return string
     */
    protected function getLinkPrefix()
    {
        return 'tickets/fields';
    }

    /**
     * @return string
     */
    protected function getTemplatePrefix()
    {
        return 'nf_tickets_ticket_field';
    }

    /**
     * @param AbstractFieldEntity $field
     *
     * @return \XF\Mvc\Reply\AbstractReply
     */
    protected function fieldAddEditResponse(AbstractFieldEntity $field)
    {
        $reply = parent::fieldAddEditResponse($field);

        if ($reply instanceof \XF\Mvc\Reply\View)
        {
            /** @var \NF\Tickets\Repository\Category $ticketRepo */
            $ticketRepo = $this->repository('NF\Tickets:Category');
            $categories = $ticketRepo->findCategoryList()->fetch();
            $categoryTree = $ticketRepo->createCategoryTree($categories);

            /** @var \XF\Mvc\Entity\AbstractCollection $fieldAssociations */
            $fieldAssociations = $field->getRelationOrDefault(
                'CategoryFields',
                false
            );

            $reply->setParams([
                'categoryTree' => $categoryTree,
                'categoryIds' => $fieldAssociations->pluckNamed('ticket_category_id')
            ]);
        }

        return $reply;
    }

    /**
     * @param FormAction $form
     * @param AbstractFieldEntity $field
     *
     * @return FormAction
     */
    protected function saveAdditionalData(
        FormAction $form,
        AbstractFieldEntity $field
    )
    {
        $additionalOptions = $this->filter([
            'display_list' => 'bool',
            'expiry_length_type' => 'str',
            'expiry_length_unit' => 'uint'
        ]);

        if ($this->filter('expiry_enabled', 'str') === 'never')
        {
	        $additionalOptions['expiry_length_type'] = 'never';
        }

        $userGroupFields = [
            'usable_user_group',
            'viewable_user_group',
            'viewable_owner_user_group'
        ];
        foreach ($userGroupFields as $userGroupField)
        {
            $value = $this->filter($userGroupField, 'str');
            $column = "{$userGroupField}_ids";

            if ($value === 'all')
            {
                $additionalOptions[$column] = [-1];
            }
            else
            {
                $additionalOptions[$column] = $this->filter(
                    $column,
                    'array-uint'
                );
            }
        }

        $form->setup(function() use ($field, $additionalOptions)
        {
            $field->bulkSet($additionalOptions);
        });

        $categoryIds = $this->filter('ticket_category_ids', 'array-uint');
        $form->complete(function() use ($field, $categoryIds)
        {
            /** @var \NF\Tickets\Repository\CategoryField $repo */
            $repo = $this->repository('NF\Tickets:CategoryField');
            $repo->updateFieldAssociations($field, $categoryIds);
        });

        return $form;
    }
}
