<?php

namespace NF\Tickets\ApprovalQueue;

use XF\ApprovalQueue\AbstractHandler;
use XF\Mvc\Entity\Entity;

/**
 * Class Threadmark
 *
 * @package SV\Threadmarks\ApprovalQueue
 */
class TicketMessage extends AbstractHandler
{
    /**
     * @param Entity|\NF\Tickets\Entity\Ticket $content
     * @param null                             $error
     * @return bool
     */
    protected function canActionContent(Entity $content, &$error = null)
    {
        return $content->canApproveUnapprove($error);
    }

    /**
     * @return array
     */
    public function getEntityWith()
    {
        return ['Ticket', 'Ticket.Category'];
    }

    /**
     * @param Entity|\NF\Tickets\Entity\Message $content
     */
    public function actionApprove(Entity $content)
    {
        /** @var \NF\Tickets\Service\Message\Approver $approver */
        $approver = \XF::service('NF\Tickets:Message\Approver', $content);
        $approver->setNotifyRunTime(1); // may be a lot happening
        $approver->approve();
    }

    /**
     * @param Entity|\NF\Tickets\Entity\Message $content
     */
    public function actionDelete(Entity $content)
    {
        $this->quickUpdate($content, 'message_state', 'deleted');
    }

    public function actionSpamClean(\NF\Tickets\Entity\Message $content)
    {
        if (!$content->User)
        {
            return;
        }

        $this->_spamCleanInternal($content->User);
    }
}