<?php

namespace NF\Tickets\Attachment;

use XF\Attachment\AbstractHandler;
use XF\Entity\Attachment;
use XF\Mvc\Entity\Entity;

use function intval;

/**
 * Class KBArticle
 *
 * @package NF\Tickets\Attachment
 */
class KBArticle extends AbstractHandler
{
    /**
     * @return array
     */
    public function getContainerWith()
    {
        $visitor = \XF::visitor();

        return ['Category', 'Category.Permissions|' . $visitor->permission_combination_id];
    }

    /**
     * @param Attachment $attachment
     * @param Entity     $container
     * @param null       $error
     * @return bool
     * @noinspection PhpParameterByRefIsNotUsedAsReferenceInspection
     */
    public function canView(Attachment $attachment, Entity $container, &$error = null)
    {
        /** @var \NF\Tickets\Entity\KBArticle $container */
        $category = $container->Category;
        if (!$category || !$container->canView())
        {
            return false;
        }

        return $category->canViewAttachments($error);
    }

    /**
     * @param array $context
     * @param null  $error
     * @return bool
     */
    public function canManageAttachments(array $context, &$error = null)
    {
        $em = \XF::em();

        if (!empty($context['kb_article_id']))
        {
            /** @var \NF\Tickets\Entity\KBArticle $kbArticle */
            $kbArticle = $em->find('NF\Tickets:KBArticle', intval($context['kb_article_id']), ['Category']);
            if (!$kbArticle || !$kbArticle->canView() || !$kbArticle->canEdit())
            {
                return false;
            }

            $category = $kbArticle->Category;
        }
        else if (!empty($context['kb_category_id']))
        {
            /** @var \NF\Tickets\Entity\KBCategory $category */
            $category = $em->find('NF\Tickets:KBCategory', intval($context['kb_category_id']));
            if (!$category || !$category->canView())
            {
                return false;
            }
        }
        else
        {
            return false;
        }

        return $category->canUploadAndManageAttachments();
    }

    /**
     * @param Attachment                               $attachment
     * @param Entity|\NF\Tickets\Entity\KBArticle|null $container
     * @throws \XF\PrintableException
     */
    public function onAttachmentDelete(Attachment $attachment, Entity $container = null)
    {
        if (!$container)
        {
            return;
        }

        $container->attach_count--;
        $container->save();
    }

    /**
     * @param array $context
     * @return array
     */
    public function getConstraints(array $context)
    {
        /** @var \XF\Repository\Attachment $attachmentRepo */
        $attachmentRepo = \XF::repository('XF:Attachment');

        return $attachmentRepo->getDefaultAttachmentConstraints();
    }

    /**
     * @param array $context
     * @return int|null
     */
    public function getContainerIdFromContext(array $context)
    {
        return isset($context['kb_article_id']) ? intval($context['kb_article_id']) : null;
    }

    /**
     * @param Entity|\NF\Tickets\Entity\Message $container
     * @param array                             $extraParams
     * @return mixed|string
     */
    public function getContainerLink(Entity $container, array $extraParams = [])
    {
        return \XF::app()->router('public')->buildLink('knowledge-base', $container, $extraParams);
    }

    /**
     * @param Entity|\NF\Tickets\Entity\KBArticle|null $entity
     * @param array                                  $extraContext
     * @return array
     */
    public function getContext(Entity $entity = null, array $extraContext = [])
    {
        if ($entity instanceof \NF\Tickets\Entity\KBArticle)
        {
            $extraContext['kb_article_id'] = $entity->kb_article_id;
        }
        else if ($entity instanceof \NF\Tickets\Entity\KBCategory)
        {
            $extraContext['kb_category_id'] = $entity->kb_category_id;
        }
        else
        {
            throw new \InvalidArgumentException('Entity must be kb article or category');
        }

        return $extraContext;
    }
}