<?php

namespace NF\Tickets\Bookmark;

use XF\Bookmark\AbstractHandler;
use XF\Mvc\Entity\Entity;

class Message extends AbstractHandler
{
    /**
     * @param Entity $content
     * @return \XF\Phrase
     */
    public function getContentTitle(Entity $content)
    {
        /** @var \NF\Tickets\Entity\Message $content */
        $ticket = $content->Ticket;
        if ($content->isFirstMessage())
        {
            return \XF::phrase('nf_tickets_ticket_x', [
                'title' => $ticket->title,
            ]);
        }
        else
        {
            return \XF::phrase('nf_tickets_message_in_ticket_x', [
                'title' => $ticket->title,
            ]);
        }
    }

    /**
     * @param Entity|\NF\Tickets\Entity\Message $content
     * @return string
     */
    public function getContentRoute(Entity $content)
    {
        return 'tickets/messages';
    }

    /**
     * @param Entity $content
     * @return null|\XF\Entity\User
     */
    public function getContentUser(Entity $content)
    {
        /** @var \NF\Tickets\Entity\Message $content */
        $ticket = $content->Ticket;

        if ($ticket && $content->isFirstMessage())
        {
            return $ticket->User;
        }
        else
        {
            return $content->User;
        }
    }

    /**
     * @param Entity|\NF\Tickets\Entity\Message $content
     * @return string
     */
    public function getContentLink(Entity $content)
    {
        if ($content->isFirstMessage())
        {
            return \XF::app()->router('public')->buildLink('canonical:tickets', $content->Ticket);
        }
        else
        {
            return \XF::app()->router('public')->buildLink('canonical:tickets/messages', $content);
        }
    }

    public function getEntityWith()
    {
        $visitor = \XF::visitor();

        return ['Ticket', 'Ticket.Category', 'Ticket.Category.AnonymousUser', 'Ticket.Category.Permissions|' . $visitor->permission_combination_id];
    }
}