<?php

namespace NF\Tickets\ControllerPlugin;

use XF\ControllerPlugin\AbstractPlugin;
use XF\Mvc\ParameterBag;
use XF\Mvc\Reply\AbstractReply;

class Ticket extends AbstractPlugin
{
    public function getMessageLink(\NF\Tickets\Entity\Message $message)
    {
        $ticket = $message->Ticket;
        if (!$ticket)
        {
            throw new \LogicException("Message has no ticket");
        }

        $page = \floor($message->position / $this->options()->messagesPerPage) + 1;

        return $this->buildLink('tickets', $ticket, ['page' => $page]) . '#message-' . $message->message_id;
    }

    /** @var null|\XF\Entity\User */
    protected static $bannedVisitor = null;
    /** @var null|\XF\Entity\User */
    protected static $originalBannedVisitor = null;

    /** @noinspection PhpUnusedParameterInspection */
    public function preDispatchForTicket($isDiscouraged, $action, ParameterBag $params)
    {
        /** @var \NF\Tickets\XF\Entity\User $visitor */
        $visitor = \XF::visitor();

        if (!$visitor->canViewTickets($error))
        {
            throw $this->exception($this->noPermission($error));
        }

        $visitor = \XF::visitor();

        if (self::$bannedVisitor)
        {
            // visitor is a shimmed user entity that is faking not being banned
            self::$bannedVisitor->setTrusted('is_banned', false);
            \XF::setVisitor(self::$bannedVisitor);
        }
        else if (!$visitor->user_id)
        {
            $visitor->setOption('nf_force_captcha', \XF::options()->nftRequireCaptcha);
        }
        else if (!self::$bannedVisitor &&
                 !empty(\XF::options()->nfTicketsAllowBannedTicketUse) &&
                 $visitor->is_banned &&
                 !$isDiscouraged)
        {
            $guestBannedUser = $this->getUserRepo()->getGuestUser($visitor->username);
            $visitorArr = $visitor->toArray();
            unset($visitorArr['user_state']);
            unset($visitorArr['is_banned']);
            unset($visitorArr['permission_combination_id']);
            foreach ($visitorArr as $key => $value)
            {
                $guestBannedUser->setTrusted($key, $value);
            }
            foreach ($visitor->structure()->relations as $key => $config)
            {
                $guestBannedUser->hydrateRelation($key, $visitor->get($key));
            }

            if (!empty(\XF::options()->nftRequireCaptcha))
            {
                $guestBannedUser->setOption('nf_force_captcha', true);
            }

            $guestBannedUser->setOption('nf_is_banned', true);

            self::$originalBannedVisitor = $visitor;
            self::$bannedVisitor = $guestBannedUser;
            // ban checks are done in preDispatchType, which is before preDispatchController
            \XF::setVisitor($guestBannedUser);
        }
    }

    /** @noinspection PhpUnusedParameterInspection */
    public function postDispatchForTicket($action, ParameterBag $params, AbstractReply &$reply)
    {
        // ensure user styling is correct
        if (self::$bannedVisitor)
        {
            self::$bannedVisitor->setTrusted('is_banned', true);
            \XF::setVisitor(self::$bannedVisitor);
        }
    }

    protected function getUserRepo(): \NF\Tickets\XF\Repository\User
    {
        /** @noinspection PhpIncompatibleReturnTypeInspection */
        return $this->repository('XF:User');
    }
}