<?php

namespace NF\Tickets\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

/**
 * COLUMNS
 * @property int|null $category_read_id
 * @property int $user_id
 * @property int $ticket_category_id
 * @property int $category_read_date
 *
 * RELATIONS
 * @property \XF\Entity\User $User
 * @property \NF\Tickets\Entity\Category $Category
 */
class CategoryRead extends Entity
{
    /**
     * @param Structure $structure
     * @return Structure
     */
    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_nf_tickets_category_read';
        $structure->shortName = 'NF\Tickets:CategoryRead';
        $structure->primaryKey = 'category_read_id';
        $structure->columns = [
            'category_read_id'   => ['type' => self::UINT, 'autoIncrement' => true, 'nullable' => true],
            'user_id'            => ['type' => self::UINT, 'required' => true],
            'ticket_category_id' => ['type' => self::UINT, 'required' => true],
            'category_read_date' => ['type' => self::UINT, 'required' => true],
        ];
        $structure->getters = [];
        $structure->relations = [
            'User'     => [
                'entity'     => 'XF:User',
                'type'       => self::TO_ONE,
                'conditions' => 'user_id',
                'primary'    => true,
            ],
            'Category' => [
                'entity'     => 'NF\Tickets:Category',
                'type'       => self::TO_ONE,
                'conditions' => 'ticket_category_id',
                'primary'    => true,
            ],
        ];

        return $structure;
    }
}