<?php

namespace NF\Tickets\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

/**
 * COLUMNS
 * @property int $user_id
 * @property int $ticket_category_id
 * @property string $notify_on
 * @property bool $send_alert
 * @property bool $send_email
 * @property bool $include_children
 *
 * RELATIONS
 * @property \NF\Tickets\Entity\Category $Category
 * @property \XF\Entity\User $User
 */
class CategoryWatch extends Entity
{
    /**
     * @param Structure $structure
     * @return Structure
     */
    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_nf_tickets_category_watch';
        $structure->shortName = 'NF\Tickets:CategoryWatch';
        $structure->primaryKey = ['user_id', 'ticket_category_id'];
        $structure->columns = [
            'user_id'            => ['type' => self::UINT, 'required' => true],
            'ticket_category_id' => ['type' => self::UINT, 'required' => true],
            'notify_on'          => [
                'type'          => self::STR, 'default' => '',
                'allowedValues' => ['', 'ticket', 'message'],
            ],
            'send_alert'         => ['type' => self::BOOL, 'default' => false],
            'send_email'         => ['type' => self::BOOL, 'default' => false],
            'include_children'   => ['type' => self::BOOL, 'default' => false],
        ];
        $structure->getters = [];
        $structure->relations = [
            'Category' => [
                'entity'     => 'NF\Tickets:Category',
                'type'       => self::TO_ONE,
                'conditions' => 'ticket_category_id',
                'primary'    => true,
            ],
            'User'     => [
                'entity'     => 'XF:User',
                'type'       => self::TO_ONE,
                'conditions' => 'user_id',
                'primary'    => true,
            ],
        ];

        return $structure;
    }
}