<?php

namespace NF\Tickets\Entity;

use XF\Entity\User as UserEntity;
use NF\Tickets\XF\Entity\User as ExtendedUserEntity;
use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

/**
 * COLUMNS
 * @property int $escalation_id
 * @property string $escalation_title
 * @property string $escalation_description
 * @property bool $active
 * @property int $execution_order
 * @property array $user_criteria
 * @property array $ticket_criteria
 * @property string $message_body
 * @property string $message_from_username
 * @property string $alert_body
 * @property string $alert_from_username
 * @property array $actions
 *
 * RELATIONS
 * @property \XF\Entity\User $MessageStarter
 * @property \XF\Entity\User $AlertSender
 */
class Escalation extends Entity
{
    protected function _preSave()
    {
        // ensure int-array field are int's to simplify matching in critera
        $criteriaData = $this->ticket_criteria;
        foreach(['ticket_status','ticket_priority','ticket_category'] as $criteria)
        {
            if (isset($criteriaData[$criteria]))
            {
                $criteriaData[$criteria]['data'] = \array_map('intval', $criteriaData[$criteria]['data']);
            }
        }
        $this->ticket_criteria = $criteriaData;
    }

    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_nf_tickets_escalation';
        $structure->shortName = 'NF\Tickets:Escalation';
        $structure->primaryKey = 'escalation_id';
        $structure->columns = [
            'escalation_id'          => ['type' => self::UINT, 'autoIncrement' => true],
            'escalation_title'       => [
                'type'     => self::STR,
                'required' => 'please_enter_valid_title', 'maxLength' => 100,
            ],
            'escalation_description' => ['type' => self::STR, 'default' => ''],
            'active'                 => ['type' => self::BOOL, 'default' => 1],
            'execution_order'        => ['type' => self::UINT, 'default' => 1],
            'user_criteria'          => ['type' => self::JSON_ARRAY, 'default' => []],
            'ticket_criteria'        => ['type' => self::JSON_ARRAY, 'default' => []],
            'message_body'           => ['type' => self::STR, 'default' => ''],
            'message_from_username'  => ['type' => self::STR, 'default' => ''],
            'alert_body'             => ['type' => self::STR, 'default' => ''],
            'alert_from_username'    => ['type' => self::STR, 'default' => ''],
            'actions'                => ['type' => self::JSON_ARRAY, 'default' => []],
        ];
        $structure->getters = [];
        $structure->relations = [
            'MessageStarter' => [
                'entity' => 'XF:User',
                'type' => self::TO_ONE,
                'conditions' => [
                    ['username', '=', '$message_from_username']
                ]
            ],
            'AlertSender' => [
                'entity' => 'XF:User',
                'type' => self::TO_ONE,
                'conditions' => [
                    ['username', '=', '$message_from_username']
                ]
            ]
        ];

        return $structure;
    }
}