<?php

namespace NF\Tickets\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

/**
 * COLUMNS
 * @property int|null $priority_id
 * @property int $display_order
 * @property bool $active
 *
 * GETTERS
 * @property \XF\Phrase $title
 *
 * RELATIONS
 * @property \XF\Entity\Phrase $MasterTitle
 */
class Priority extends Entity
{
	public function canDelete(): bool
	{
		return false; // soon
		return !$this->isSystemPriority();
	}

	public function isSystemPriority(): bool
	{
		return ((int)\XF::options()->nfTicketsDefaultPriority === $this->priority_id);
	}

    /**
     * @return \XF\Phrase
     */
    public function getTitle()
    {
        return \XF::phrase($this->getPhraseName());
    }

    public function getPhraseName()
    {
        return 'nf_tickets_priority.' . $this->priority_id;
    }

    public function getMasterPhrase()
    {
        $phrase = $this->MasterTitle;

        if (!$phrase)
        {
            /** @var \XF\Entity\Phrase $phrase */
            $phrase = $this->_em->create('XF:Phrase');
            $phrase->title = $this->_getDeferredValue(function () { return $this->getPhraseName(); }, 'save');
            $phrase->language_id = 0;
            $phrase->addon_id = '';
        }

        return $phrase;
    }

    protected function _postDelete()
    {
        if ($this->MasterTitle)
        {
            $this->MasterTitle->delete();
        }

        $defaultPriority = \XF::options()->nfTicketsDefaultPriority;
        if ($defaultPriority && $defaultPriority === $this->priority_id)
        {
            /** @var \XF\Entity\Option $option */
            $option = \XF::finder('XF:Option')->whereId('nfTicketsDefaultPriority')->fetchOne();
            $option->option_value = null;
            $option->save();
        }
    }

    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_nf_tickets_priority';
        $structure->shortName = 'NF\Tickets:Priority';
        $structure->primaryKey = 'priority_id';
        $structure->columns = [
            'priority_id'   => ['type' => self::UINT, 'autoIncrement' => true, 'nullable' => true],
            'display_order' => ['type' => self::UINT, 'default' => 0],
            'active'        => ['type' => self::BOOL, 'default' => true],
        ];
        $structure->behaviors = [];
        $structure->getters = [
            'title' => true,
        ];
        $structure->relations = [
            'MasterTitle' => [
                'entity'     => 'XF:Phrase',
                'type'       => self::TO_ONE,
                'conditions' => [
                    ['language_id', '=', 0],
                    ['title', '=', 'nf_tickets_priority.', '$priority_id'],
                ],
            ],
        ];

        return $structure;
    }

    protected function getPriorityRepo(): \NF\Tickets\Repository\Priority
    {
        /** @noinspection PhpIncompatibleReturnTypeInspection */
        return $this->repository('NF\Tickets:Priority');
    }
}