<?php

namespace NF\Tickets\Entity;

use NF\Tickets\XF\Entity\User;
use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

/**
 * COLUMNS
 * @property int $user_id
 * @property string $notify_on
 * @property bool $send_alert
 * @property bool $send_email
 *
 * RELATIONS
 * @property \XF\Entity\User $User
 */
class QueueWatch extends Entity
{
	public static function getStructure(Structure $structure): Structure
	{
		$structure->table = 'xf_nf_tickets_queue_watch';
		$structure->shortName = 'NF\Tickets:QueueWatch';
		$structure->primaryKey = 'user_id';
		$structure->columns = [
			'user_id'            => ['type' => self::UINT, 'required' => true],
			'notify_on'          => [
				'type'          => self::STR, 'default' => '',
				'allowedValues' => ['', 'ticket', 'message'],
			],
			'send_alert'         => ['type' => self::BOOL, 'default' => false],
			'send_email'         => ['type' => self::BOOL, 'default' => false],
		];
		$structure->getters = [];
		$structure->relations = [
			'User' => [
				'entity'     => 'XF:User',
				'type'       => self::TO_ONE,
				'conditions' => 'user_id',
				'primary'    => true,
			],
		];

		return $structure;
	}
}