<?php

namespace NF\Tickets\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

/**
 * COLUMNS
 * @property int|null $response_id
 * @property int $user_id
 * @property string $response_title
 * @property int $response_category_id
 * @property string $message
 * @property bool $active
 * @property int $display_order
 *
 * RELATIONS
 * @property \NF\Tickets\Entity\ResponseCategory $Category
 */
class Response extends Entity
{
    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_nf_tickets_response';
        $structure->shortName = 'NF\Tickets:Response';
        $structure->primaryKey = 'response_id';
        $structure->contentType = 'ticket_response';
        $structure->columns = [
            'response_id'          => ['type' => self::UINT, 'autoIncrement' => true, 'nullable' => true],
            'user_id'              => ['type' => self::UINT, 'required' => true],
            'response_title'       => ['type' => self::STR, 'maxLength' => 50, 'required' => 'please_enter_valid_title'],
            'response_category_id' => ['type' => self::UINT, 'default' => 0],
            'message'              => ['type' => self::STR, 'required' => 'please_enter_valid_message'],
            'active'               => ['type' => self::BOOL, 'default' => true],
            'display_order'        => ['type' => self::UINT, 'default' => 10],
        ];

        $structure->relations = [
            'Category' => [
                'entity'     => 'NF\Tickets:ResponseCategory',
                'type'       => self::TO_ONE,
                'conditions' => 'response_category_id',
                'primary'    => true,
            ],
        ];

        return $structure;
    }
}