<?php

namespace NF\Tickets\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

/**
 * COLUMNS
 * @property int|null $response_category_id
 * @property string $category_name
 * @property int $display_order
 *
 * RELATIONS
 * @property \XF\Mvc\Entity\AbstractCollection|\NF\Tickets\Entity\Response[] $Responses
 */
class ResponseCategory extends Entity
{
    protected function _postDelete()
    {
        foreach ($this->Responses as $response)
        {
            $response->fastUpdate('response_category_id', 0);
        }
    }

    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_nf_tickets_response_category';
        $structure->shortName = 'NF\Tickets:ResponseCategory';
        $structure->primaryKey = 'response_category_id';
        $structure->contentType = 'ticket_response_category';
        $structure->columns = [
            'response_category_id' => ['type' => self::UINT, 'autoIncrement' => true, 'nullable' => true],
            'category_name'        => ['type' => self::STR, 'maxLength' => 50, 'required' => true],
            'display_order'        => ['type' => self::UINT, 'default' => 10],
        ];

        $structure->relations = [
            'Responses' => [
                'entity'     => 'NF\Tickets:Response',
                'type'       => self::TO_MANY,
                'conditions' => 'response_category_id',
                'primary'    => true,
            ],
        ];

        return $structure;
    }
}