<?php

namespace NF\Tickets\Entity;

use NF\Tickets\XF\Entity\User;
use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

/**
 * COLUMNS
 * @property int|null $ticket_email_id
 * @property int $log_date
 * @property int $email_date
 * @property string $message_type
 * @property string $action_taken
 * @property int|null $user_id
 * @property string|null $sender
 * @property string $raw_message
 * @property string|null $diagnostic_info
 *
 * RELATIONS
 * @property \XF\Entity\User $User
 */
class TicketEmailLog extends Entity
{
    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_nf_tickets_ticket_email_log';
        $structure->shortName = 'NF\Tickets:TicketEmailLog';
        $structure->primaryKey = 'ticket_email_id';

        $structure->columns = [
            'ticket_email_id' => [
                'type' => self::UINT,
                'autoIncrement' => true,
                'nullable' => true
            ],
            'log_date' => ['type' => self::UINT, 'default' => \XF::$time],
            'email_date' => ['type' => self::UINT, 'default' => \XF::$time],
            'message_type' => ['type' => self::STR, 'maxLength' => 25, 'default' => 'unknown'],
            'action_taken' => ['type' => self::STR, 'maxLength' => 25, 'default' => ''],
            'user_id' => ['type' => self::UINT, 'nullable' => true],
            'sender' => ['type' => self::STR, 'maxLength' => 255, 'nullable' => true],
            'raw_message' => ['type' => self::BINARY, 'default' => ''],
            'diagnostic_info' => ['type' => self::STR, 'nullable' => true]
        ];
        $structure->relations = [
            'User' => [
                'entity' => 'XF:User',
                'type' => self::TO_ONE,
                'conditions' => 'user_id',
                'primary' => true,
            ],
        ];

        return $structure;
    }
}