<?php

namespace NF\Tickets\Entity;

use XF\Entity\AbstractField;
use XF\Mvc\Entity\Structure;

/**
 * COLUMNS
 * @property string $field_id
 * @property int $display_order
 * @property string $field_type
 * @property array $field_choices
 * @property string $match_type
 * @property array $match_params
 * @property int $max_length
 * @property bool $required
 * @property string $display_template
 * @property string $display_group
 * @property string $user_editable
 * @property bool $moderator_editable
 * @property string $wrapper_template
 * @property bool $display_list
 * @property array $usable_user_group_ids
 * @property array $viewable_user_group_ids
 * @property array $viewable_owner_user_group_ids
 * @property string $expiry_length_type
 * @property int $expiry_length_unit
 *
 * GETTERS
 * @property \XF\Phrase $title
 * @property \XF\Phrase $description
 *
 * RELATIONS
 * @property \XF\Entity\Phrase $MasterTitle
 * @property \XF\Entity\Phrase $MasterDescription
 * @property \XF\Mvc\Entity\AbstractCollection|\NF\Tickets\Entity\CategoryField[] $CategoryFields
 */
class TicketField extends AbstractField
{
	/**
	 * @return string
	 */
	protected function getClassIdentifier()
	{
		return 'NF\Tickets:TicketField';
	}

	/**
	 * @return string
	 */
	protected static function getPhrasePrefix()
	{
		return 'nf_tickets_ticket_field';
	}

	protected function _postDelete()
	{
		/** @var \NF\Tickets\Repository\CategoryField $repo */
		$repo = $this->repository('NF\Tickets:CategoryField');
		$repo->removeFieldAssociations($this);

		$this->db()->delete('xf_nf_tickets_ticket_field_value', 'field_id = ?', $this->field_id);

		parent::_postDelete();
	}

	/**
	 * @param Structure $structure
	 *
	 * @return Structure
	 */
	public static function getStructure(Structure $structure)
	{
		self::setupDefaultStructure(
			$structure,
			'xf_nf_tickets_ticket_field',
			'NF\Tickets:TicketField',
			[
				'groups' => ['sidebar_ticket_info', 'sidebar_user_info', 'above_message', 'below_message', 'new_tab'],
				'has_user_editable' => true,
				'has_user_editable_once' => true,
				'has_moderator_editable' => true,
				'has_wrapper_template' => true
			]
		);

		$structure->columns['display_list'] = [
			'type' => self::BOOL,
			'default' => true,
		];
		$structure->columns['usable_user_group_ids'] = [
			'type' => self::LIST_COMMA,
			'default' => [-1],
		];
		$structure->columns['viewable_user_group_ids'] = [
			'type' => self::LIST_COMMA,
			'default' => [-1],
		];
		$structure->columns['viewable_owner_user_group_ids'] = [
			'type' => self::LIST_COMMA,
			'default' => [-1],
		];
		$structure->columns['expiry_length_type'] = [
			'type' => self::STR, 'default' => 'never',
			'allowedValues' => ['never', 'days', 'weeks', 'months', 'years'],
		];
		$structure->columns['expiry_length_unit'] = [
			'type' => self::UINT,
			'max' => 65535,
			'default' => 0,
		];

		$structure->relations['CategoryFields'] = [
			'entity' => 'NF\Tickets:CategoryField',
			'type' => self::TO_MANY,
			'conditions' => 'field_id',
		];

		return $structure;
	}
}