<?php

namespace NF\Tickets\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

/**
 * COLUMNS
 * @property int $ticket_id
 * @property string $email
 * @property string $username
 * @property string $password
 *
 * RELATIONS
 * @property \NF\Tickets\Entity\Ticket $Ticket
 */
class TicketGuestParticipant extends Entity
{
    public static function getStructure(Structure $structure)
    {
        $structure->shortName = 'NF\Tickets:TicketGuestParticipant';

        $structure->table = 'xf_nf_tickets_ticket_guest_participant';
        $structure->primaryKey = ['ticket_id', 'email'];

        $structure->columns = [
            'ticket_id' => [
                'type'      => self::UINT,
                'required'  => true,
                'writeOnce' => true
            ],
            'email'     => [
                'type'      => self::STR,
                'maxLength' => 120,
                'required'  => 'please_enter_valid_email',
                'match'     => 'email',
                'writeOnce' => true
            ],
            'username'  => [
                'type'      => self::STR,
                'maxLength' => 50,
                'required'  => true
            ],
            'password'  => [
                'type'      => self::STR,
                'required'  => true,
                'writeOnce' => true
            ]
        ];
        $structure->relations = [
            'Ticket' => [
                'entity'     => 'NF\Tickets:Ticket',
                'type'       => self::TO_ONE,
                'conditions' => 'ticket_id',
                'primary'    => true
            ]
        ];

        return $structure;
    }

    protected function getTicketRepo(): \NF\Tickets\Repository\Ticket
    {
        /** @noinspection PhpIncompatibleReturnTypeInspection */
        return $this->repository('NF\Tickets:Ticket');
    }
}