<?php

namespace NF\Tickets\Entity;

use XF\Entity\AbstractPrefix;
use XF\Mvc\Entity\Structure;

/**
 * COLUMNS
 * @property int|null $prefix_id
 * @property int $prefix_group_id
 * @property int $display_order
 * @property int $materialized_order
 * @property string $css_class
 * @property array $allowed_user_group_ids
 *
 * GETTERS
 * @property string|\XF\Phrase $title
 * @property bool $has_usage_help
 * @property array $category_ids
 *
 * RELATIONS
 * @property \XF\Entity\Phrase $MasterTitle
 * @property \NF\Tickets\Entity\TicketPrefixGroup $PrefixGroup
 * @property \XF\Mvc\Entity\AbstractCollection|\NF\Tickets\Entity\CategoryPrefix[] $CategoryPrefixes
 */
class TicketPrefix extends AbstractPrefix
{
    protected function getClassIdentifier()
    {
        return 'NF\Tickets:TicketPrefix';
    }

    protected static function getContentType()
    {
        return 'nf_tickets_ticket';
    }

    /**
     * @return array
     */
    public function getCategoryIds()
    {
        if (!$this->prefix_id)
        {
            return [];
        }

        return $this->db()->fetchAllColumn("
			SELECT ticket_category_id
			FROM xf_nf_tickets_category_prefix
			WHERE prefix_id = ?
		", $this->prefix_id);
    }

    protected function _postDelete()
    {
        parent::_postDelete();

        /** @var \NF\Tickets\Repository\CategoryPrefix $repo */
        $repo = $this->repository('NF\Tickets:CategoryPrefix');

        $repo->removePrefixAssociations($this);
    }

    public static function getStructure(Structure $structure)
    {
        self::setupDefaultStructure($structure, 'xf_nf_tickets_ticket_prefix', 'NF\Tickets:TicketPrefix');

        $structure->getters['category_ids'] = true;

        $structure->relations['CategoryPrefixes'] = [
            'entity' => 'NF\Tickets:CategoryPrefix',
            'type' => self::TO_MANY,
            'conditions' => 'prefix_id'
        ];

        return $structure;
    }
}