<?php

namespace NF\Tickets\Import\Data;

use XF\Import\Data\AbstractEmulatedData;

use function is_string;

class Ticket extends AbstractEmulatedData
{
    protected $watchers = [];

    public function getImportType()
    {
        return 'ticket';
    }

    protected function getEntityShortName()
    {
        return 'NF\Tickets:Ticket';
    }

    public function setCustomFields(array $customFields)
    {
        foreach ($customFields AS &$fieldValue)
        {
            if (is_string($fieldValue))
            {
                $fieldValue = $this->convertToUtf8($fieldValue);
            }
        }

        $this->custom_fields = $customFields;
    }

    public function addTicketWatcher($userId, $emailSubscribe)
    {
        $this->watchers[$userId] = $emailSubscribe;
    }

    public function setTicketPassword($password)
    {
        $this->password = password_hash($password, PASSWORD_DEFAULT);
    }

    protected function preSave($oldId)
    {
        $this->forceNotEmpty('username', $oldId);
        $this->forceNotEmpty('title', $oldId);

        if (empty($this->user_name))
        {
            $this->user_name = $this->username;
        }

        if (!$this->password)
        {
            /** @var \NF\Tickets\Repository\Ticket $repo */
            $repo = $this->repository('NF\Tickets:Ticket');

            $this->setTicketPassword($repo->generatePassword());
        }
    }

    protected function postSave($oldId, $newId)
    {
        if ($this->watchers)
        {
            /** @var \NF\Tickets\Import\DataHelper\Ticket $ticketHelper */
            $ticketHelper = $this->dataManager->helper('NF\Tickets:Ticket');
            $ticketHelper->importTicketWatchBulk($newId, $this->watchers);
        }
    }
}