<?php

namespace NF\Tickets\Import\Data;

use XF\Import\Data\AbstractEmulatedData;

class TicketPrefix extends AbstractEmulatedData
{
    protected $title = '';

    protected $categoryIds = [];

    public function getImportType()
    {
        return 'ticket_prefix';
    }

    protected function getEntityShortName()
    {
        return 'NF\Tickets:TicketPrefix';
    }

    public function setTitle($title)
    {
        $this->title = $title;
    }

    public function setCategories(array $categoryIds)
    {
        $this->categoryIds = $categoryIds;
    }

    protected function postSave($oldId, $newId)
    {
        /** @var \NF\Tickets\Entity\TicketPrefix $prefix */
        $prefix = $this->em()->find('NF\Tickets:TicketPrefix', $newId);
        if ($prefix)
        {
            $this->insertMasterPhrase($prefix->getPhraseName(), $this->title);

            $this->em()->detachEntity($prefix);
        }

        if ($this->categoryIds)
        {
            $insert = [];
            foreach ($this->categoryIds AS $categoryId)
            {
                $insert[] = [
                    'ticket_category_id' => $categoryId,
                    'prefix_id' => $newId
                ];
            }

            $this->db()->insertBulk('xf_nf_tickets_category_prefix', $insert, false, false, 'IGNORE');
        }

        /** @var \NF\Tickets\Repository\TicketPrefix $repo */
        $repo = $this->repository('NF\Tickets:TicketPrefix');

        \XF::runOnce('nfRebuildTicketPrefixImport', function() use ($repo)
        {
            $repo->rebuildPrefixMaterializedOrder();
            $repo->rebuildPrefixCache();
        });
    }
}