<?php

namespace NF\Tickets\InlineMod\Thread;

use NF\Tickets\Entity\Ticket;
use NF\Tickets\Repository\Category;
use NF\Tickets\XF\Entity\Thread;
use XF\Http\Request;
use XF\InlineMod\AbstractAction;
use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\Entity\Entity;

use function count, intval;

class ToTicket extends AbstractAction
{
    /** @var \NF\Tickets\Entity\Category|null */
	protected $targetCategory;
	/** @var int|null */
	protected $targetCategoryId;

	public function getTitle()
	{
		return \XF::phrase('nf_tickets_convert_to_tickets...');
	}

	protected function canApplyToEntity(Entity $entity, array $options, &$error = null)
	{
		/** @var Thread $entity */
		return $entity->canConvertToTicket();
	}

	protected function applyToEntity(Entity $entity, array $options)
	{
		/** @var Ticket $entity */

		/** @var \NF\Tickets\Entity\Category $category */
		$category = $this->getTargetCategory($options['target_category_id']);
		if (!$category)
		{
			throw new \InvalidArgumentException("No target specified");
		}

		/** @var \NF\Tickets\Service\Ticket\ThreadConverter $converter */
		$converter = $this->app()->service('NF\Tickets:Ticket\ThreadConverter', $entity, $category);

		if ($options['alert'])
		{
			$converter->setSendAlert(true, $options['alert_reason']);
		}

		$converter->save();
		$converter->sendNotifications();

		$converter->deleteThread();

		$this->returnUrl = $this->app()->router('public')->buildLink('tickets/categories', $category);
	}

	public function getBaseOptions()
	{
		return [
			'target_category_id' => 0,
			'alert' => false,
			'alert_reason' => ''
		];
	}

	public function renderForm(AbstractCollection $entities, \XF\Mvc\Controller $controller)
	{
		/** @var Category $categoryRepo */
		$categoryRepo = $this->app()->repository('NF\Tickets:Category');
		$categoryTree = $categoryRepo->createCategoryTree($categoryRepo->getViewableCategories());

		$viewParams = [
			'threads' => $entities,
			'total' => count($entities),
			'categoryTree' => $categoryTree,
			'first' => $entities->first(),
		];

		return $controller->view('NF\Tickets:Public:InlineMod\Thread\ToTicket', 'nf_tickets_inline_mod_thread_to_ticket', $viewParams);
	}

	public function getFormOptions(AbstractCollection $entities, Request $request)
	{
		$options = [
			'target_category_id' => $request->filter('target_category_id', 'uint'),
			'apply_ticket_prefix' => $request->filter('apply_ticket_prefix', 'bool'),
			'prefix_id' => $request->filter('prefix_id', 'uint'),
			'alert' => $request->filter('starter_alert', 'bool'),
			'alert_reason' => $request->filter('starter_alert_reason', 'str')
		];
		if (!$options['apply_ticket_prefix'])
		{
			$options['prefix_id'] = null;
		}

		return $options;
	}

	/**
	 * @param $categoryId
	 * @return \NF\Tickets\Entity\Category|null
	 */
	protected function getTargetCategory($categoryId)
	{
		$categoryId = intval($categoryId);

		if ($this->targetCategoryId && $this->targetCategoryId === $categoryId)
		{
			return $this->targetCategory;
		}
		if (!$categoryId)
		{
			return null;
		}

        /** @var \NF\Tickets\Entity\Category $category */
		$category = $this->app()->em()->find('NF\Tickets:Category', $categoryId);
		if (!$category)
		{
			throw new \InvalidArgumentException("Invalid target category ($categoryId)");
		}

		$this->targetCategoryId = $categoryId;
		$this->targetCategory = $category;

		return $this->targetCategory;
	}
}