<?php

namespace NF\Tickets\InlineMod;

use XF\InlineMod\AbstractHandler;
use XF\Mvc\Entity\Entity;

class Ticket extends AbstractHandler
{
    public function getPossibleActions()
    {
        $actions = [];

        $actions['delete'] = $this->getActionHandler('NF\Tickets:Ticket\Delete');

        $actions['undelete'] = $this->getSimpleActionHandler(
            \XF::phrase('nf_tickets_undelete_tickets'),
            'canUndelete',
            function(Entity $entity)
            {
                /** @var \NF\Tickets\Entity\Ticket $entity */
                if ($entity->discussion_state === 'deleted')
                {
                    $entity->discussion_state = 'visible';
                    $entity->save();
                }
            }
        );

        $actions['approve'] = $this->getSimpleActionHandler(
            \XF::phrase('nf_tickets_approve_tickets'),
            'canApproveUnapprove',
            function(Entity $entity)
            {
                /** @var \NF\Tickets\Entity\Ticket $entity */
                if ($entity->discussion_type !== 'redirect' && $entity->discussion_state === 'moderated')
                {
                    /** @var \NF\Tickets\Service\Ticket\Approver $approver */
                    $approver = \XF::service('NF\Tickets:Ticket\Approver', $entity);
                    $approver->setNotifyRunTime(1); // may be a lot happening
                    $approver->approve();
                }
            }
        );

        $actions['unapprove'] = $this->getSimpleActionHandler(
            \XF::phrase('nf_tickets_unapprove_tickets'),
            'canApproveUnapprove',
            function(Entity $entity)
            {
                /** @var \NF\Tickets\Entity\Ticket $entity */
                if ($entity->discussion_state === 'visible')
                {
                    $entity->discussion_state = 'moderated';
                    $entity->save();
                }
            }
        );

        $actions['reassign'] = $this->getActionHandler('NF\Tickets:Ticket\Reassign');
        $actions['move'] = $this->getActionHandler('NF\Tickets:Ticket\Move');
        $actions['merge'] = $this->getActionHandler('NF\Tickets:Ticket\Merge');
        $actions['apply_status'] = $this->getActionHandler('NF\Tickets:Ticket\ApplyStatus');
        $actions['apply_priority'] = $this->getActionHandler('NF\Tickets:Ticket\ApplyPriority');
        $actions['apply_prefix'] = $this->getActionHandler('NF\Tickets:Ticket\ApplyPrefix');

        return $actions;
    }

    public function getEntityWith()
    {
        $visitor = \XF::visitor();

        return ['Category', 'Category.Permissions|' . $visitor->permission_combination_id];
    }
}