<?php

namespace NF\Tickets\InlineMod\Ticket;

use NF\Tickets\Entity\TicketPrefix;
use XF\Http\Request;
use XF\InlineMod\AbstractAction;
use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\Entity\Entity;

use function count;

class ApplyPrefix extends AbstractAction
{
    public function getTitle()
    {
        return \XF::phrase('apply_prefix...');
    }

    protected function canApplyToEntity(Entity $entity, array $options, &$error = null)
    {
        /** @var \NF\Tickets\Entity\Ticket $entity */
        return $entity->canEdit($error);
    }

    protected function applyToEntity(Entity $entity, array $options)
    {
        /** @var \NF\Tickets\Entity\Ticket $entity */
        if (!$entity->Category->isPrefixValid($options['prefix_id']))
        {
            return;
        }

        /** @var \NF\Tickets\Service\Ticket\Editor $editor */
        $editor = $this->app()->service('NF\Tickets:Ticket\Editor', $entity);
        $editor->setPerformValidations(false);
        $editor->setPrefix($options['prefix_id']);
        if ($editor->validate($errors))
        {
            $editor->save();
        }
    }

    public function getBaseOptions()
    {
        return [
            'prefix_id' => null
        ];
    }

    public function renderForm(AbstractCollection $entities, \XF\Mvc\Controller $controller)
    {
        /** @var AbstractCollection|\NF\Tickets\Entity\Category $categories */
        $categories = $entities->pluckNamed('Category', 'ticket_category_id');
        $prefixIds = [];

        foreach ($categories AS $category)
        {
            $prefixIds = \array_merge($prefixIds, \array_keys($category->prefix_cache));
        }

        $prefixes = $this->app()->finder('NF\Tickets:TicketPrefix')
            ->where('prefix_id', \array_unique($prefixIds))
            ->order('materialized_order')
            ->fetch();

        $prefixes = $prefixes->filter(function(TicketPrefix $prefix)
        {
            return $prefix->isUsableByUser();
        });

        if (!$prefixes->count())
        {
            return $controller->error(\XF::phrase('nf_tickets_there_no_ticket_prefixes_available_for_selected_categories'));
        }

        $selectedPrefix = 0;
        $prefixCounts = [0 => 0];
        foreach ($entities AS $ticket)
        {
            $ticketPrefixId = $ticket->prefix_id;

            if (!isset($prefixCounts[$ticketPrefixId]))
            {
                $prefixCounts[$ticketPrefixId] = 1;
            }
            else
            {
                $prefixCounts[$ticketPrefixId]++;
            }

            if ($prefixCounts[$ticketPrefixId] > $prefixCounts[$selectedPrefix])
            {
                $selectedPrefix = $ticketPrefixId;
            }
        }

        $viewParams = [
            'tickets' => $entities,
            'prefixes' => $prefixes->groupBy('prefix_group_id'),
            'categoryCount' => count($categories->keys()),
            'selectedPrefix' => $selectedPrefix,
            'total' => count($entities)
        ];
        return $controller->view('NF\Tickets:Public:InlineMod\Ticket\ApplyPrefix', 'nf_tickets_inline_mod_ticket_apply_prefix', $viewParams);
    }

    public function getFormOptions(AbstractCollection $entities, Request $request)
    {
        return [
            'prefix_id' => $request->filter('prefix_id', 'uint')
        ];
    }
}