<?php

namespace NF\Tickets\InlineMod\Ticket;

use NF\Tickets\Entity\Priority;
use NF\Tickets\Entity\Ticket;
use XF\Http\Request;
use XF\InlineMod\AbstractAction;
use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\Entity\Entity;

class ApplyPriority extends AbstractAction
{
    /** @var int|null */
    protected $targetPriorityId;
    /** @var Priority|null */
    protected $targetPriority;

    public function getTitle()
    {
        return \XF::phrase('nf_tickets_apply_priority');
    }

    protected function canApplyToEntity(Entity $entity, array $options, &$error = null)
    {
        /** @var Ticket $entity */
        return $entity->canUpdatePriority();
    }

    protected function applyToEntity(Entity $entity, array $options)
    {
        /** @var Ticket $entity */

        $priority = $this->getTargetPriority($options['target_priority_id']);
        if (!$priority)
        {
            throw new \InvalidArgumentException("No priority specified");
        }

        $entity->priority = $priority->priority_id;
        $entity->save();
    }

    public function getBaseOptions()
    {
        return [
            'target_priority_id' => 0,
        ];
    }

    public function renderForm(AbstractCollection $entities, \XF\Mvc\Controller $controller)
    {
        /** @var \NF\Tickets\Repository\Priority $priorityRepo */
        $priorityRepo = $this->app()->repository('NF\Tickets:Priority');
        $priorities = $priorityRepo->findPrioritiesForList();

        $viewParams = [
            'tickets'    => $entities,
            'total'      => $entities->count(),
            'priorities' => $priorities,
            'first'      => $entities->first(),
        ];

        return $controller->view('NF\Tickets:Public:InlineMod\Ticket\ApplyPriority', 'nf_tickets_inline_mod_ticket_apply_priority', $viewParams);
    }

    public function getFormOptions(AbstractCollection $entities, Request $request)
    {
        $options = [
            'target_priority_id' => $request->filter('target_priority_id', 'uint'),
        ];

        return $options;
    }

    /**
     * @param int $priorityId
     * @return null|Priority
     */
    protected function getTargetPriority(int $priorityId)
    {
        if ($this->targetPriorityId && $this->targetPriorityId === $priorityId)
        {
            return $this->targetPriority;
        }
        if (!$priorityId)
        {
            return null;
        }

        /** @var Priority $category */
        $priority = $this->app()->em()->find('NF\Tickets:Priority', $priorityId);
        if (!$priority)
        {
            throw new \InvalidArgumentException("Invalid target priority ($priorityId)");
        }

        $this->targetPriorityId = $priorityId;
        $this->targetPriority = $priority;

        return $this->targetPriority;
    }
}