<?php

namespace NF\Tickets\InlineMod\Ticket;

use NF\Tickets\Entity\Ticket;
use NF\Tickets\Repository\Status;
use XF\Http\Request;
use XF\InlineMod\AbstractAction;
use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\Entity\Entity;

use function intval;

class ApplyStatus extends AbstractAction
{
    protected $targetStatusId;
    protected $targetStatus;

    public function getTitle()
    {
        return \XF::phrase('nf_tickets_apply_status');
    }

    protected function canApplyToEntity(Entity $entity, array $options, &$error = null)
    {
        /** @var Ticket $entity */
        return $entity->canUpdateStatus();
    }

    protected function applyToEntity(Entity $entity, array $options)
    {
        /** @var Ticket $entity */

        $status = $this->getTargetStatus($options['apply_status_id']);
        if (!$status)
        {
            throw new \InvalidArgumentException("No status specified");
        }

        $entity->status_id = $status->status_id;
        $entity->save();
    }

    public function getBaseOptions()
    {
        return [
            'apply_status_id' => 0,
        ];
    }

    public function renderForm(AbstractCollection $entities, \XF\Mvc\Controller $controller)
    {
        /** @var Status $statusRepo */
        $statusRepo = $this->app()->repository('NF\Tickets:Status');
        $statuses = $statusRepo->findStatusesForList();

        $viewParams = [
            'tickets'  => $entities,
            'total'    => $entities->count(),
            'statuses' => $statuses,
            'first'    => $entities->first(),
        ];

        return $controller->view('NF\Tickets:Public:InlineMod\Ticket\ApplyStatus', 'nf_tickets_inline_mod_ticket_apply_status', $viewParams);
    }

    public function getFormOptions(AbstractCollection $entities, Request $request)
    {
        $options = [
            'apply_status_id' => $request->filter('apply_status_id', 'uint'),
        ];

        return $options;
    }

    /**
     * @param integer $statusId
     * @return null|\NF\Tickets\Entity\Status
     */
    protected function getTargetStatus($statusId)
    {
        $statusId = intval($statusId);

        if ($this->targetStatusId && $this->targetStatusId === $statusId)
        {
            return $this->targetStatus;
        }
        if (!$statusId)
        {
            return null;
        }

        /** @var \NF\Tickets\Entity\Status $status */
        $status = $this->app()->em()->find('NF\Tickets:Status', $statusId);
        if (!$status || !$status->active)
        {
            throw new \InvalidArgumentException("Invalid target status ($statusId)");
        }

        $this->targetStatusId = $statusId;
        $this->targetStatus = $status;

        return $this->targetStatus;
    }
}