<?php

namespace NF\Tickets\InlineMod\Ticket;

use NF\Tickets\Entity\Ticket;
use NF\Tickets\Service\Ticket\Deleter;
use XF\Http\Request;
use XF\InlineMod\AbstractAction;
use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\Entity\Entity;

use function count;

class Delete extends AbstractAction
{
    public function getTitle()
    {
        return \XF::phrase('nf_tickets_delete_tickets');
    }

    protected function canApplyToEntity(Entity $entity, array $options, &$error = null)
    {
        /** @var Ticket $entity */
        return $entity->canDelete($options['type'] ?: 'soft');
    }

    protected function applyToEntity(Entity $entity, array $options)
    {
        /** @var Deleter $deleter */
        $deleter = $this->app()->service('NF\Tickets:Ticket\Deleter', $entity);

        if ($options['alert'])
        {
            $deleter->setSendAlert(true, $options['alert_reason']);
        }

        $deleter->delete($options['type'], $options['reason']);

        if ($options['type'] === 'hard')
        {
            $this->returnUrl = $this->app()->router('public')->buildLink('tickets/queue');
        }
    }

    public function renderForm(AbstractCollection $entities, \XF\Mvc\Controller $controller)
    {
        $viewParams = [
            'tickets'       => $entities,
            'total'         => count($entities),
            'canHardDelete' => $this->canApply($entities, ['type' => 'hard']),
        ];

        return $controller->view('NF\Tickets:Public:InlineMod\Ticket\Delete', 'nf_tickets_inline_mod_ticket_delete', $viewParams);
    }

    public function getBaseOptions()
    {
        return [
            'type'         => 'soft',
            'reason'       => '',
            'alert'        => false,
            'alert_reason' => '',
        ];
    }

    public function getFormOptions(AbstractCollection $entities, Request $request)
    {
        return [
            'type'         => $request->filter('hard_delete', 'bool') ? 'hard' : 'soft',
            'reason'       => $request->filter('reason', 'str'),
            'alert'        => $request->filter('starter_alert', 'bool'),
            'alert_reason' => $request->filter('starter_alert_reason', 'str'),
        ];
    }
}