<?php

namespace NF\Tickets\InlineMod\Ticket;

use NF\Tickets\Entity\Ticket;
use NF\Tickets\Service\Ticket\Merger;
use XF\Http\Request;
use XF\InlineMod\AbstractAction;
use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\Entity\Entity;

use function count;

class Merge extends AbstractAction
{
    public function getTitle()
    {
        return \XF::phrase('nf_tickets_merge_tickets');
    }

    protected function canApplyInternal(AbstractCollection $entities, array $options, &$error)
    {
        $result = parent::canApplyInternal($entities, $options, $error);

        if ($result)
        {
            if ($options['target_ticket_id'])
            {
                if (!isset($entities[$options['target_ticket_id']]))
                {
                    return false;
                }
            }

            if ($entities->count() < 2)
            {
                return false;
            }
        }

        return $result;
    }

    protected function canApplyToEntity(Entity $entity, array $options, &$error = null)
    {
        /** @var Ticket $entity */
        return $entity->canMerge();
    }

    protected function applyInternal(AbstractCollection $entities, array $options)
    {
        if (!$options['target_ticket_id'])
        {
            throw new \InvalidArgumentException("No target ticket selected");
        }

        $source = $entities->toArray();
        $target = $source[$options['target_ticket_id']];
        unset($source[$options['target_ticket_id']]);

        /** @var Merger $merger */
        $merger = $this->app()->service('NF\Tickets:Ticket\Merger', $target);

        if ($options['alert'])
        {
            $merger->setSendAlert(true, $options['alert_reason']);
        }

        $merger->merge($source);

        $this->returnUrl = $this->app()->router()->buildLink('tickets', $target);
    }

    protected function applyToEntity(Entity $entity, array $options)
    {
        throw new \LogicException("applyToEntity should not be called on ticket merging");
    }

    public function getBaseOptions()
    {
        return [
            'target_ticket_id' => 0,
            'alert'            => false,
            'alert_reason'     => '',
        ];
    }

    public function renderForm(AbstractCollection $entities, \XF\Mvc\Controller $controller)
    {
        $viewParams = [
            'tickets' => $entities,
            'total'   => count($entities),
            'first'   => $entities->first(),
        ];

        return $controller->view('NF\Tickets:Public:InlineMod\Ticket\Merge', 'nf_tickets_inline_mod_ticket_merge', $viewParams);
    }

    public function getFormOptions(AbstractCollection $entities, Request $request)
    {
        $options = [
            'target_ticket_id' => $request->filter('target_ticket_id', 'uint'),
            'alert'            => $request->filter('starter_alert', 'bool'),
            'alert_reason'     => $request->filter('starter_alert_reason', 'str'),
        ];

        return $options;
    }
}