<?php

namespace NF\Tickets\InlineMod\Ticket;

use NF\Tickets\Entity\Ticket;
use NF\Tickets\Repository\Category;
use XF\Http\Request;
use XF\InlineMod\AbstractAction;
use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\Entity\Entity;

use function count;

class Move extends AbstractAction
{
    /** @var int|null */
    protected $targetCategoryId;
    /** @var \NF\Tickets\Entity\Category|null */
    protected $targetCategory;

    public function getTitle()
    {
        return \XF::phrase('nf_tickets_move_tickets');
    }

    protected function canApplyToEntity(Entity $entity, array $options, &$error = null)
    {
        /** @var Ticket $entity */
        return $entity->canMove();
    }

    protected function applyToEntity(Entity $entity, array $options)
    {
        /** @var Ticket $entity */

        $category = $this->getTargetCategory((int)$options['target_category_id']);
        if (!$category)
        {
            throw new \InvalidArgumentException("No target specified");
        }

        /** @var \NF\Tickets\Service\Ticket\Move $mover */
        $mover = $this->app()->service('NF\Tickets:Ticket\Move', $entity);

        /* TODO: Implement these?
        if ($options['alert'])
        {
            $mover->setSendAlert(true, $options['alert_reason']);
        }

        if ($options['notify_watchers'])
        {
            $mover->setNotifyWatchers();
        }*/

        $mover->move($category);

        $this->returnUrl = $this->app()->router('public')->buildLink('tickets/categories', $category);
    }

    public function getBaseOptions()
    {
        return [
            'target_category_id' => 0,
        ];
    }

    public function renderForm(AbstractCollection $entities, \XF\Mvc\Controller $controller)
    {
        /** @var Category $categoryRepo */
        $categoryRepo = $this->app()->repository('NF\Tickets:Category');
        $categoryTree = $categoryRepo->createCategoryTree($categoryRepo->getViewableCategories());

        $viewParams = [
            'tickets'      => $entities,
            'total'        => count($entities),
            'categoryTree' => $categoryTree,
            'first'        => $entities->first(),
        ];

        return $controller->view('NF\Tickets:Public:InlineMod\Ticket\Move', 'nf_tickets_inline_mod_ticket_move', $viewParams);
    }

    public function getFormOptions(AbstractCollection $entities, Request $request)
    {
        $options = [
            'target_category_id' => $request->filter('target_category_id', 'uint'),
        ];

        return $options;
    }

    /**
     * @param int $categoryId
     * @return \NF\Tickets\Entity\Category|null
     */
    protected function getTargetCategory(int $categoryId)
    {
        if ($this->targetCategoryId && $this->targetCategoryId === $categoryId)
        {
            return $this->targetCategory;
        }
        if (!$categoryId)
        {
            return null;
        }

        /** @var \NF\Tickets\Entity\Priority $category */
        $category = $this->app()->em()->find('NF\Tickets:Category', $categoryId);
        if (!$category)
        {
            throw new \InvalidArgumentException("Invalid target category ($categoryId)");
        }

        $this->targetCategoryId = $categoryId;
        $this->targetCategory = $category;

        return $this->targetCategory;
    }
}