<?php

namespace NF\Tickets\InlineMod\Ticket;

use NF\Tickets\Entity\Ticket;
use XF\Http\Request;
use XF\InlineMod\AbstractAction;
use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\Entity\Entity;

class Reassign extends AbstractAction
{
    /** @var int|null */
    protected $targetUserId;
    /** @var \XF\Entity\User|null */
    protected $targetUser;

    public function getTitle()
    {
        return \XF::phrase('nf_tickets_reassign_tickets');
    }

    protected function canApplyInternal(AbstractCollection $entities, array $options, &$error)
    {
        $result = parent::canApplyInternal($entities, $options, $error);

        if ($result)
        {
            if ($options['confirmed'] && !$options['target_user_id'])
            {
                $error = \XF::phrase('requested_user_not_found');

                return false;
            }
        }

        return $result;
    }

    protected function canApplyToEntity(Entity $entity, array $options, &$error = null)
    {
        /** @var Ticket $entity */
        return $entity->canReassign();
    }

    protected function applyToEntity(Entity $entity, array $options)
    {
        $user = $this->getTargetUser($options['target_user_id']);
        if (!$user)
        {
            throw new \InvalidArgumentException("No target specified");
        }

        /** @var \NF\Tickets\Service\Ticket\Reassign $reassigner */
        $reassigner = $this->app()->service('NF\Tickets:Ticket\Reassign', $entity);

        $reassigner->reassignTo($user);
    }

    public function getBaseOptions()
    {
        return [
            'target_user_id' => 0,
            'confirmed'      => false,
        ];
    }

    public function renderForm(AbstractCollection $entities, \XF\Mvc\Controller $controller)
    {
        $viewParams = [
            'tickets' => $entities,
            'total'   => $entities->count(),
        ];

        return $controller->view('NF\Tickets:Public:InlineMod\Ticket\Reassign', 'nf_tickets_inline_mod_ticket_reassign', $viewParams);
    }

    public function getFormOptions(AbstractCollection $entities, Request $request)
    {
        $username = $request->filter('username', 'str');
        /** @var \XF\Entity\User $user */
        $user = $this->app()->em()->findOne('XF:User', ['username' => $username]);

        return [
            'target_user_id' => $user ? $user->user_id : 0,
            'confirmed'      => true,
        ];
    }

    /**
     * @param int $userId
     * @return null|\XF\Entity\User
     */
    protected function getTargetUser(int $userId)
    {
        if ($this->targetUserId && $this->targetUserId === $userId)
        {
            return $this->targetUser;
        }
        if (!$userId)
        {
            return null;
        }

        /** @var \XF\Entity\User $user */
        $user = $this->app()->em()->find('XF:User', $userId);
        if (!$user)
        {
            throw new \InvalidArgumentException("Invalid target user ($userId)");
        }

        $this->targetUserId = $userId;
        $this->targetUser = $user;

        return $this->targetUser;
    }
}