<?php

namespace NF\Tickets\Install\Data;

use SV\StandardLib\InstallerHelper;
use XF\Db\Schema\Alter;
use XF\Db\Schema\Create;

class MySql
{

    use InstallerHelper;

    public function getTables()
    {
        $tables = [];

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_category'] = function($table)
        {
            $this->addOrChangeColumn($table, 'ticket_category_id', 'int')->autoIncrement();
            $this->addOrChangeColumn($table, 'title', 'varchar', 100);
            $this->addOrChangeColumn($table, 'description', 'text');
            $this->addOrChangeColumn($table, 'parent_category_id', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'depth', 'smallint', 5)->setDefault(0);
            $this->addOrChangeColumn($table, 'lft', 'int')->unsigned()->setDefault(0);
            $this->addOrChangeColumn($table, 'rgt', 'int')->unsigned()->setDefault(0);
            $this->addOrChangeColumn($table, 'display_order', 'int')->unsigned()->setDefault(0);
            $this->addOrChangeColumn($table, 'ticket_count', 'int')->unsigned()->setDefault(0);
            $this->addOrChangeColumn($table, 'last_ticket', 'int')->unsigned()->setDefault(0);
            $this->addOrChangeColumn($table, 'last_ticket_title', 'varchar', 150);
            $this->addOrChangeColumn($table, 'last_ticket_id', 'int')->unsigned()->setDefault(0);
            $this->addOrChangeColumn($table, 'breadcrumb_data', 'blob');
            $this->addOrChangeColumn($table, 'thread_node_id', 'int')->unsigned()->setDefault(0);
            $this->addOrChangeColumn($table, 'thread_prefix_id', 'int')->unsigned()->setDefault(0);
            $this->addOrChangeColumn($table, 'field_cache', 'mediumblob');
            $this->addOrChangeColumn($table, 'priority_selection', 'int')->unsigned()->setDefault(1);
            $this->addOrChangeColumn($table, 'empty_message_allowed', 'int')->unsigned()->setDefault(0);
            $this->addOrChangeColumn($table, 'hide_message_field', 'int')->unsigned()->setDefault(0);
            $this->addOrChangeColumn($table, 'category_emails', 'text');
            $this->addOrChangeColumn($table, 'incoming_ticket_email_settings', 'mediumblob')->nullable();
            $this->addOrChangeColumn($table, 'prefix_cache', 'mediumblob');
            $this->addOrChangeColumn($table, 'default_prefix_id', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'require_prefix', 'tinyint')->setDefault(0);
            $this->addOrChangeColumn($table, 'anonymous_user_id', 'int')->nullable()->setDefault(null);
	        $this->addOrChangeColumn($table, 'count_messages', 'tinyint', 1)->setDefault(1);
	        $this->addOrChangeColumn($table, 'allow_opening', 'tinyint', 1)->setDefault(1);
            $table->addKey(['ticket_category_id', 'lft'], 'parent_category_id_lft');
            $table->addKey(['lft', 'rgt'], 'lft_rgt');
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_category_field'] = function($table)
        {
            $this->addOrChangeColumn($table, 'field_id', 'varbinary', 25);
            $this->addOrChangeColumn($table, 'ticket_category_id', 'int', 11);
            $table->addPrimaryKey(['field_id', 'ticket_category_id']);
            $table->addKey('ticket_category_id');
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_category_read'] = function($table)
        {
            $this->addOrChangeColumn($table, 'category_read_id', 'int')->autoIncrement();
            $this->addOrChangeColumn($table, 'user_id', 'int')->unsigned();
            $this->addOrChangeColumn($table, 'ticket_category_id', 'int');
            $this->addOrChangeColumn($table, 'category_read_date', 'int');
            $table->addPrimaryKey('category_read_id');
            $table->addUniqueKey(['user_id', 'ticket_category_id'], 'user_id_category_id');
            $table->addKey('ticket_category_id', 'category_id');
            $table->addKey('category_read_date');
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_category_watch'] = function($table)
        {
            $this->addOrChangeColumn($table, 'user_id', 'int');
            $this->addOrChangeColumn($table, 'ticket_category_id', 'int');
            $this->addOrChangeColumn($table, 'notify_on')->resetDefinition()->type('enum')->values(['', 'ticket', 'message']);
            $this->addOrChangeColumn($table, 'send_alert', 'tinyint', 3);
            $this->addOrChangeColumn($table, 'send_email', 'tinyint', 3);
            $this->addOrChangeColumn($table, 'include_children', 'tinyint', 3);
            $table->addPrimaryKey(['user_id', 'ticket_category_id']);
            $table->addKey(['ticket_category_id', 'notify_on'], 'category_id_notify_on');
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_escalation'] = function($table)
        {
            $this->addOrChangeColumn($table, 'escalation_id', 'int')->autoIncrement();
            $this->addOrChangeColumn($table, 'escalation_title', 'varchar', 150)->setDefault('');
            $this->addOrChangeColumn($table, 'escalation_description', 'text');
            $this->addOrChangeColumn($table, 'active', 'tinyint', 3)->setDefault(1);
            $this->addOrChangeColumn($table, 'execution_order', 'int')->setDefault(10);
            $this->addOrChangeColumn($table, 'actions', 'mediumblob');
            $this->addOrChangeColumn($table, 'user_criteria', 'mediumblob');
            $this->addOrChangeColumn($table, 'ticket_criteria', 'mediumblob');
            $this->addOrChangeColumn($table, 'message_body', 'text');
            $this->addOrChangeColumn($table, 'message_from_username', 'varchar', 50);
            $this->addOrChangeColumn($table, 'alert_body', 'text');
            $this->addOrChangeColumn($table, 'alert_from_username', 'varchar', 50);
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_escalation_trigger_log'] = function($table)
        {
            $this->addOrChangeColumn($table, 'trigger_id', 'int')->autoIncrement();
            $this->addOrChangeColumn($table, 'ticket_id', 'int');
            $this->addOrChangeColumn($table, 'escalation_id', 'int');
            $this->addOrChangeColumn($table, 'trigger_date', 'int');
            $this->addOrChangeColumn($table, 'changes', 'mediumblob')->nullable(true)->setDefault(null);
            $table->addKey('trigger_date');
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_message'] = function($table)
        {
            $this->addOrChangeColumn($table, 'message_id', 'int')->autoIncrement();
            $this->addOrChangeColumn($table, 'ticket_id', 'int');
            $this->addOrChangeColumn($table, 'user_id', 'int');
            $this->addOrChangeColumn($table, 'username', 'varchar', 50);
            $this->addOrChangeColumn($table, 'user_name', 'varchar', 50);
            $this->addOrChangeColumn($table, 'user_email', 'varchar', 120);
            $this->addOrChangeColumn($table, 'message_date', 'int', 10);
            $this->addOrChangeColumn($table, 'message', 'mediumtext');
            $this->addOrChangeColumn($table, 'ip_id', 'bigint')->setDefault(0);
            $this->addOrChangeColumn($table, 'message_state')
                ->resetDefinition()
                ->type('enum')
                ->values(['visible', 'hidden', 'moderated', 'deleted']);
            $this->addOrChangeColumn($table, 'attach_count', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'position', 'int');
            $this->addOrChangeColumn($table, 'reaction_score', 'int')->unsigned(false)->setDefault(0);
            $this->addOrChangeColumn($table, 'reactions', 'blob')->nullable();
            $this->addOrChangeColumn($table, 'reaction_users', 'blob');
            $this->addOrChangeColumn($table, 'warning_id', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'warning_message', 'varchar', 255)->setDefault('');
            $this->addOrChangeColumn($table, 'last_edit_date', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'last_edit_user_id', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'edit_count', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'embed_metadata', 'blob')->nullable(true);
            $this->addOrChangeColumn($table, 'anonymized', 'tinyint', 1)->setDefault(0);
			$this->addOrChangeColumn($table, 'user_agent', 'varchar', 255)->nullable()->setDefault(null);
            $table->addKey(['position', 'message_id'], 'message_id_position');
            $table->addKey('user_id');
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_redirect'] = function($table)
        {
            $this->addOrChangeColumn($table, 'ticket_id', 'int');
            $this->addOrChangeColumn($table, 'target_url', 'text');
            $this->addOrChangeColumn($table, 'redirect_key', 'varchar', 50)->setDefault('');
            $this->addOrChangeColumn($table, 'expiry_date', 'int')->setDefault(0);
            $table->addPrimaryKey('ticket_id');
            $table->addKey(['redirect_key', 'expiry_date'], 'redirect_key_expiry_date');
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_response'] = function($table)
        {
            $this->addOrChangeColumn($table, 'response_id', 'int')->autoIncrement();
            $this->addOrChangeColumn($table, 'user_id', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'response_title', 'varchar', 50);
            $this->addOrChangeColumn($table, 'response_category_id', 'int');
            $this->addOrChangeColumn($table, 'message', 'text');
            $this->addOrChangeColumn($table, 'active', 'tinyint', 1);
            $this->addOrChangeColumn($table, 'display_order', 'int', 10);
            $table->addKey('user_id');
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_response_category'] = function($table)
        {
            $this->addOrChangeColumn($table, 'response_category_id', 'int')->autoIncrement();
            $this->addOrChangeColumn($table, 'category_name', 'varchar', 50);
            $this->addOrChangeColumn($table, 'display_order', 'int', 10);
            $table->addKey('display_order');
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_spam_log'] = function($table)
        {
            if ($table instanceof Alter)
            {
                $table->dropIndexes('content_type');
                $table->dropColumns(['content_type', 'content_id']);
            }

            $this->addOrChangeColumn($table, 'trigger_log_id', 'int')->autoIncrement();
            $this->addOrChangeColumn($table, 'ticket_id', 'int');
            $this->addOrChangeColumn($table, 'log_date', 'int');
            $this->addOrChangeColumn($table, 'user_id', 'int');
            $this->addOrChangeColumn($table, 'username', 'varchar', 100)->nullable(true)->setDefault(null);
            $this->addOrChangeColumn($table, 'ip_address', 'varbinary', 16);
            $this->addOrChangeColumn($table, 'result', 'varbinary', 25);
            $this->addOrChangeColumn($table, 'details', 'mediumblob');
            $this->addOrChangeColumn($table, 'request_state', 'mediumblob');
            $table->addKey('log_date');
            $table->addKey('ticket_id');
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_ticket'] = function($table)
        {
            $this->addOrChangeColumn($table, 'ticket_id', 'int')->autoIncrement();
            $this->addOrChangeColumn($table, 'ticket_ref', 'varchar', 25);
            $this->addOrChangeColumn($table, 'title', 'varchar', 150);
            $this->addOrChangeColumn($table, 'user_id', 'int', 10);
            $this->addOrChangeColumn($table, 'username', 'varchar', 50);
            $this->addOrChangeColumn($table, 'user_name', 'varchar', 50);
            $this->addOrChangeColumn($table, 'user_email', 'varchar', 120);
            $this->addOrChangeColumn($table, 'password', 'varchar', 255);
            $this->addOrChangeColumn($table, 'start_date', 'int', 10);
            $this->addOrChangeColumn($table, 'first_message_id', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'first_message_date', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'first_message_reaction_score', 'int')->unsigned(false)->setDefault(0);
            $this->addOrChangeColumn($table, 'first_message_reactions', 'blob')->nullable();
            $this->addOrChangeColumn($table, 'priority')->resetDefinition()->type('tinyint', 3)->nullable()
                ->setDefault(null);
            $this->addOrChangeColumn($table, 'status_id', 'int');
            $this->addOrChangeColumn($table, 'ticket_state')->resetDefinition()->type('enum')->values([
                'open',
                'pending',
                'resolved'
            ]);
            $this->addOrChangeColumn($table, 'ticket_locked', 'tinyint', 3)->setDefault(0);
            $this->addOrChangeColumn($table, 'discussion_state')->resetDefinition()->type('enum')->values([
                'visible',
                'moderated',
                'deleted',
                'shadow_ban',
            ])->setDefault('visible');
            $this->addOrChangeColumn($table, 'discussion_type', 'varchar', 25)->setDefault('');
            $this->addOrChangeColumn($table, 'assigner_user_id', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'assigner_username', 'varchar', 50)->setDefault(' ');
            $this->addOrChangeColumn($table, 'assigned_user_id', 'int');
            $this->addOrChangeColumn($table, 'assigned_username', 'varchar', 50)->setDefault(' ');
            $this->addOrChangeColumn($table, 'ticket_category_id', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'last_message_id', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'last_message_date', 'int');
            $this->addOrChangeColumn($table, 'last_message_user_id', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'last_message_username', 'varchar', 50)->setDefault('');
            $this->addOrChangeColumn($table, 'last_modified_date', 'int');
            $this->addOrChangeColumn($table, 'last_modified_user_id', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'last_modified_username', 'varchar', 50)->setDefault('');
            $this->addOrChangeColumn($table, 'last_status_change_date', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'reply_count', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'prefix_id', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'custom_fields', 'mediumblob');
            $this->addOrChangeColumn($table, 'thread_id', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'starter_user_id', 'int');
            $this->addOrChangeColumn($table, 'starter_username', 'varchar', 50);
            $table->addKey('ticket_ref');
            $table->addKey('last_modified_date');
            $table->addKey(['ticket_category_id', 'assigned_user_id'], 'ticket_category_assigned_user');
            $table->addKey(['last_modified_user_id', 'last_modified_date'], 'user_id_modified');
            $table->addKey('thread_id');

            if ($table instanceof Alter)
            {
                $table->dropIndexes('assigned_user_id_state');
            }
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_ticket_field'] = function($table)
        {
            $this->addOrChangeColumn($table, 'field_id', 'varbinary', 25);
            $this->addOrChangeColumn($table, 'display_group', 'varbinary', 25)->setDefault('sidebar');
            $this->addOrChangeColumn($table, 'display_order', 'int')->setDefault(1);
            $this->addOrChangeColumn($table, 'field_type', 'varchar', 50)->setDefault('textbox');
            $this->addOrChangeColumn($table, 'field_choices', 'blob');
            $this->addOrChangeColumn($table, 'fill_callback_class', 'varchar', 75)->setDefault('');
            $this->addOrChangeColumn($table, 'fill_callback_method', 'varchar', 75)->setDefault('');
            $this->addOrChangeColumn($table, 'match_type')->resetDefinition()->type('varbinary', 25)
                ->setDefault('none');
            $this->addOrChangeColumn($table, 'match_params', 'blob');
            $this->addOrChangeColumn($table, 'max_length', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'display_template', 'text');
	        $this->addOrChangeColumn($table, 'wrapper_template', 'text');
	        $this->addOrChangeColumn($table, 'display_list', 'tinyint')->setDefault(0);
            $this->addOrChangeColumn($table, 'required', 'tinyint', 3)->setDefault(0);
            $this->addOrChangeColumn($table, 'usable_user_group_ids', 'blob');
            $this->addOrChangeColumn($table, 'viewable_user_group_ids', 'blob');
            $this->addOrChangeColumn($table, 'viewable_owner_user_group_ids', 'blob');
            $this->addOrChangeColumn($table, 'expiry_length_type', 'enum')->values(['never','days','weeks','months','years']);
            $this->addOrChangeColumn($table, 'expiry_length_unit', 'smallint', 5);
	        $this->addOrChangeColumn($table, 'user_editable', 'enum')->values(['yes','once','never'])->setDefault('yes');
	        $this->addOrChangeColumn($table, 'moderator_editable', 'tinyint', 3)->setDefault(0);

	        $table->addKey('field_id');
            $table->addKey(['display_group', 'display_order'], 'display_group_order');
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_ticket_field_value'] = function($table)
        {
            $this->addOrChangeColumn($table, 'ticket_id', 'int');
            $this->addOrChangeColumn($table, 'field_id', 'varbinary', 25);
            $this->addOrChangeColumn($table, 'field_value', 'mediumtext');
            $table->addPrimaryKey(['ticket_id', 'field_id']);
            $table->addKey('field_id');
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_ticket_read'] = function($table)
        {
            $this->addOrChangeColumn($table, 'ticket_read_id', 'int')->autoIncrement();
            $this->addOrChangeColumn($table, 'user_id', 'int');
            $this->addOrChangeColumn($table, 'ticket_id', 'int');
            $this->addOrChangeColumn($table, 'ticket_read_date', 'int');
            $table->addUniqueKey(['user_id', 'ticket_id'], 'user_id_ticket_id');
            $table->addKey('ticket_id');
            $table->addKey('ticket_read_date');
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_ticket_watch'] = function($table)
        {
            $this->addOrChangeColumn($table, 'user_id', 'int');
            $this->addOrChangeColumn($table, 'ticket_id', 'int');
            $this->addOrChangeColumn($table, 'email_subscribe', 'tinyint', 3)->setDefault(0);
            $this->addOrChangeColumn($table, 'watch_key', 'varchar', 16)->setDefault('');
            $table->addPrimaryKey(['user_id', 'ticket_id']);
            $table->addKey(['ticket_id', 'email_subscribe'], 'ticket_id_email_subscribe');
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_user_watch'] = function($table)
        {
            $this->addOrChangeColumn($table, 'user_id', 'int');
            $this->addOrChangeColumn($table, 'ticket_starter_id', 'int');
            $this->addOrChangeColumn($table, 'notify_on')->resetDefinition()->type('enum')->values(['', 'ticket']);
            $this->addOrChangeColumn($table, 'send_alert', 'tinyint', 3);
            $this->addOrChangeColumn($table, 'send_email', 'tinyint', 3);
            $table->addPrimaryKey(['user_id', 'ticket_starter_id']);
            $table->addKey(['ticket_starter_id', 'notify_on'], 'ticket_user_id_notify_on');
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_status'] = function($table)
        {
            /** @var Create|Alter $table */
            $this->addOrChangeColumn($table, 'status_id', 'int')->autoIncrement();
            //$this->addOrChangeColumn($table, 'status_type')->resetDefinition()->type('enum')->values(['active', 'open', 'close'])->setDefault('active');
            $this->addOrChangeColumn($table, 'include_in_counts', 'tinyint', 1)->setDefault(1);
            $this->addOrChangeColumn($table, 'include_in_defaults', 'tinyint', 1)->setDefault(1);
            $this->addOrChangeColumn($table, 'is_close', 'tinyint', 1)->setDefault(0);
            $this->addOrChangeColumn($table, 'alert_on_transition', 'tinyint', 1)->setDefault(1);
            $this->addOrChangeColumn($table, 'display_order', 'int');
            $this->addOrChangeColumn($table, 'active', 'tinyint', 3)->setDefault(1);
            $this->addOrChangeColumn($table, 'css_class', 'varchar', 50)->setDefault('');
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_priority'] = function($table)
        {
            /** @var Create|Alter $table */
            $this->addOrChangeColumn($table, 'priority_id', 'int')->autoIncrement();
            $this->addOrChangeColumn($table, 'display_order', 'int');
            $this->addOrChangeColumn($table, 'active', 'tinyint', 3)->setDefault(1);
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_ticket_participant'] = function($table)
        {
            $this->addOrChangeColumn($table, 'ticket_id', 'int');
            $this->addOrChangeColumn($table, 'user_id', 'int');
            $this->addOrChangeColumn($table, 'last_read_date', 'int')->setDefault(0);
            $table->addPrimaryKey(['ticket_id', 'user_id']);
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_ticket_guest_participant'] = function($table)
        {
            $this->addOrChangeColumn($table, 'ticket_id', 'int');
            $this->addOrChangeColumn($table, 'email', 'varchar', 120); // Same length as xf_user.email
            $this->addOrChangeColumn($table, 'username', 'varchar', 50);
            $this->addOrChangeColumn($table, 'password', 'varchar', 255);
            $table->addPrimaryKey(['ticket_id', 'email']);
            $table->addUniqueKey('username');
        };

        $tables['xf_nf_tickets_ticket_prefix_group'] = function($table)
        {
            $this->addOrChangeColumn($table, 'prefix_group_id', 'int')->nullable()->autoIncrement();
            $this->addOrChangeColumn($table, 'display_order', 'int')->setDefault(1);
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_ticket_prefix'] = function($table)
        {
            $this->addOrChangeColumn($table, 'prefix_id', 'int')->nullable()->autoIncrement();
            $this->addOrChangeColumn($table, 'prefix_group_id', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'display_order', 'int')->setDefault(1);
            $this->addOrChangeColumn($table, 'materialized_order', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'css_class', 'varchar', 50)->setDefault('label label--primary');
            $this->addOrChangeColumn($table, 'allowed_user_group_ids', 'blob');
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_category_prefix'] = function($table)
        {
            $this->addOrChangeColumn($table, 'ticket_category_id', 'int');
            $this->addOrChangeColumn($table, 'prefix_id', 'int');
            $table->addPrimaryKey(['ticket_category_id', 'prefix_id']);
            $table->addKey('prefix_id');
        };

        /**
         * @param $table Alter|Create
         */
        $tables['xf_nf_tickets_ticket_email_log'] = function($table)
        {
            $this->addOrChangeColumn($table, 'ticket_email_id', 'int')->autoIncrement();
            $this->addOrChangeColumn($table, 'log_date', 'int');
            $this->addOrChangeColumn($table, 'email_date', 'int');
            $this->addOrChangeColumn($table, 'message_type', 'varchar', 25);
            $this->addOrChangeColumn($table, 'action_taken', 'varchar', 25);
            $this->addOrChangeColumn($table, 'user_id', 'int')->nullable();
            $this->addOrChangeColumn($table, 'sender', 'varchar', 255)->nullable();
            $this->addOrChangeColumn($table, 'raw_message', 'mediumblob');
            $this->addOrChangeColumn($table, 'diagnostic_info', 'text')->nullable();
            $table->addKey('log_date');
        };

        $tables['xf_nf_tickets_ban_email'] = function($table)
        {
            /** @var Create|Alter $table */
            $this->addOrChangeColumn($table,'banned_email', 'varchar', 120);
            $this->addOrChangeColumn($table,'shadow_ban', 'int')->setDefault(1);
            $this->addOrChangeColumn($table,'create_user_id', 'int')->setDefault(0);
            $this->addOrChangeColumn($table,'create_date', 'int')->setDefault(0);
            $this->addOrChangeColumn($table,'reason', 'varchar', 255)->setDefault('');
            $this->addOrChangeColumn($table,'last_triggered_date', 'int')->setDefault(0);
            $table->addPrimaryKey('banned_email');
            $table->addKey('create_date');
        };

        $tables['xf_nf_tickets_kb_category'] = function($table)
        {
            $this->addOrChangeColumn($table, 'kb_category_id', 'int')->autoIncrement();
            $this->addOrChangeColumn($table, 'title', 'varchar', 100);
            $this->addOrChangeColumn($table, 'description', 'mediumtext');
            $this->addOrChangeColumn($table, 'parent_category_id', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'display_order', 'int')->unsigned()->setDefault(0);
            $this->addOrChangeColumn($table, 'depth', 'smallint', 5)->setDefault(0);
            $this->addOrChangeColumn($table, 'lft', 'int')->unsigned()->setDefault(0);
            $this->addOrChangeColumn($table, 'rgt', 'int')->unsigned()->setDefault(0);
            $this->addOrChangeColumn($table, 'breadcrumb_data', 'blob');
        };

        $tables['xf_nf_tickets_kb_article'] = function($table)
        {
            $this->addOrChangeColumn($table, 'kb_article_id', 'int')->autoIncrement();
            $this->addOrChangeColumn($table, 'kb_category_id', 'int');
            $this->addOrChangeColumn($table, 'title', 'varchar', 100);
            $this->addOrChangeColumn($table, 'content', 'mediumtext');
            $this->addOrChangeColumn($table, 'views', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'attach_count', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'embed_metadata', 'blob')->nullable(true);
            $this->addOrChangeColumn($table, 'created_at', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'updated_at', 'int')->setDefault(0);
        };

        $tables['xf_nf_tickets_kb_article_view'] = function($table)
        {
            /** @var Create|Alter $table */
            if ($table instanceof Create)
            {
                $table->engine('MEMORY');
            }

            $this->addOrChangeColumn($table, 'kb_article_id', 'int');
            $this->addOrChangeColumn($table, 'total', 'int');
            $table->addPrimaryKey('kb_article_id');
        };

	    $tables['xf_nf_tickets_queue_watch'] = function($table)
	    {
		    $this->addOrChangeColumn($table, 'user_id', 'int');
		    $this->addOrChangeColumn($table, 'notify_on')->resetDefinition()->type('enum')->values(['', 'ticket', 'message']);
		    $this->addOrChangeColumn($table, 'send_alert', 'tinyint', 3);
		    $this->addOrChangeColumn($table, 'send_email', 'tinyint', 3);
		    $table->addPrimaryKey('user_id');
	    };

        return $tables;
    }

    /**
     * @return array
     */
    public function getAlterTables(): array
    {
        $tables = [];

        $tables['xf_user'] = function(Alter $table)
        {
            $this->addOrChangeColumn($table, 'nf_tickets_count', 'int')->setDefault(0);
            $this->addOrChangeColumn($table, 'nf_tickets_unread', 'int')->setDefault(0);
            $table->addKey('nf_tickets_count');
            $table->addKey('nf_tickets_unread');
        };

		$tables['xf_rm_category'] = function (Alter $table)
		{
			$this->addOrChangeColumn($table, 'nf_tickets_category_id', 'int')->nullable()->setDefault(null);
		};

		$tables['xf_rm_resource'] = function (Alter $table)
		{
			$this->addOrChangeColumn($table, 'nf_tickets_category_id', 'int')->nullable()->setDefault(null);
		};

        return $tables;
    }

    /**
     * @return array
     */
    public function getRemoveAlterTables()
    {
        $tables = [];

        $tables['xf_user'] = function(Alter $table)
        {
            $table->dropIndexes(['nf_tickets_count', 'nf_tickets_unread']);
            $table->dropColumns(['nf_tickets_count', 'nf_tickets_unread']);
        };

        return $tables;
    }

    public function getData()
    {
        $data = [];

        $data['xf_nf_tickets_status'] = "
            INSERT IGNORE INTO xf_nf_tickets_status
              (status_id, display_order, include_in_counts, include_in_defaults, is_close, alert_on_transition, active, css_class)
            VALUES
            (1, 10, 1, 1, 0, 1, 1, 'label label--primary'),
            (2, 20, 0, 1, 0, 0, 1, 'label label--accent'),
            (3, 30, 0, 0, 1, 1, 1, 'label label--green')
        ";

        $data['xf_nf_tickets_priority'] = "
            INSERT IGNORE INTO xf_nf_tickets_priority
              (priority_id, display_order, active)          
            VALUES
            (1, 10, 1),
            (2, 20, 1),
            (3, 30, 1),
            (4, 40, 1)
        ";

        return $data;
    }

    public function getPhrases()
    {
        $phrases = [];

        $phrases['nf_tickets_status.1'] = 'Open';
        $phrases['nf_tickets_status.2'] = 'Pending';
        $phrases['nf_tickets_status.3'] = 'Resolved';
        $phrases['nf_tickets_priority.1'] = 'Low';
        $phrases['nf_tickets_priority.2'] = 'Medium';
        $phrases['nf_tickets_priority.3'] = 'High';
        $phrases['nf_tickets_priority.4'] = 'Urgent';

        return $phrases;
    }
}
