<?php

namespace NF\Tickets\Job;

use XF\Job\AbstractJob;

use function count;

class CategoryDelete extends AbstractJob
{
    protected $defaultData = [
        'ticket_category_id' => null,
        'count'              => 0,
        'total'              => null,
    ];

    public function run($maxRunTime)
    {
        $s = microtime(true);

        if (!$this->data['ticket_category_id'])
        {
            throw new \InvalidArgumentException('Cannot delete tickets without a ticket_category_id.');
        }

        $finder = $this->app->finder('NF\Tickets:Ticket')
                            ->where('ticket_category_id', $this->data['ticket_category_id']);

        if ($this->data['total'] === null)
        {
            $this->data['total'] = $finder->total();
            if (!$this->data['total'])
            {
                return $this->complete();
            }
        }

        $ids = $finder->pluckFrom('ticket_id')->fetch(1000);
        if (!$ids)
        {
            return $this->complete();
        }

        $continue = !(count($ids) < 1000);

        foreach ($ids AS $id)
        {
            $this->data['count']++;

            $ticket = $this->app->find('NF\Tickets:Ticket', $id);
            if (!$ticket)
            {
                continue;
            }
            $ticket->delete(false);

            if ($maxRunTime && microtime(true) - $s > $maxRunTime)
            {
                $continue = true;
                break;
            }
        }

        if ($continue)
        {
            return $this->resume();
        }
        else
        {
            return $this->complete();
        }
    }

    public function getStatusMessage()
    {
        $actionPhrase = \XF::phrase('deleting');
        $typePhrase = \XF::phrase('nf_tickets_tickets');

        return \sprintf('%s... %s (%s/%s)', $actionPhrase, $typePhrase,
            \XF::language()->numberFormat($this->data['count']), \XF::language()->numberFormat($this->data['total'])
        );
    }

    public function canCancel()
    {
        return true;
    }

    public function canTriggerByChoice()
    {
        return false;
    }
}