<?php

namespace NF\Tickets\Job;

use NF\Tickets\SubContainer\TicketEmail;
use XF\Job\AbstractJob;
use XF\Job\JobResult;

class TicketEmails extends AbstractJob
{
    protected $defaultData = [
        'start' => null,
        'category' => null
    ];

    /**
     * @param int $maxRunTime
     *
     * @return JobResult
     */
    public function run($maxRunTime)
    {
        if (!$this->data['start'])
        {
            $this->data['start'] = time();
        }

        /** @var \NF\Tickets\Entity\Category $ticketCategory */
        $ticketCategory = $this->app->em()->find('NF\Tickets:Category', $this->data['category']);

        if (!$ticketCategory || !$ticketCategory->incoming_ticket_email_settings)
        {
            return $this->complete();
        }

        /** @var TicketEmail $ticketEmailContainer */
        $ticketEmailContainer = $this->app->container('nfTicketsEmail');

        $storage = $ticketEmailContainer->storage($ticketCategory);
        if (!$storage)
        {
            return $this->complete();
        }

        $processor = $ticketEmailContainer->processor($ticketCategory);
        $finished = $processor->processFromStorage($storage, $maxRunTime);

        $storage->close();

        if ($finished)
        {
            return $this->complete();
        }

        if (time() - $this->data['start'] > 60 * 15)
        {
            // We don't want this to run forever, so limit it to 15 minutes.
            // We process emails oldest first, so it shouldn't cause any to be missed, though it might delay some.
            return $this->complete();
        }

        return $this->resume();
    }

    public function getStatusMessage()
    {
        return \XF::phrase('nf_tickets_processing_ticket_emails');
    }

    public function canCancel()
    {
        return false;
    }

    public function canTriggerByChoice()
    {
        return false;
    }
}