<?php

use function is_array;

class NixFifty_Tickets_DataWriter_TicketUserWatch extends XenForo_DataWriter
{
    /**
     * Gets the fields that are defined for the table. See parent for explanation.
     *
     * @return array
     */
    protected function _getFields()
    {
        return [
            'xf_nixfifty_ticket_user_watch' => [
                'user_id'           => ['type' => self::TYPE_UINT, 'required' => true],
                'ticket_starter_id' => ['type' => self::TYPE_UINT, 'required' => true],
                'notify_on'         => [
                    'type'          => self::TYPE_STRING, 'default' => '',
                    'allowedValues' => ['', 'ticket'],
                ],
                'send_alert'        => ['type' => self::TYPE_BOOLEAN, 'default' => 0],
                'send_email'        => ['type' => self::TYPE_BOOLEAN, 'default' => 0],
            ],
        ];
    }

    /**
     * Gets the actual existing data out of data that was passed in. See parent for explanation.
     *
     * @param mixed
     * @param mixed $data
     * @return array|false
     */
    protected function _getExistingData($data)
    {
        if (!is_array($data))
        {
            return false;
        }
        else if (isset($data['user_id'], $data['ticket_starter_id']))
        {
            $userId = $data['user_id'];
            $starterId = $data['ticket_starter_id'];
        }
        else if (isset($data[0], $data[1]))
        {
            $userId = $data[0];
            $starterId = $data[1];
        }
        else
        {
            return false;
        }

        return ['xf_nixfifty_ticket_user_watch' => $this->_getTicketUserWatchModel()->getUserTicketWatchByTicketStarterId($userId, $starterId)];
    }

    /**
     * Gets SQL condition to update the existing record.
     *
     * @param $tableName
     * @return string
     */
    protected function _getUpdateCondition($tableName)
    {
        return 'user_id = ' . $this->_db->quote($this->getExisting('user_id'))
               . ' AND ticket_starter_id = ' . $this->_db->quote($this->getExisting('ticket_starter_id'));
    }

    /**
     * @return NixFifty_Tickets_Model_TicketUserWatch
     */
    protected function _getTicketUserWatchModel()
    {
        return $this->getModelFromCache('NixFifty_Tickets_Model_TicketUserWatch');
    }
}
