<?php

use function is_array;

/**
 * Data writer for ticket watch records.
 *
 * @package XenForo_Ticket
 */
class NixFifty_Tickets_DataWriter_TicketWatch extends XenForo_DataWriter
{
    /**
     * Gets the fields that are defined for the table. See parent for explanation.
     *
     * @return array
     */
    protected function _getFields()
    {
        return [
            'xf_nixfifty_ticket_watch' => [
                'user_id'         => ['type' => self::TYPE_UINT, 'required' => true],
                'ticket_id'       => ['type' => self::TYPE_UINT, 'required' => true],
                'email_subscribe' => ['type' => self::TYPE_BOOLEAN, 'default' => 0],
            ],
        ];
    }

    /**
     * Gets the actual existing data out of data that was passed in. See parent for explanation.
     *
     * @param mixed
     * @param mixed $data
     * @return array|false
     */
    protected function _getExistingData($data)
    {
        if (!is_array($data))
        {
            return false;
        }
        else if (isset($data['user_id'], $data['ticket_id']))
        {
            $userId = $data['user_id'];
            $ticketId = $data['ticket_id'];
        }
        else if (isset($data[0], $data[1]))
        {
            $userId = $data[0];
            $ticketId = $data[1];
        }
        else
        {
            return false;
        }

        return ['xf_nixfifty_ticket_watch' => $this->_getTicketWatchModel()->getUserTicketWatchByTicketId($userId, $ticketId)];
    }

    /**
     * Gets SQL condition to update the existing record.
     *
     * @param $tableName
     * @return string
     */
    protected function _getUpdateCondition($tableName)
    {
        return 'user_id = ' . $this->_db->quote($this->getExisting('user_id'))
               . ' AND ticket_id = ' . $this->_db->quote($this->getExisting('ticket_id'));
    }

    /**
     * @return NixFifty_Tickets_Model_TicketWatch
     */
    protected function _getTicketWatchModel()
    {
        return $this->getModelFromCache('NixFifty_Tickets_Model_TicketWatch');
    }
}
