<?php

namespace NF\Tickets\Mail;

use NF\Tickets\Entity\Ticket;

use function count;

class Mail extends \XF\Mail\Mail
{
    const TICKET_HMAC_FORMAT = '%s:%d';

    protected $replyHmac;
    protected $replyVerpBase;

    /** @var Ticket */
    protected $ticket;

    public function setTo($email, $name = null)
    {
        parent::setTo($email, $name);

        if ($this->ticket)
        {
            $this->replyHmac = $this->mailer->calculateBounceHmac(\sprintf(self::TICKET_HMAC_FORMAT, $email, $this->ticket->ticket_id));

            if ($this->replyVerpBase)
            {
                $this->applyReplyVerp();
            }
        }

        return $this;
    }

    public function setTicket(Ticket $ticket)
    {
        $this->ticket = $ticket;

        if ($this->ticket->Category->incoming_ticket_email_settings)
        {
            $this->setReplyTo($this->ticket->Category->incoming_ticket_email_settings['address'], null, true);
        }

        return $this;
    }

    public function setReplyTo($email, $name = null, $useVerp = false)
    {
        parent::setReplyTo($email, $name);

        if ($useVerp)
        {
            $this->replyVerpBase = $email;
        }

        return $this;
    }

    protected function applyReplyVerp()
    {
        $verpAddress = $this->getReplyVerpAddress($this->replyHmac, $this->replyVerpBase);
        if ($verpAddress)
        {
            try
            {
                $this->message->setReplyTo($verpAddress);
            }
            /** @noinspection PhpRedundantCatchClauseInspection */
            catch (\Swift_SwiftException $e)
            {
                $this->applySetupError($e);
            }
        }

        return $verpAddress;
    }

    protected function getReplyVerpAddress($hmac, $verpBase, $to = null)
    {
        if (!$hmac || !$verpBase)
        {
            return null;
        }

        if (!$to)
        {
            $toAll = $this->message->getTo();
            if (!$toAll || count($toAll) > 1)
            {
                // 0 or 2+ to addresses, so we can't really do verp
                return null;
            }

            $to = \key($toAll);
        }

        $verpValue = \str_replace('@', '=', $to);
        $verpAddress = \str_replace('@', "+{$this->ticket->ticket_id}+{$hmac}+$verpValue@", $verpBase);
        $verpAddress = \preg_replace('/["\'\s\\\\]/', '', $verpAddress);

        return $verpAddress;
    }
}