<?php

namespace NF\Tickets\ModeratorLog;

use XF\Entity\ModeratorLog;
use XF\ModeratorLog\AbstractHandler;
use XF\Mvc\Entity\Entity;

class Message extends AbstractHandler
{
    public function isLoggable(Entity $content, $action, \XF\Entity\User $actor)
    {
        /** @var \NF\Tickets\Entity\Message $content */
        switch ($action)
        {
            case 'edit':
                if ($actor->user_id === $content->user_id)
                {
                    return false;
                }
        }

        return parent::isLoggable($content, $action, $actor);
    }

    /**
     * @param Entity $content
     * @param        $field
     * @param        $newValue
     * @param        $oldValue
     * @return array|bool|string
     */
    protected function getLogActionForChange(Entity $content, $field, $newValue, $oldValue)
    {
        /** @var \NF\Tickets\Entity\Message $content */
        switch ($field)
        {
            case 'message':
                return 'edit';

            case 'message_state':
                if ($newValue === 'visible' && $oldValue === 'moderated')
                {
                    return 'approve';
                }
                else if ($newValue === 'visible' && $oldValue === 'deleted')
                {
                    return 'undelete';
                }
                else if ($newValue === 'deleted')
                {
                    $reason = $content->DeletionLog ? $content->DeletionLog->delete_reason : '';

                    return ['delete_soft', ['reason' => $reason]];
                }
                else if ($newValue === 'moderated')
                {
                    return 'unapprove';
                }
                break;
        }

        return false;
    }

    protected function setupLogEntityContent(ModeratorLog $log, Entity $content)
    {
        /** @var \NF\Tickets\Entity\Message $content */
        $ticket = $content->Ticket;
        $log->content_user_id = $content->user_id;
        $log->content_username = $content->username;
        $log->content_title = $ticket->title;
        $log->content_url = \XF::app()->router('public')->buildLink('nopath:tickets/messages', $content);
        $log->discussion_content_type = 'nf_tickets_message';
        $log->discussion_content_id = $content->message_id;
    }

    public function getContentTitle(ModeratorLog $log)
    {
        if ($log->discussion_content_type === 'nf_tickets_ticket')
        {
            return \XF::phrase('nf_tickets_message_by_x_in_ticket_y', [
                'user'  => $log->content_username,
                'title' => \XF::app()->stringFormatter()->censorText($log->content_title),
            ]);
        }
        else
        {
            return \XF::phrase('nf_tickets_message_by_x', [
                'user' => $log->content_username,
            ]);
        }
    }
}