<?php

namespace NF\Tickets\ModeratorLog;

use NF\Tickets\Entity\TicketPrefix as TicketPrefixEntity;
use XF\Entity\ModeratorLog;
use XF\ModeratorLog\AbstractHandler;
use XF\Mvc\Entity\Entity;

class Ticket extends AbstractHandler
{
    public function isLoggable(Entity $content, $action, \XF\Entity\User $actor)
    {
        /** @var \NF\Tickets\Entity\Ticket $content */
        switch ($action)
        {
            case 'title':
            case 'custom_fields':
                if ($actor->user_id === $content->user_id)
                {
                    return false;
                }
        }

        return parent::isLoggable($content, $action, $actor);
    }

    protected function getLogActionForChange(Entity $content, $field, $newValue, $oldValue)
    {
        /** @var \NF\Tickets\Entity\Ticket $content */
        switch ($field)
        {
            case 'custom_fields':
                return 'custom_fields_edit';

            case 'thread_id':
                return 'discussion_thread';

            case 'title':
                return ['title', ['old' => $oldValue]];

            case 'status_id':
                /** @var \NF\Tickets\Entity\Status $status */
                $status = \XF::em()->find('NF\Tickets:Status', $oldValue);
                $oldStatus = $status ? $status->title : '';

                return ['status', ['old' => $oldStatus]];

            case 'prefix_id':
                /** @var TicketPrefixEntity $prefix */
                $prefix = \XF::em()->find('NF\Tickets:TicketPrefix', $oldValue);
                $oldPrefix = $prefix ? $prefix->title : '';

                return ['prefix', ['old' => $oldPrefix]];

            case 'priority':
                /** @var \NF\Tickets\Entity\Priority $priority */
                $priority = \XF::em()->find('NF\Tickets:Priority', $oldValue);
                $oldPriority = $priority ? $priority->title : '';

                return ['priority', ['old' => $oldPriority]];

            case 'ticket_locked':
                return ($newValue ? 'lock' : 'unlock');

            case 'ticket_category_id':
                /** @var \NF\Tickets\Entity\Category $category */
                $category = \XF::em()->find('NF\Tickets:Category', $oldValue);
                $oldTitle = $category ? $category->title : '';

                return ['move', ['from' => $oldTitle]];

            case 'assigned_user_id':
                if (!$newValue && $oldValue)
                {
                    /** @var \XF\Entity\User $oldUser */
                    $oldUser = \XF::em()->find('XF:User', $oldValue);
                    $from = $oldUser ? $oldUser->username : '';

                    return ['unassigned', ['user' => $from]];
                }

                /** @var \XF\Entity\User $oldUser */
                $oldUser = \XF::em()->find('XF:User', $newValue);
                $to = $oldUser ? $oldUser->username : '';

                return ['reassign', ['user' => $to]];

            case 'discussion_state':
                if ($newValue === 'visible' && $oldValue === 'moderated')
                {
                    return 'approve';
                }
                else if ($newValue === 'visible' && $oldValue === 'deleted')
                {
                    return 'undelete';
                }
                else if ($newValue === 'deleted')
                {
                    $deleteLog = $content->DeletionLog;
                    $reason = $deleteLog ? $deleteLog->delete_reason : '';

                    return ['delete_soft', ['reason' => $reason]];
                }
                else if ($newValue === 'moderated')
                {
                    return 'unapprove';
                }
                break;
        }

        return false;
    }

    protected function setupLogEntityContent(ModeratorLog $log, Entity $content)
    {
        /** @var \NF\Tickets\Entity\Ticket $content */
        $log->content_user_id = $content->user_id;
        $log->content_username = $content->username;
        $log->content_title = $content->title;
        $log->content_url = \XF::app()->router('public')->buildLink('nopath:tickets', $content);
        $log->discussion_content_type = 'nf_tickets_ticket';
        $log->discussion_content_id = $content->ticket_id;
    }

    protected function getActionPhraseParams(ModeratorLog $log)
    {
        if ($log->action === 'edit')
        {
            return ['elements' => \implode(', ', \array_keys($log->action_params))];
        }
        else
        {
            return parent::getActionPhraseParams($log);
        }
    }
}