<?php

namespace NF\Tickets\Notifier\Message;

/**
 * Class CategoryWatch
 *
 * @package NF\Tickets\Notifier\Message
 */
class CategoryWatch extends AbstractWatch
{
    /**
     * @return array
     */
    protected function getApplicableActionTypes()
    {
        return ['reply', 'ticket'];
    }

    /**
     * @return array
     */
    protected function getDefaultWatchNotifyData()
    {
        $message = $this->message;

        $finder = $this->app()->finder('NF\Tickets:CategoryWatch');

        $finder->where('ticket_category_id', $message->Ticket->ticket_category_id)
               ->where('User.user_state', '=', 'valid')
               ->where('User.is_banned', '=', 0)
               ->whereOr(
                   ['send_alert', '>', 0],
                   ['send_email', '>', 0]
               );

        if ($this->actionType === 'reply')
        {
            $finder->where('notify_on', 'message');
        }
        else
        {
            $finder->where('notify_on', ['ticket', 'message']);
        }

        $activeLimit = $this->app()->options()->watchAlertActiveOnly;
        if (!empty($activeLimit['enabled']))
        {
            $finder->where('User.last_activity', '>=', \XF::$time - 86400 * $activeLimit['days']);
        }

        $notifyData = [];
        foreach ($finder->fetchColumns(['user_id', 'send_alert', 'send_email']) AS $watch)
        {
            $notifyData[$watch['user_id']] = [
                'alert' => (bool)$watch['send_alert'],
                'email' => (bool)$watch['send_email'],
            ];
        }

        return $notifyData;
    }

    /**
     * @return string
     */
    protected function getWatchEmailTemplateName()
    {
        return 'nf_tickets_watched_category_' . ($this->actionType === 'ticket' ? 'ticket' : 'reply');
    }
}