<?php

namespace NF\Tickets\Notifier\Message;

class QueueWatch extends AbstractWatch
{
	protected function getApplicableActionTypes(): array
	{
		return ['reply', 'ticket'];
	}

	protected function getDefaultWatchNotifyData(): array
	{
		$finder = $this->app()->finder('NF\Tickets:QueueWatch');

		$finder
			->where('User.user_state', '=', 'valid')
			->where('User.is_banned', '=', 0)
			->whereOr(
				['send_alert', '>', 0],
				['send_email', '>', 0]
			);

		if ($this->actionType === 'reply')
		{
			$finder->where('notify_on', 'message');
		}
		else
		{
			$finder->where('notify_on', ['ticket', 'message']);
		}

		$activeLimit = $this->app()->options()->watchAlertActiveOnly;
		if (!empty($activeLimit['enabled']))
		{
			$finder->where('User.last_activity', '>=', \XF::$time - 86400 * $activeLimit['days']);
		}

		$notifyData = [];
		foreach ($finder->fetchColumns(['user_id', 'send_alert', 'send_email']) AS $watch)
		{
			$notifyData[$watch['user_id']] = [
				'alert' => (bool)$watch['send_alert'],
				'email' => (bool)$watch['send_email'],
			];
		}

		return $notifyData;
	}

	protected function getWatchEmailTemplateName(): string
	{
		return 'nf_tickets_watched_queue_' . ($this->actionType === 'ticket' ? 'ticket' : 'reply');
	}
}