<?php

namespace NF\Tickets\Notifier\Message;

use NF\Tickets\Entity\Message;
use XF\Notifier\AbstractNotifier;

/**
 * Class Quote
 *
 * @package NF\Tickets\Notifier\Message
 */
class Quote extends AbstractNotifier
{
    /**
     * @var Message
     */
    protected $message;

    /**
     * Quote constructor.
     *
     * @param \XF\App $app
     * @param Message $message
     */
    public function __construct(\XF\App $app, Message $message)
    {
        parent::__construct($app);

        $this->message = $message;
    }

    /**
     * @param \XF\Entity\User $user
     * @return bool
     */
    public function canNotify(\XF\Entity\User $user)
    {
        return $user->user_id !== $this->message->user_id;
    }

    /**
     * @param \XF\Entity\User $user
     * @return bool
     */
    public function sendAlert(\XF\Entity\User $user)
    {
        return \XF::asVisitor($user, function () use ($user) {
            $message = $this->message;
            $fromUser = $message->getAnonymousUser(true);

            return $this->basicAlert(
                $user,
                $fromUser->user_id,
                $fromUser->username,
                'nf_tickets_message',
                $message->message_id,
                'quote',
                ['depends_on_addon_id' => 'NF/Tickets']
            );
        });
    }
}