<?php

namespace NF\Tickets\Notifier\Message;

/**
 * Class TicketWatch
 *
 * @package NF\Tickets\Notifier\Message
 */
class TicketWatch extends AbstractWatch
{
    /**
     * @return array
     */
    protected function getApplicableActionTypes()
    {
        return ['reply'];
    }

    /**
     * @return array
     */
    protected function getDefaultWatchNotifyData()
    {
        $message = $this->message;

        if ($message->isFirstMessage())
        {
            return [];
        }

        $finder = $this->app()->finder('NF\Tickets:TicketWatch');

        $finder->where('ticket_id', $message->ticket_id)
               ->where('User.user_state', '=', 'valid');
        if (empty(\XF::options()->nfTicketsAllowBannedTicketUse))
        {
            $finder->where('User.is_banned', '=', 0);
        }

        $activeLimit = $this->app()->options()->watchAlertActiveOnly;
        if (!empty($activeLimit['enabled']))
        {
            $finder->where('User.last_activity', '>=', \XF::$time - 86400 * $activeLimit['days']);
        }

        $notifyData = [];
        foreach ($finder->fetchColumns(['user_id', 'email_subscribe']) AS $watch)
        {
            $notifyData[$watch['user_id']] = [
                'alert' => true,
                'email' => (bool)$watch['email_subscribe'],
            ];
        }

        return $notifyData;
    }

    /**
     * @return string
     */
    protected function getWatchEmailTemplateName()
    {
        return 'nf_tickets_watched_ticket_reply';
    }
}