<?php

namespace NF\Tickets\Notifier\Ticket;

use NF\Tickets\Entity\Ticket;
use XF\Entity\User;
use XF\Notifier\AbstractNotifier;

class Invite extends AbstractNotifier
{

    /**
     * @var Ticket
     */
    protected $ticket;

    /**
     * @var User
     */
    protected $from;

    public function __construct(\XF\App $app, Ticket $ticket, User $from = null)
    {
        parent::__construct($app);

        $this->ticket = $ticket;
        $this->from = $from ?: $ticket->User;
    }

    /**
     * @param User $user
     * @return bool
     */
    public function canNotify(User $user)
    {
        return $user->user_id !== $this->ticket->user_id;
    }

    /**
     * @param User $user
     * @return bool
     */
    public function sendAlert(User $user)
    {
        return \XF::asVisitor($user, function () use ($user) {
            $ticket = $this->ticket;
            $fromUser = ($this->from === null || $this->from->user_id)
                ? $ticket->getAnonymousUser(true)
                : $this->from;

            return $this->basicAlert(
                $user,
                $fromUser->user_id,
                $fromUser->username,
                'nf_tickets_ticket',
                $ticket->ticket_id,
                'invite',
                ['depends_on_addon_id' => 'NF/Tickets']
            );
        });
    }

    /**
     * @param User $user
     * @return bool
     */
    public function sendEmail(User $user)
    {
        if (!$user->email || $user->user_state !== 'valid')
        {
            return false;
        }

        $params = [
            'ticket'   => $this->ticket,
            'category' => $this->ticket->Category,
            'receiver' => $user,
        ];

        $template = 'nf_tickets_ticket_invite_ticket';

        $this->app()->mailer()->newMail()
             ->setToUser($user)
             ->setTemplate($template, $params)
             ->queue();

        return true;
    }
}