<?php

namespace NF\Tickets\Permission;

use NF\Tickets\Entity\Category;
use XF\Mvc\Entity\Entity;
use XF\Permission\TreeContentPermissions;

/**
 * Class CategoryPermissions
 *
 * @package NF\Tickets\Permission
 */
class CategoryPermissions extends TreeContentPermissions
{
    /**
     * @return string
     */
    public function getContentType()
    {
        return 'nf_tickets_category';
    }

    /**
     * @return \XF\Phrase
     */
    public function getAnalysisTypeTitle()
    {
        return \XF::phrase('nf_tickets_ticket_category_permissions');
    }

    /**
     * @param Entity|Category $entity
     * @return mixed|null
     */
    public function getContentTitle(Entity $entity)
    {
        return $entity->title;
    }

    /**
     * @param \XF\Entity\Permission $permission
     * @return bool
     */
    public function isValidPermission(\XF\Entity\Permission $permission)
    {
        return ($permission->permission_group_id === 'ticket');
    }

    /**
     * @return \XF\Tree
     */
    public function getContentTree()
    {
        /** @var \NF\Tickets\Repository\Category $categoryRepo */
        $categoryRepo = $this->builder->em()->getRepository('NF\Tickets:Category');

        return $categoryRepo->createCategoryTree($categoryRepo->findCategoryList()->fetch());
    }

    /**
     * @param       $contentId
     * @param array $calculated
     * @param array $childPerms
     * @return array
     */
    protected function getFinalPerms($contentId, array $calculated, array &$childPerms)
    {
        if (!isset($calculated['ticket']))
        {
            $calculated['ticket'] = [];
        }

        $final = $this->builder->finalizePermissionValues($calculated['ticket']);

        if (empty($final['view']))
        {
            $childPerms['ticket']['view'] = 'deny';
        }

        return $final;
    }

    /**
     * @param       $contentId
     * @param array $calculated
     * @param array $childPerms
     * @return array
     */
    protected function getFinalAnalysisPerms($contentId, array $calculated, array &$childPerms)
    {
        $final = $this->builder->finalizePermissionValues($calculated);

        if (empty($final['ticket']['view']))
        {
            $childPerms['ticket']['view'] = 'deny';
        }

        return $final;
    }
}