<?php

namespace NF\Tickets\Pub\Controller;

use XF;
use XF\Util\Ip;
use XF\Mvc\ParameterBag;
use XF\Service\FloodCheck;
use XF\Mvc\Entity\Repository;
use XF\Mvc\Reply\AbstractReply;
use NF\Tickets\Repository\Status;
use NF\Tickets\XF\Repository\User;
use NF\Tickets\Repository\Priority;
use NF\Tickets\Repository\TicketPrefix;
use XF\Pub\Controller\AbstractController;

class AbstractTicketController extends AbstractController
{
	protected function preDispatchType($action, ParameterBag $params)
	{
		/** @var \NF\Tickets\ControllerPlugin\Ticket $plugin */
		$plugin = $this->plugin('NF\Tickets:Ticket');
		$plugin->preDispatchForTicket($this->isDiscouraged(), $action, $params);

		parent::preDispatchType($action, $params);
	}

	protected function postDispatchType($action, ParameterBag $params, AbstractReply &$reply)
	{
		/** @var \NF\Tickets\ControllerPlugin\Ticket $plugin */
		$plugin = $this->plugin('NF\Tickets:Ticket');
		$plugin->postDispatchForTicket($action, $params, $reply);

		parent::postDispatchType($action, $params, $reply);
	}

	/** @var null|bool */
	protected $isDiscouraged = null;

	protected function isDiscouraged()
	{
		if ($this->isDiscouraged !== null)
		{
			return $this->isDiscouraged;
		}
		$this->isDiscouraged = parent::isDiscouraged();

		return $this->isDiscouraged;
	}

	public function assertNotFlooding($action, $floodingLimit = null)
	{
		$visitor = XF::visitor();
		if (!$visitor->hasPermission('general', 'bypassFloodCheck'))
		{
			$userId = $visitor->user_id;
			if (!$userId)
			{
				// xf_flood_check.user_id is unsigned 32 bits integer.
				// Use the IP address crc32'ed as a stand-in for the userid to fit into the field.
				// set the high bit to ensure it is unlikely to cause a collision with a valid user
				$binaryIp = Ip::convertIpStringToBinary($this->app->request()->getIp());
				$userId = crc32($binaryIp) | (1 << 31);
			}

			/** @var FloodCheck $floodChecker */
			$floodChecker = $this->service('XF:FloodCheck');
			$timeRemaining = $floodChecker->checkFlooding($action, $userId, $floodingLimit);
			if ($timeRemaining)
			{
				throw $this->exception($this->responseFlooding($timeRemaining));
			}

			return;
		}

		parent::assertNotFlooding($action, $floodingLimit);
	}

	protected function getUserRepo(): User
	{
        /** @noinspection PhpIncompatibleReturnTypeInspection */
		return $this->repository('XF:User');
	}

	protected function getTicketRepo(): \NF\Tickets\Repository\Ticket
	{
        /** @noinspection PhpIncompatibleReturnTypeInspection */
		return $this->repository('NF\Tickets:Ticket');
	}

	protected function getCategoryRepo(): \NF\Tickets\Repository\Category
	{
        /** @noinspection PhpIncompatibleReturnTypeInspection */
		return $this->repository('NF\Tickets:Category');
	}

	protected function getPriorityRepo(): Priority
	{
        /** @noinspection PhpIncompatibleReturnTypeInspection */
		return $this->repository('NF\Tickets:Priority');
	}

	protected function getPrefixRepo(): TicketPrefix
	{
        /** @noinspection PhpIncompatibleReturnTypeInspection */
		return $this->repository('NF\Tickets:TicketPrefix');
	}

	protected function getStatusRepo(): Status
	{
        /** @noinspection PhpIncompatibleReturnTypeInspection */
		return $this->repository('NF\Tickets:Status');
	}

	protected function getMessageRepo(): \NF\Tickets\Repository\Message
	{
        /** @noinspection PhpIncompatibleReturnTypeInspection */
		return $this->repository('NF\Tickets:Message');
	}

	protected function getResponseRepo(): \NF\Tickets\Repository\Response
	{
        /** @noinspection PhpIncompatibleReturnTypeInspection */
		return $this->repository('NF\Tickets:Response');
	}
}