<?php

namespace NF\Tickets\Pub\Controller;

use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\ParameterBag;

class Member extends AbstractTicketController
{
    public function actionIndex(ParameterBag $params)
    {
        if ($params->user_id)
        {
            return $this->rerouteController('NF\Tickets:Member', 'Member', $params);
        }
        // TODO: member stats?

        return $this->notFound();
    }

    public function actionMember(ParameterBag $params)
    {
        /** @var \XF\Entity\User $user */
        $user = $this->assertRecordExists('XF:User', $params->user_id);

        /** @var \NF\Tickets\XF\Entity\User $visitor */
        $visitor = \XF::visitor();

        $canView = $user->user_id === $visitor->user_id ? $visitor->canViewTickets() : $visitor->canViewTicketsOthers();

        if (!$canView)
        {
            return $this->noPermission();
        }

        /** @var \NF\Tickets\ControllerPlugin\TicketList $memberTicketList */
        $memberTicketList = $this->plugin('NF\Tickets:TicketList');

        $categoryParams = $memberTicketList->getCategoryListData();
        /** @var AbstractCollection $categories */
        $categories = $categoryParams['categories'];
        $viewableCategoryIds = $categories->keys();

        $listParams = $memberTicketList->getCoreListData($viewableCategoryIds, null, $user, true);

        $this->assertValidPage($listParams['page'], $listParams['perPage'], $listParams['total'], 'tickets/members', $user);
        $this->assertCanonicalUrl($this->buildLink('tickets/members', $user, ['page' => $listParams['page']]));

        $viewParams = $categoryParams + $listParams;

        return $this->view('NF\Tickets:Member\View', 'nf_tickets_member_view', $viewParams);
    }

    public function actionGuest()
    {
        /** @var \NF\Tickets\ControllerPlugin\TicketList $memberTicketList */
        $memberTicketList = $this->plugin('NF\Tickets:TicketList');

        $categoryParams = $memberTicketList->getCategoryListData();
        /** @var AbstractCollection $categories */
        $categories = $categoryParams['categories'];
        $viewableCategoryIds = $categories->keys();

        $listParams = $memberTicketList->getCoreListData($viewableCategoryIds, null, \XF::visitor());

        $this->assertValidPage($listParams['page'], $listParams['perPage'], $listParams['total'], 'tickets');
        $this->assertCanonicalUrl($this->buildLink('tickets', null, ['page' => $listParams['page']]));

        $viewParams = $categoryParams + $listParams;

        return $this->view('NF\Tickets:Member\View', 'nf_tickets_member_view', $viewParams);
    }

    public function actionFilters(ParameterBag $params)
    {
        /** @var \XF\Entity\User $user */
        $user = $this->assertRecordExists('XF:User', $params->user_id);

        /** @var \NF\Tickets\ControllerPlugin\TicketList $memberTicketListPlugin */
        $memberTicketListPlugin = $this->plugin('NF\Tickets:TicketList');

        return $memberTicketListPlugin->actionFilters(null, $user);
    }

    public static function getActivityDetails(array $activities)
    {
        return \XF::phrase('nf_tickets_viewing_tickets');
    }
}