<?php

namespace NF\Tickets\Pub\Controller;

use XF\Mvc\Reply\View;
use XF\Mvc\ParameterBag;
use XF\Mvc\Entity\Entity;
use NF\Tickets\XF\Entity\User;
use XF\Mvc\Reply\AbstractReply;
use NF\Tickets\Repository\Response;
use XF\Mvc\Entity\AbstractCollection;
use function strlen;

/**
 * Class Ticket
 *
 * @package NF\Tickets\Pub\Controller
 */
class Ticket extends AbstractTicketController
{
    /** @noinspection PhpParameterByRefIsNotUsedAsReferenceInspection */
    protected function postDispatchController($action, ParameterBag $params, AbstractReply &$reply)
	{
		if ($reply instanceof View)
		{
			if (!\XF::visitor()->user_id)
			{
				/** @var \NF\Tickets\Entity\Ticket $ticket */
				$ticket = $reply->getParam('ticket');

				if ($ticket && $ticket->canView())
				{
					$this->getTicketRepo()->updateGuestTicketAccess($ticket);
				}
			}

			$viewParams = $reply->getParams();
			$category = null;

			if (isset($viewParams['category']))
			{
				$category = $viewParams['category'];
			}
			if (isset($viewParams['ticket']))
			{
				$category = $viewParams['ticket']->Category;
			}
			if ($category)
			{
				$reply->setContainerKey('nfTicketsCategory-' . $category->ticket_category_id);
			}
		}
	}

	/**
	 * @param ParameterBag $parameterBag
	 * @return \XF\Mvc\Reply\Reroute|View
     */
	public function actionIndex(ParameterBag $parameterBag)
	{
		/** @var User $visitor */
		$visitor = \XF::visitor();
		/** @noinspection PhpUndefinedFieldInspection */
		if ($parameterBag->ticket_id)
		{
			return $this->rerouteController(__CLASS__, 'view', $parameterBag);
		}
		else if ($visitor->canAccessTicketQueue($error))
		{
			return $this->rerouteController(__CLASS__, 'queue', $parameterBag);
		}
		else if (!$visitor->user_id)
		{
			/** @var \NF\Tickets\ControllerPLugin\TicketList $overviewPlugin */
			$overviewPlugin = $this->plugin('NF\Tickets:TicketList');

			$guestTicketIds = $overviewPlugin->getGuestTicketIds();
			if (!$guestTicketIds)
			{
				return $this->rerouteController(__CLASS__, 'create');
			}

			return $this->rerouteController('NF\Tickets:Member', 'Guest');
		}
		else
		{
			return $this->rerouteController('NF\Tickets:Member', 'Member', ['user_id' => \XF::visitor()->user_id]);
		}
	}

	/**
	 * @return \XF\Mvc\Reply\Reroute
	 */
	public function actionYours()
	{
		return $this->rerouteController('NF\Tickets:Member', 'Member', ['user_id' => \XF::visitor()->user_id]);
	}

	/**
	 * @param ParameterBag $parameterBag
	 * @return AbstractReply
     * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionQueue(ParameterBag $parameterBag)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		if ($parameterBag->ticket_id)
		{
			return $this->rerouteController(__CLASS__, 'view', $parameterBag);
		}

		/** @var User $visitor */
		$visitor = \XF::visitor();
		if (!$visitor->canAccessTicketQueue($error))
		{
			return $this->noPermission($error);
		}
        $visitor->updateTicketLastRead();

		/** @var \NF\Tickets\ControllerPlugin\TicketList $queueListPlugin */
		$queueListPlugin = $this->plugin('NF\Tickets:TicketList');

		$categoryParams = $queueListPlugin->getCategoryListData();
		/** @var AbstractCollection $categories */
		$categories = $categoryParams['categories'];
		$viewableCategoryIds = $categories->keys();

		$listParams = $queueListPlugin->getCoreListData($viewableCategoryIds);

		$this->assertValidPage($listParams['page'], $listParams['perPage'], $listParams['total'], 'tickets/queue');
		$this->assertCanonicalUrl($this->buildLink('tickets/queue', null, ['page' => $listParams['page']]));

		$viewParams = $categoryParams + $listParams;

		return $this->view(
			'NF\Tickets:List\Queue',
			'nf_tickets_list_queue',
			$viewParams
		);
	}

	public function actionQueueFilters()
	{
		/** @var User $visitor */
		$visitor = \XF::visitor();
		if (!$visitor->canAccessTicketQueue($error))
		{
			return $this->noPermission($error);
		}

		/** @var \NF\Tickets\ControllerPlugin\TicketList $queuePlugin */
		$queuePlugin = $this->plugin('NF\Tickets:TicketList');

		return $queuePlugin->actionFilters();
	}

	public function actionQueueWatch()
	{
		/** @var User $visitor */
		$visitor = \XF::visitor();
		if (!$visitor->canAccessTicketQueue($error))
		{
			return $this->noPermission($error);
		}

		if ($this->isPost())
		{
			if ($this->filter('stop', 'bool'))
			{
				$action = 'delete';
				$config = [];
			}
			else
			{
				$action = 'watch';
				$config = [
					'notify_on' => $this->filter('notify_on', 'str'),
					'send_alert' => $this->filter('send_alert', 'bool'),
					'send_email' => $this->filter('send_email', 'bool'),
				];
			}

			/** @var \NF\Tickets\Repository\QueueWatch $watchRepo */
			$watchRepo = $this->repository('NF\Tickets:QueueWatch');
			$watchRepo->setWatchState($visitor, $action, $config);

			$redirect = $this->redirect($this->buildLink('tickets/queue'));
			$redirect->setJsonParam('switchKey', $action === 'delete' ? 'watch' : 'unwatch');
			return $redirect;
		}
		else
		{
			$viewParams = [
				'isWatched' => !empty($visitor->TicketQueueWatch),
			];
			return $this->view('NF\Tickets:Queue\Watch', 'nf_tickets_queue_watch', $viewParams);
		}
	}

	public function actionNew(ParameterBag $parameterBag)
	{
		return $this->rerouteController(__CLASS__, 'create', $parameterBag);
	}

	/**
	 * @param ParameterBag $parameterBag
	 * @return AbstractReply
     * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionCreate(/** @noinspection PhpUnusedParameterInspection */ ParameterBag $parameterBag)
	{
		/** @var User $visitor */
		$visitor = \XF::visitor();
		if (!$visitor->canCreateTicket($error))
		{
			return $this->noPermission($error);
		}

		$this->assertCanonicalUrl($this->buildLink('tickets/create'));

		$categoryRepo = $this->getCategoryRepo();

		$categories = $categoryRepo->getViewableCategories();
		$canAdd = false;

		/** @var \NF\Tickets\Entity\Category $category */
		foreach ($categories AS $category)
		{
			if ($category->canCreateTicket($error))
			{
				$canAdd = true;
				break;
			}
		}

		if (!$canAdd)
		{
			return $this->noPermission($error);
		}

		$categoryTree = $categoryRepo->createCategoryTree($categories);
		$categoryTree = $categoryTree->filter(null, function ($id, \NF\Tickets\Entity\Category $category, $depth, $children)
		{
			if ($children)
			{
				return true;
			}

			if ($category->canCreateTicket())
			{
				return true;
			}

			return false;
		});

		$categoryExtras = $categoryRepo->getCategoryListExtras($categoryTree);

		$viewParams = [
			'categoryTree' => $categoryTree,
			'categoryExtras' => $categoryExtras,
		];

		return $this->view('NF\Tickets:Ticket\CategoryChooser', 'nf_tickets_ticket_category_chooser', $viewParams);
	}

	/**
	 * @param ParameterBag $params
	 * @return \XF\Mvc\Reply\Error|View|\XF\Mvc\Reply\Redirect
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionView(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);

		$ticketRepo = $this->getTicketRepo();

        $page = $this->filterPage();
		$perPage = $this->options()->messagesPerPage;

		$this->assertValidPage($page, $perPage, $ticket->reply_count + 1, 'tickets', $ticket);
		$this->assertCanonicalUrl($this->buildLink('tickets', $ticket, ['page' => $page]));

		$messageRepo = $this->getMessageRepo();

		$messageList = $messageRepo->findMessagesForTicketView($ticket)->onPage($page, $perPage);
		$messages = $messageList->fetch();

		/** @var \XF\Repository\Attachment $attachmentRepo */
		$attachmentRepo = $this->repository('XF:Attachment');
		$attachmentRepo->addAttachmentsToContent($messages, 'nf_tickets_message');

		/** @var \NF\Tickets\Entity\Message $lastMessage */
		$lastMessage = $messages->last();
		if (!$lastMessage)
		{
			if ($page > 1)
			{
				return $this->redirect($this->buildLink('tickets', $ticket));
			}

			// should never really happen
			return $this->error(\XF::phrase('something_went_wrong_please_try_again'));
		}

		/** @var \NF\Tickets\Entity\Message $firstMessage */
		$firstMessage = $messages->first();

		/** @var \NF\Tickets\Entity\Message $message */
		$canInlineMod = false;
		foreach ($messages AS $message)
		{
			if ($message->canUseInlineModeration())
			{
				$canInlineMod = true;
				break;
			}
		}

		$firstUnread = null;
		foreach ($messages AS $message)
		{
			if ($message->isUnread())
			{
				$firstUnread = $message;
				break;
			}
		}

		/** @var Response $responseRepo */
		$responseRepo = $this->repository('NF\Tickets:Response');
		$responseData = $responseRepo->getResponseListData();

		$categoryRepo = $this->getCategoryRepo();
		$categories = $categoryRepo->getViewableCategories();

		$prefix = $ticket->Prefix;
		$prefixes = $ticket->Category->getUsablePrefixes($prefix);

		$ticketRepo->markTicketReadByVisitor($ticket, $lastMessage->message_date);
		/** @var \XF\Repository\UserAlert $userAlertRepo */
		$userAlertRepo = $this->repository('XF:UserAlert');
		$userAlertRepo->markUserAlertsReadForContent('nf_tickets_message', $messages->keys());

		$viewParams = [
			'ticket' => $ticket,
			'category' => $ticket->Category,
			'messages' => $messages,
			'firstMessage' => $firstMessage,
			'lastMessage' => $lastMessage,
			'firstUnread' => $firstUnread,

			'canInlineMod' => $canInlineMod,

			'page' => $page,
			'perPage' => $perPage,

			'attachmentData' => $this->getReplyAttachmentData($ticket),

			'categoryTree' => $categoryRepo->createCategoryTree($categories),
			'priorities' => $this->getPriorityRepo()->findPrioritiesForList(true)->fetch(),
			'statuses' => $this->getStatusRepo()->findStatusesForList(true, $ticket->status_id)->fetch(),
			'prefixes' => $prefixes,
			'responseCategories' => $responseData['responseCategories'],
			'responseTotal' => $responseData['responseTotal'],
		];

		return $this->view('NF\Tickets:Ticket\View', 'nf_tickets_ticket_view', $viewParams);
	}

	public function actionConfirmPassword(ParameterBag $params)
	{
		$this->assertPostOnly();

		$ticket = $this->assertTicketExists($params->ticket_id);
		// user is logged in or can already see the ticket, just try redirecting to the ticket
		if (\XF::visitor()->user_id || $ticket->canView())
		{
			return $this->redirect($this->buildLink('tickets', $ticket));
		}

		if (!$ticket->viewableToGuests())
		{
			return $this->notFound(\XF::phrase('nf_tickets_requested_ticket_not_found'));
		}

		$password = $this->filter('password', 'str');

		if ($ticket->confirmPassword($password))
		{
			$this->getTicketRepo()->updateGuestTicketAccess($ticket);

			return $this->redirect($this->buildLink('tickets', $ticket));
		}

		return $this->error(\XF::phrase('incorrect_password'));
	}

	public function actionUnread(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);

		if (!\XF::visitor()->user_id)
		{
			return $this->redirect($this->buildLink('tickets', $ticket));
		}

		$messageRepo = $this->getMessageRepo();
		$firstUnreadDate = $ticket->getVisitorReadDate();

		$forceNew = $this->filter('new', 'bool');

		if (!$forceNew && $firstUnreadDate <= $this->getTicketRepo()->getReadMarkingCutOff())
		{
			return $this->redirect($this->buildLink('tickets', $ticket));
		}

		$findFirstUnread = $messageRepo->findNewestMessagesInTicket($ticket, $firstUnreadDate);
		$firstUnread = $findFirstUnread->skipIgnored()->fetchOne();

		if (!$firstUnread)
		{
			$firstUnread = $ticket->LastMessage;
		}

		if (!$firstUnread)
		{
			return $this->redirect($this->buildLink('tickets', $ticket));
		}

		if ($firstUnread->message_id === $ticket->first_message_id)
		{
			return $this->redirect($this->buildLink('tickets', $ticket));
		}

		/** @var \NF\Tickets\ControllerPlugin\Ticket $plugin */
		$plugin = $this->plugin('NF\Tickets:Ticket');

		return $this->redirect($plugin->getMessageLink($firstUnread));
	}

	public function actionLatest(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);
		$message = $ticket->LastMessage;

		if ($message)
		{
			/** @var \NF\Tickets\ControllerPlugin\Ticket $plugin */
			$plugin = $this->plugin('NF\Tickets:Ticket');

			return $this->redirect($plugin->getMessageLink($message));
		}
		else
		{
			return $this->redirect($this->buildLink('tickets', $ticket));
		}
	}

	public function actionActivity(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);

		$maxItems = $this->options()->newsFeedMaxItems;

		/** @var \XF\Repository\NewsFeed $newsFeedRepo */
		$newsFeedRepo = $this->repository('XF:NewsFeed');

		$beforeId = $this->filter('before_id', 'uint');

		$newsFeedFinder = $newsFeedRepo->findMembersActivity($ticket->User);
		$newsFeedFinder->beforeFeedId($beforeId);

		/** @var \XF\Entity\NewsFeed[]|AbstractCollection $newsFeed */
		$newsFeed = $newsFeedFinder->fetch($maxItems * 2);
		$newsFeedRepo->addContentToNewsFeedItems($newsFeed);

		$newsFeed = $newsFeed->filterViewable();
		$newsFeed = $newsFeed->slice(0, $maxItems);

		$categoryRepo = $this->getCategoryRepo();
		$categories = $categoryRepo->getViewableCategories();

		$viewParams = [
			'ticket' => $ticket,
			'user' => $ticket->User,
			'newsFeedItems' => $newsFeed,
			'oldestItemId' => $newsFeed->count() ? \min(\array_keys($newsFeed->toArray())) : 0,
			'beforeId' => $beforeId,

			'priorities' => $this->getPriorityRepo()->findPrioritiesForList()->fetch(),
			'statuses' => $this->getStatusRepo()->findStatusesForList()->fetch(),

			'categoryTree' => $categoryRepo->createCategoryTree($categories),
		];

		return $this->view('NF\Tickets:Ticket\Activity', 'nf_tickets_ticket_activity', $viewParams);
	}

	public function actionField(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);

		$fieldId = $this->filter('field', 'str');
		$tabFields = $ticket->getExtraFieldTabs();

		if (!isset($tabFields[$fieldId]))
		{
			return $this->redirect($this->buildLink('tickets', $ticket));
		}

        $fieldSet = $ticket->custom_fields;
		$definition = $fieldSet->getDefinition($fieldId);
		$fieldValue = $fieldSet->getFieldValue($fieldId);
		$categoryRepo = $this->getCategoryRepo();
		$categories = $categoryRepo->getViewableCategories();

		$viewParams = [
			'ticket' => $ticket,
			'category' => $ticket->Category,

			'fieldId' => $fieldId,
			'fieldDefinition' => $definition,
			'fieldValue' => $fieldValue,

			'categoryTree' => $categoryRepo->createCategoryTree($categories),
			'priorities' => $this->getPriorityRepo()->findPrioritiesForList()->fetch(),
			'statuses' => $this->getStatusRepo()->findStatusesForList()->fetch(),
		];

		return $this->view('NF\Tickets:Ticket\Field', 'nf_tickets_ticket_field', $viewParams);
	}

	public function actionReply(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);
		if (!$ticket->canReply($error))
		{
			return $this->noPermission($error);
		}

		$defaultMessage = '';
		$forceAttachmentHash = null;

		$quote = $this->filter('quote', 'uint');
		if ($quote)
		{
			/** @var \NF\Tickets\Entity\Message $message */
			$message = $this->em()->find('NF\Tickets:Message', $quote, 'User');
			if ($message && $message->ticket_id === $ticket->ticket_id && $message->canView())
			{
				$defaultMessage = $message->getQuoteWrapper(
					$this->app->stringFormatter()->getBbCodeForQuote($message->message, 'post')
				);
				$forceAttachmentHash = '';
			}
		}
		else if ($this->request->exists('requires_captcha'))
		{
            /** @var \XF\ControllerPlugin\Editor $editorPlugin */
            $editorPlugin = $this->plugin('XF:Editor');
			$defaultMessage = $editorPlugin->fromInput('message');
			$forceAttachmentHash = $this->filter('attachment_hash', 'str');
		}
		else
		{
			$defaultMessage = $ticket->draft_message->message;
		}

		/** @var Response $responseRepo */
		$responseRepo = $this->repository('NF\Tickets:Response');
		$responseData = $responseRepo->getResponseListData();

		$viewParams = [
			'ticket' => $ticket,
			'category' => $ticket->Category,
			'attachmentData' => $this->getReplyAttachmentData($ticket, $forceAttachmentHash),
			'statuses' => $this->getStatusRepo()->findStatusesForList(true, $ticket->status_id)->fetch(),
			'responseCategories' => $responseData['responseCategories'],
			'responseTotal' => $responseData['responseTotal'],
			'defaultMessage' => $defaultMessage,
		];
		return $this->view('NF\Tickets:Ticket\Reply', 'nf_tickets_ticket_reply', $viewParams);
	}

	/**
	 * @param ParameterBag $params
	 *
	 * @return AbstractReply
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionAddMessage(ParameterBag $params)
	{
		$this->assertPostOnly();

		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);
		if (!$ticket->canReply($error))
		{
			return $this->noPermission($error);
		}

		if ($this->filter('no_captcha', 'bool')) // JS is disabled so user hasn't seen Captcha.
		{
			$this->request->set('requires_captcha', true);

			return $this->rerouteController(__CLASS__, 'reply', $params);
		}

		if (!$this->captchaIsValid())
		{
			return $this->error(\XF::phrase('did_not_complete_the_captcha_verification_properly'));
		}

		$replier = $this->setupTicketReply($ticket);
		$replier->checkForSpam();

		if (!$replier->validate($errors))
		{
			return $this->error($errors);
		}
		$this->assertNotFlooding('nf_tickets_message');
		$message = $replier->save();

		$this->finalizeTicketReply($replier);

		if ($this->filter('_xfWithData', 'bool') && $this->request->exists('last_date'))
		{
			// request was from quick reply
			$lastDate = $this->filter('last_date', 'uint');

			return $this->getNewMessagesReply($ticket, $lastDate);
		}

		$this->getTicketRepo()->markTicketReadByVisitor($ticket);
		$confirmation = \XF::phrase('your_message_has_been_posted');

		return $this->redirect($this->buildLink('tickets/messages', $message), $confirmation);
	}

	/**
	 * @param \NF\Tickets\Entity\Ticket $ticket
	 *
	 * @return \NF\Tickets\Service\Ticket\Replier
	 */
	protected function setupTicketReply(\NF\Tickets\Entity\Ticket $ticket)
	{
		/** @var \XF\ControllerPlugin\Editor $editorPlugin */
		$editorPlugin = $this->plugin('XF:Editor');
		$message = $editorPlugin->fromInput('message');

		/** @var \NF\Tickets\Service\Ticket\Replier $replier */
		$replier = $this->service('NF\Tickets:Ticket\Replier', $ticket);

		$replier->setMessage($message);

		$visitor = \XF::visitor();
		if (!$visitor->user_id)
		{
			$name = $this->filter('user_name', 'str');
			$email = $this->filter('user_email', 'str');
			$replier->setGuestUser($name, $email);
		}

		if ($ticket->canActionAsAnonymized())
		{
			$anonymize = $this->filter('anonymized', 'bool');
			if ($anonymize)
			{
				$replier->setAnonymize(true);
			}
		}

		if ($ticket->Category->canUploadAndManageAttachments())
		{
			$replier->setAttachmentHash($this->filter('attachment_hash', 'str'));
		}

		if ($ticket->canUpdateStatus())
		{
			$updateState = $this->filter('reply_ticket_state', 'int');
			if ($updateState)
			{
				/** @var \NF\Tickets\Entity\Status $status */
				$status = $this->app()->find('NF\Tickets:Status', $updateState);

				if ($status && $status->exists() && $status->active)
				{
					$ticket->set('status_id', $status->status_id);
				}
			}
		}

		if ($ticket->canReassign() && $this->filter('self_assign', 'bool'))
		{
			$replier->selfAssign();
		}

		$replier->setUserAgent($this->request()->getUserAgent());

		return $replier;
	}

	/**
	 * @param \NF\Tickets\Entity\Ticket $ticket
	 * @param                           $lastDate
	 *
	 * @return View
	 */
	protected function getNewMessagesReply(\NF\Tickets\Entity\Ticket $ticket, $lastDate)
	{
		$messageRepo = $this->getMessageRepo();

		$limit = 3;

        $messageList = $messageRepo->findNewestMessagesInTicket($ticket, $lastDate)->skipIgnored()->forFullView();
		$messages = $messageList->fetch($limit + 1);

		// We fetched one more post than needed, if more than $limit posts were returned,
		// we can show the 'there are more posts' notice
		if ($messages->count() > $limit)
		{
			$firstUnshownMessage = $messages->first();

			// Remove the extra post
			$messages = $messages->pop();
		}
		else
		{
			$firstUnshownMessage = null;
		}

		/** @var \NF\Tickets\Entity\Message $firstUnshownMessage */

		// put the posts into oldest-first order
		$messages = $messages->reverse(true);

		/** @var \XF\Repository\Attachment $attachmentRepo */
		$attachmentRepo = $this->repository('XF:Attachment');
		$attachmentRepo->addAttachmentsToContent($messages, 'nf_tickets_message');

		$visitor = \XF::visitor();
		$ticketRead = $ticket->Read[$visitor->user_id];

		if ($visitor->user_id)
		{
			if (!$firstUnshownMessage || ($ticketRead && $firstUnshownMessage->message_date <= $ticketRead->ticket_read_date))
			{
				$this->getTicketRepo()->markTicketReadByVisitor($ticket);
			}

			/** @var \XF\Repository\UserAlert $userAlertRepo */
			$userAlertRepo = $this->repository('XF:UserAlert');
			$userAlertRepo->markUserAlertsReadForContent('nf_tickets_message', $messages->keys());
		}

		/** @var \NF\Tickets\Entity\Message $lastMessage */
		$lastMessage = $messages->last();
		if ($lastMessage)
		{
			$lastDate = $lastMessage->message_date;
		}

		$categories = $this->getCategoryRepo()->getViewableCategories();

		$viewParams = [
			'ticket' => $ticket,
			'category' => $ticket->Category,
			'messages' => $messages,
			'firstUnshownMessage' => $firstUnshownMessage,

			'categoryTree' => $this->getCategoryRepo()->createCategoryTree($categories),
			'priorities' => $this->getPriorityRepo()->findPrioritiesForList(true)->fetch(),
			'statuses' => $this->getStatusRepo()->findStatusesForList(true)->fetch(),
		];
		$view = $this->view('NF\Tickets:Ticket\NewMessages', 'nf_tickets_ticket_new_messages', $viewParams);
		$view->setJsonParam('lastDate', $lastDate);

		return $view;
	}

	/**
	 * @param \NF\Tickets\Service\Ticket\Replier $replier
	 */
	protected function finalizeTicketReply(\NF\Tickets\Service\Ticket\Replier $replier)
	{
		$replier->sendNotifications();

		$ticket = $replier->getTicket();
		$ticket->draft_message->delete();
		$visitor = \XF::visitor();

		$setOptions = $this->filter('_xfSet', 'array-bool');
		if ($ticket->canWatch())
		{
			/** @var \NF\Tickets\Repository\TicketWatch $ticketWatchRepo */
			$ticketWatchRepo = $this->repository('NF\Tickets:TicketWatch');

			if (isset($setOptions['watch_ticket']))
			{
				$watch = $this->filter('watch_ticket', 'bool');
				if ($watch)
				{
					$config = $this->filter([
						'email_subscribe' => 'bool',
					]);
					$ticketWatchRepo->setWatchState($ticket, $visitor, 'watch', $config);
				}
			}
			else
			{
				// use user preferences
				$ticketWatchRepo->autoWatchTicket($ticket, $visitor, false);
			}
		}

        /** @var User $visitor */
        $visitor = \XF::visitor();
        if ($visitor->canAccessTicketQueue())
        {
            $visitor->updateTicketLastRead();
        }
	}

	public function actionEdit(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);
		if (!$ticket->canEdit($error))
		{
			return $this->noPermission($error);
		}

		$category = $ticket->Category;

		if ($this->isPost())
		{
			$editor = $this->setupTicketEdit($ticket);
			$editor->checkForSpam();

			if (!$editor->validate($errors))
			{
				return $this->error($errors);
			}

			$editor->save();

			return $this->redirect($this->getDynamicRedirect($this->buildLink('tickets', $ticket)));
		}
		else
		{
			if ($category && $category->canUploadAndManageAttachments())
			{
				/** @var \XF\Repository\Attachment $attachmentRepo */
				$attachmentRepo = $this->repository('XF:Attachment');
				$attachmentData = $attachmentRepo->getEditorData('nf_tickets_message', $ticket->FirstMessage);
			}
			else
			{
				$attachmentData = null;
			}

			$prefix = $ticket->Prefix;
			$prefixes = $category->getUsablePrefixes($prefix);

			$viewParams = [
				'ticket' => $ticket,
				'category' => $category,
				'attachmentData' => $attachmentData,

				'priorities' => $this->getPriorityRepo()->findPrioritiesForList(true)->fetch(),
				'statuses' => $this->getStatusRepo()->findStatusesForList(true, $ticket->status_id)->fetch(),
				'prefixes' => $prefixes,
			];

			return $this->view('NF\Tickets:Ticket\Edit', 'nf_tickets_ticket_edit', $viewParams);
		}
	}

	public function actionResolve(ParameterBag $params)
	{
        $resolvedStatusId = (int)(\XF::options()->nftResolvedStatus ?? 0);
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);
		if ($resolvedStatusId !== 0 && !$ticket->canResolve($error))
		{
			return $this->noPermission($error);
		}

		if ($this->isPost())
		{
			$ticket->status_id = $resolvedStatusId;
			$ticket->save();

			return $this->redirect($this->buildLink('tickets', $ticket));
		}
		else
		{
			$viewParams = [
				'ticket' => $ticket,
			];

			return $this->view('NF\Tickets:Ticket\Resolve', 'nf_tickets_ticket_resolve', $viewParams);
		}
	}

	public function actionDelete(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);
		if (!$ticket->canDelete('soft', $error))
		{
			return $this->noPermission($error);
		}

		if ($this->isPost())
		{
			$type = $this->filter('hard_delete', 'bool') ? 'hard' : 'soft';
			$reason = $this->filter('reason', 'str');

			if (!$ticket->canDelete($type, $error))
			{
				return $this->noPermission($error);
			}

			/** @var \NF\Tickets\Service\Ticket\Deleter $deleter */
			$deleter = $this->service('NF\Tickets:Ticket\Deleter', $ticket);

			if ($this->filter('author_alert', 'bool') && $ticket->canSendModeratorActionAlert())
			{
				$deleter->setSendAlert(true, $this->filter('author_alert_reason', 'str'));
			}

			$deleter->delete($type, $reason);

            /** @var \XF\ControllerPlugin\InlineMod $plugin */
            $plugin = $this->plugin('XF:InlineMod');
            $plugin->clearIdFromCookie('nf_tickets', $ticket->ticket_id);

			return $this->redirect(
				$ticket->Category
					? $this->buildLink('tickets/categories', $ticket->Category)
					: $this->buildLink('tickets/queue')
			);
		}
		else
		{
			$viewParams = [
				'ticket' => $ticket,
			];

			return $this->view('NF\Tickets:Ticket\Delete', 'nf_tickets_delete', $viewParams);
		}
	}

	public function actionUndelete(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);

		/** @var \XF\ControllerPlugin\Undelete $plugin */
		$plugin = $this->plugin('XF:Undelete');
		return $plugin->actionUndelete(
			$ticket,
			$this->buildLink('tickets/undelete', $ticket),
			$this->buildLink('tickets', $ticket),
			$ticket->title,
			'discussion_state'
		);
	}

	/**
	 * @param \NF\Tickets\Entity\Ticket $ticket
	 *
	 * @return \NF\Tickets\Service\Ticket\Editor
	 */
	protected function setupTicketEdit(\NF\Tickets\Entity\Ticket $ticket)
	{
		/** @var \NF\Tickets\Service\Ticket\Editor $editor */
		$editor = $this->service('NF\Tickets:Ticket\Editor', $ticket);

		$prefixId = $this->filter('prefix_id', 'uint');
		if ($prefixId !== $ticket->prefix_id && !$ticket->Category->isPrefixUsable($prefixId))
		{
			$prefixId = 0; // not usable, just blank it out
		}
		$editor->setPrefix($prefixId);

        if ($ticket->canEditTitle())
        {
            $title = $this->filter('title', 'str');
            if (strlen($title) !== 0)
            {
                $editor->setTitle($title);
            }
        }

        if ($ticket->canUpdatePriority())
        {
            $priorityId = $this->filter('priority', 'uint');
            $priority = $this->getPriorityRepo()->getActivePriority($priorityId);
            if ($priority)
            {
                $editor->setPriority($priority->priority_id);
            }
        }

        if ($ticket->canUpdateStatus())
        {
            $statusId = $this->filter('status_id', 'uint');
            $status = $this->getStatusRepo()->getActiveStatus($statusId);
            if ($status)
            {
                $editor->setStatus($status->status_id, $status->alert_on_transition);
            }
        }

		if ($ticket->canLockUnlockTicket() && $lock = $this->filter('ticket_locked', 'bool'))
		{
			$editor->setLockState($lock);
		}

		if ($customFields = $this->filter('custom_fields', 'array'))
		{
			$editor->setCustomFields($customFields);
		}

		if ($ticket->canMove())
		{
			$targetCategoryId = $this->filter('target_category_id', 'uint');
			if ($targetCategoryId && $targetCategoryId !== $ticket->ticket_category_id)
			{
				/** @var \NF\Tickets\Entity\Category $targetCategory */
				$targetCategory = $this->em()->find('NF\Tickets:Category', $targetCategoryId);
				$ticketMove = $this->setupTicketMove($ticket, $targetCategory);
				$ticketMove->setSendAlert(true);
				$ticketMove->setNotifyWatchers(true);
				$editor->setMove($ticketMove, $targetCategory);
			}
		}

		return $editor;
	}

	public function actionNewMessages(ParameterBag $params)
	{
		$ticket = $this->assertViewableTicket($params->ticket_id, ['FirstMessage']);

		$after = $this->filter('after', 'uint');

		if (!$this->request->isXhr())
		{
			if (!$after)
			{
				return $this->redirect($this->buildLink('tickets', $ticket));
			}

			$findFirstUnread = $this->getMessageRepo()->findNextMessagesInTicket($ticket, $after);
			$firstMessage = $findFirstUnread->skipIgnored()->fetchOne();

			if (!$firstMessage)
			{
				$firstMessage = $ticket->LastMessage;
			}
            /** @var \NF\Tickets\ControllerPlugin\Ticket $plugin */
            $plugin = $this->plugin('NF\Tickets:Ticket');

			return $this->redirect($plugin->getMessageLink($firstMessage));
		}

		return $this->getNewMessagesReply($ticket, $after);
	}

    public function actionPreview(ParameterBag $params): AbstractReply
    {
        /** @noinspection PhpUndefinedFieldInspection */
        $ticket = $this->assertViewableTicket($params->ticket_id, ['FirstMessage']);
        $firstMessage = $ticket->FirstMessage;

        $viewParams = [
            'ticket'    => $ticket,
            'firstMessage' => $firstMessage,
        ];

        return $this->view('NF\Tickets:Ticket\Preview', 'nf_tickets_ticket_preview', $viewParams);
    }

	public function actionReplyPreview(ParameterBag $params): AbstractReply
	{
		$this->assertPostOnly();

		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);
		if (!$ticket->canReply($error))
		{
			return $this->noPermission($error);
		}

		$replier = $this->setupTicketReply($ticket);
		if (!$replier->validate($errors))
		{
			return $this->error($errors);
		}

		$message = $replier->getMessage();
		$attachments = [];

		$tempHash = $this->filter('attachment_hash', 'str');
		if ($tempHash && $ticket->Category->canUploadAndManageAttachments())
		{
			/** @var \XF\Repository\Attachment $attachRepo */
			$attachRepo = $this->repository('XF:Attachment');
			$attachments = $attachRepo->findAttachmentsByTempHash($tempHash)->fetch();
		}

		/** @var \XF\ControllerPlugin\BbCodePreview $plugin */
		$plugin = $this->plugin('XF:BbCodePreview');

		return $plugin->actionPreview(
			$message->message, 'nf_tickets_message', $message->User, $attachments, $ticket->canViewAttachments()
		);
	}

	public function actionMultiQuote()
	{
		$this->assertPostOnly();

		/** @var \XF\ControllerPlugin\Quote $quotePlugin */
		$quotePlugin = $this->plugin('XF:Quote');

		$quotes = $this->filter('quotes', 'json-array');
		if (!$quotes)
		{
			return $this->error(\XF::phrase('no_messages_selected'));
		}
		$quotes = $quotePlugin->prepareQuotes($quotes);

		$messageFinder = $this->finder('NF\Tickets:Message');

		$messages = $messageFinder
			->with(['User', 'Ticket', 'Ticket.Category'])
			->where('message_id', \array_keys($quotes))
			->order('message_date', 'DESC')
			->fetch()
			->filterViewable();

		if ($this->request->exists('insert'))
		{
			$insertOrder = $this->filter('insert', 'array');

			return $quotePlugin->actionMultiQuote($messages, $insertOrder, $quotes, 'nf_tickets_message');
		}
		else
		{
			$viewParams = [
				'quotes' => $quotes,
				'messages' => $messages,
			];

			return $this->view('NF\Tickets:Ticket\MultiQuote', 'nf_tickets_ticket_multi_quote', $viewParams);
		}
	}

    public function actionDraft(ParameterBag $params)
    {
        $this->assertDraftsEnabled();

        $ticket = $this->assertViewableTicket($params->ticket_id);

		/** @var \XF\ControllerPlugin\Draft $draftPlugin */
		$draftPlugin = $this->plugin('XF:Draft');

		$extraData = $this->filter([
			'attachment_hash' => 'str',
		]);

		$draftAction = $draftPlugin->updateMessageDraft($ticket->draft_message, $extraData);

		$lastDate = $this->filter('last_date', 'uint');
		$lastKnownDate = $this->filter('last_known_date', 'uint');
		$lastKnownDate = \max($lastDate, $lastKnownDate);

		// the check for last date here is to make sure that we have a date we can log in a link
		if ($lastDate)
		{
            $messageList = $this->getMessageRepo()->findNewestMessagesInTicket($ticket, $lastKnownDate)->skipIgnored();
			$hasNewMessage = ($messageList->total() > 0);
		}
		else
		{
			$hasNewMessage = false;
		}

		$viewParams = [
			'ticket' => $ticket,
			'hasNewMessage' => $hasNewMessage,
			'lastDate' => $lastDate,
			'lastKnownDate' => $lastKnownDate,
		];
		$view = $this->view('NF\Tickets:Ticket\Draft', 'nf_tickets_ticket_save_draft', $viewParams);
		$view->setJsonParam('hasNew', $hasNewMessage);
		$draftPlugin->addDraftJsonParams($view, $draftAction);

		return $view;
	}

	protected function setupTicketMove(/** @noinspection PhpUnusedParameterInspection */ \NF\Tickets\Entity\Ticket $ticket, \NF\Tickets\Entity\Category $category)
	{
		$options = $this->filter([
			'notify_watchers' => 'bool',
			'author_alert' => 'bool',
			'author_alert_reason' => 'str',
		]);

		/** @var \NF\Tickets\Service\Ticket\Move $mover */
		$mover = $this->service('NF\Tickets:Ticket\Move', $ticket);

		if ($options['author_alert'])
		{
			$mover->setSendAlert(true, $options['author_alert_reason']);
		}

		if ($options['notify_watchers'])
		{
			$mover->setNotifyWatchers();
		}

		$mover->addExtraSetup(function (/** @noinspection PhpUnusedParameterInspection */ \NF\Tickets\Entity\Ticket $ticket, \NF\Tickets\Entity\Category $category)
		{
			if ($title = $this->filter('title', 'str'))
			{
				$ticket->title = $title;
			}
		});

		return $mover;
	}

	/**
	 * @param ParameterBag $params
	 * @return AbstractReply|\XF\Mvc\Reply\Error|\XF\Mvc\Reply\Redirect|View
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionMove(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);
		if (!$ticket->canMove($error))
		{
			return $this->noPermission($error);
		}

		$category = $ticket->Category;

		if ($this->isPost())
		{
			$targetCategoryId = $this->filter('target_category_id', 'uint');

			/** @var \NF\Tickets\Entity\Category $targetCategory */
			$targetCategory = $this->em()->find('NF\Tickets:Category', $targetCategoryId);
			if (!$targetCategory || !$targetCategory->canView())
			{
				return $this->error(\XF::phrase('requested_category_not_found'));
			}

			$this->setupTicketMove($ticket, $targetCategory)->move($targetCategory);

			return $this->redirect($this->buildLink('tickets', $ticket));
		}
		else
		{
			$categoryRepo = $this->getCategoryRepo();
			$categories = $categoryRepo->getViewableCategories();

			$viewParams = [
				'ticket' => $ticket,
				'category' => $category,
				'categoryTree' => $categoryRepo->createCategoryTree($categories),
			];

			return $this->view('NF\Tickets:Ticket\Move', 'nf_tickets_ticket_move', $viewParams);
		}
	}

	public function actionAssign(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);
		if (!$ticket->canReassign($error))
		{
			return $this->noPermission();
		}

		if (!$ticket->canView())
		{
			return $this->error(\XF::phrase('nf_tickets_you_may_only_reassign_tickets_to_users_with_permission_to_view'));
		}

		/** @var \NF\Tickets\Service\Ticket\Reassign $reassigner */
		$reassigner = $this->service('NF\Tickets:Ticket\Reassign', $ticket);
		$reassigner->reassignTo(\XF::visitor());

		return $this->redirect($this->buildLink('tickets', $ticket));
	}

	public function actionWatch(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);
		if (!$ticket->canWatch($error))
		{
			return $this->noPermission($error);
		}

		$visitor = \XF::visitor();

		if ($this->isPost())
		{
			if ($this->filter('stop', 'bool'))
			{
				$action = 'delete';
				$config = [];
			}
			else
			{
				$action = 'watch';
				$config = [
					'email_subscribe' => $this->filter('email_subscribe', 'bool'),
				];
			}

			/** @var \NF\Tickets\Repository\TicketWatch $watchRepo */
			$watchRepo = $this->repository('NF\Tickets:TicketWatch');
			$watchRepo->setWatchState($ticket, $visitor, $action, $config);

			$redirect = $this->redirect($this->buildLink('tickets', $ticket));
			$redirect->setJsonParam('switchKey', $action === 'delete' ? 'watch' : 'unwatch');

			return $redirect;
		}
		else
		{
			$viewParams = [
				'ticket' => $ticket,
				'isWatched' => !empty($ticket->Watch[$visitor->user_id]),
				'category' => $ticket->Category,
			];

			return $this->view('NF\Tickets:Ticket\Watch', 'nf_tickets_ticket_watch', $viewParams);
		}
	}

	public function actionReassign(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);
		if (!$ticket->canReassign($error))
		{
			return $this->noPermission($error);
		}

		if ($this->isPost())
		{
			/** @var \NF\Tickets\Service\Ticket\Reassign $reassigner */
			$reassigner = $this->service('NF\Tickets:Ticket\Reassign', $ticket);

			if ($newUsername = $this->filter('username', 'str'))
			{
				/** @var \XF\Entity\User $user */
				$user = $this->em()->findOne('XF:User', ['username' => $newUsername]);
				if (!$user)
				{
					return $this->error(\XF::phrase('requested_user_not_found'));
				}

				$canTargetView = \XF::asVisitor($user, function () use ($ticket)
				{
					return $ticket->canView();
				});
				if (!$canTargetView)
				{
					return $this->error(\XF::phrase('nf_tickets_you_may_only_reassign_tickets_to_users_with_permission_to_view'));
				}

				$reassigner->reassignTo($user);
			}
			else
			{
				$reassigner->unassign();
			}

			return $this->redirect($this->buildLink('tickets', $ticket));
		}
		else
		{
			$viewParams = [
				'ticket' => $ticket,
				'category' => $ticket->Category,
			];

			return $this->view('NF\Tickets:Ticket\Reassign', 'nf_tickets_ticket_reassign', $viewParams);
		}
	}

	public function actionQuickClose(ParameterBag $params)
	{
		$this->assertPostOnly();

		$ticket = $this->assertViewableTicket($params->ticket_id);
		if (!$ticket->canLockUnlockTicket($error))
		{
			return $this->error($error);
		}

		if ($ticket->ticket_locked)
		{
			$ticket->ticket_locked = false;
		}
		else
		{
			$ticket->ticket_locked = true;
		}

		$ticket->save();

		$reply = $this->redirect($this->getDynamicRedirect());
		$reply->setJsonParams([
			'switchKey' => ($ticket->ticket_locked ? 'unlock' : 'lock'),
		]);

		return $reply;
	}

	public function actionChangeThread(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);
		if (!$ticket->canChangeDiscussionThread($error))
		{
			return $this->noPermission($error);
		}

		if ($this->isPost())
		{
			/** @var \NF\Tickets\Service\Ticket\ChangeDiscussion $changer */
			$changer = $this->service('NF\Tickets:Ticket\ChangeDiscussion', $ticket);

			$threadAction = $this->filter('thread_action', 'str');
			if ($threadAction === 'disconnect')
			{
				$changer->disconnectDiscussion();
			}
			else
			{
				$threadUrl = $this->filter('thread_url', 'str');

				if (!$changer->changeThreadByUrl($threadUrl, true, $error))
				{
					return $this->error($error);
				}
			}

			return $this->redirect($this->buildLink('tickets', $ticket));
		}
		else
		{
			$viewParams = [
				'ticket' => $ticket,
				'category' => $ticket->Category,
			];

			return $this->view('NF\Tickets:Ticket\ChangeThread', 'nf_tickets_change_thread', $viewParams);
		}
	}

	public function actionModeratorActions(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);
		if (!$ticket->canViewModeratorLogs($error))
		{
			return $this->noPermission($error);
		}

		$breadcrumbs = $ticket->getBreadcrumbs();
		$title = $ticket->title;

		/** @noinspection PhpUndefinedFieldInspection */
		$this->request()->set('page', $params->page);

		/** @var \XF\ControllerPlugin\ModeratorLog $modLogPlugin */
		$modLogPlugin = $this->plugin('XF:ModeratorLog');

		return $modLogPlugin->actionModeratorActions(
			$ticket,
			['tickets/moderator-actions', $ticket],
			$title, $breadcrumbs
		);
	}

	public function actionInvite(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);

		if (!$ticket->canInvite())
		{
			return $this->noPermission();
		}

		if ($this->isPost())
		{
			/** @var \NF\Tickets\Service\Ticket\Invite $inviter */
			$inviter = $this->service('NF\Tickets:Ticket\Invite', $ticket, \XF::visitor());

			$invite = $this->filter('invite', 'array-str');
			switch ($invite['type'])
			{
				case 'member':
					$inviter->setInvitees($invite['usernames']);
					break;
				case 'guest':
					$inviter->inviteGuest($invite['email'], $invite['username']);
					break;
			}
			if ($ticket->canActionAsAnonymized())
			{
				$anonymize = $this->filter('anonymized', 'bool');
				if ($anonymize)
				{
					$inviter->setAnonymize(true);
				}
			}

			if (!$inviter->validate($errors))
			{
				return $this->error($errors);
			}

			$inviter->save();
			$inviter->sendNotifications();

			return $this->redirect($this->buildLink('tickets', $ticket));
		}
		else
		{
			$viewParams = [
				'ticket' => $ticket,
			];

			return $this->view('NF\Tickets:Ticket\Invite', 'nf_tickets_ticket_invite', $viewParams);
		}
	}

	public function actionInviteGuest(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);

		if (!$ticket->canInvite())
		{
			return $this->noPermission();
		}

		if ($this->isPost())
		{
			$input = $this->filter([
				'email' => 'str',
				'username' => 'str',
			]);

			/** @var \NF\Tickets\Service\Ticket\Invite $inviter */
			$inviter = $this->service('NF\Tickets:Ticket\Invite', $ticket, \XF::visitor());
			$inviter->inviteGuest($input['email'], $input['username']);
			if ($ticket->canActionAsAnonymized())
			{
				$anonymize = $this->filter('anonymized', 'bool');
				if ($anonymize)
				{
					$inviter->setAnonymize(true);
				}
			}
			if (!$inviter->validate($errors))
			{
				return $this->error($errors);
			}

			$inviter->save();
			$inviter->sendNotifications();

			return $this->redirect($this->buildLink('tickets', $ticket));
		}
		else
		{
			$viewParams = [
				'ticket' => $ticket,
			];

			return $this->view('NF\Tickets:Ticket\InviteGuest', 'nf_tickets_ticket_invite_guest', $viewParams);
		}
	}

	public function actionDeinvite(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);

		if (!$ticket->canInvite())
		{
			return $this->noPermission();
		}

		$userId = $this->filter('user_id', 'uint');

		if (!$ticket->Participants[$userId])
		{
			return $this->notFound();
		}

		if ($this->isPost())
		{
			$ticket->Participants[$userId]->delete();

			return $this->redirect($this->buildLink('tickets', $ticket));
		}
		else
		{
			$viewParams = [
				'ticket' => $ticket,
				'user' => $ticket->Participants[$userId]->User,
			];

			return $this->view('NF\Tickets:Ticket\DeInvite', 'nf_tickets_ticket_deinvite', $viewParams);
		}
	}

	public function actionMarkRead()
	{
		$visitor = \XF::visitor();
		if (!$visitor->user_id)
		{
			return $this->noPermission();
		}

		$markDate = $this->filter('date', 'uint');
		if (!$markDate)
		{
			$markDate = \XF::$time;
		}

		if ($this->isPost())
		{
			$categoryRepo = $this->getCategoryRepo();
			$categoryList = $categoryRepo->getViewableCategories();
			$categoryRepo->markCategoriesReadByVisitor($categoryList, $markDate);

			return $this->redirect(
				$this->getDynamicRedirect('tickets'),
				\XF::phrase('nf_tickets_all_tickets_marked_as_read')
			);
		}
		else
		{
			$viewParams = [
				'date' => $markDate,
			];

			return $this->view('NF\Tickets:Ticket\MarkRead', 'nf_tickets_tickets_mark_read', $viewParams);
		}
	}

	public function actionApproveUser(ParameterBag $params)
	{
        if (!\XF::visitor()->canApproveRejectUser())
        {
            return $this->noPermission();
        }

		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);

		if (!$ticket->canManage($error))
		{
			return $this->noPermission($error);
		}

		/** @var \XF\Repository\ApprovalQueue $approvalQueueRepo */
		$approvalQueueRepo = $this->app->repository('XF:ApprovalQueue');
		$handler = $approvalQueueRepo->getApprovalQueueHandler('user', true);

		$user = $ticket->User;
		if (!$user
			|| !$approvalQueueRepo->isContentAwaitingApproval('user', $user->user_id)
		)
		{
			return $this->noPermission();
		}

        $resolvedStatusId = (int)(\XF::options()->nftResolvedStatus ?? 0);
		if ($this->isPost())
		{
			$handler->setInput($this->request()->getInput());

			$handler->performAction('approve', $user);

            $resolveTicket = $this->filter('resolveTicket', 'bool');
            if ($resolveTicket)
            {
                $this->changeTicketStatusOnApprovalQueue($ticket, $resolvedStatusId);
            }

			return $this->redirect($this->buildLink('tickets', $ticket));
		}

		$viewParams = [
            'newStatus' => $this->getStatusRepo()->getActiveStatus($resolvedStatusId),
			'ticket' => $ticket,
			'user' => $user
		];

		return $this->view('NF\Tickets:Ticket\ApproveUser', 'nf_tickets_ticket_approve_user', $viewParams);
	}

	public function actionRejectUser(ParameterBag $params)
	{
        if (!\XF::visitor()->canApproveRejectUser())
        {
            return $this->noPermission();
        }

		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);

		if (!$ticket->canManage($error))
		{
			return $this->noPermission($error);
		}

		/** @var \XF\Repository\ApprovalQueue $approvalQueueRepo */
		$approvalQueueRepo = $this->app->repository('XF:ApprovalQueue');
		$handler = $approvalQueueRepo->getApprovalQueueHandler('user', true);

		$user = $ticket->User;
		if (!$user
			|| !$approvalQueueRepo->isContentAwaitingApproval('user', $user->user_id)
		)
		{
			return $this->noPermission();
		}

        $deletedStatusId = (int)(\XF::options()->nftDeletedStatus ?? 0);
		if ($this->isPost())
		{
			$handler->setInput($this->request()->getInput());

			$handler->performAction('reject', $user);

            $resolveTicket = $this->filter('resolveTicket', 'bool');
            if ($resolveTicket)
            {
                $this->changeTicketStatusOnApprovalQueue($ticket, $deletedStatusId);
            }

			return $this->redirect($this->buildLink('tickets', $ticket));
		}

		$viewParams = [
            'newStatus' => $this->getStatusRepo()->getActiveStatus($deletedStatusId),
			'ticket' => $ticket,
			'user' => $user
		];

		return $this->view('NF\Tickets:Ticket\RejectUser', 'nf_tickets_ticket_reject_user', $viewParams);
	}

    protected function changeTicketStatusOnApprovalQueue(\NF\Tickets\Entity\Ticket $ticket, int $statusId) : void
    {
        /** @var \NF\Tickets\Service\Ticket\Editor $editor */
        $editor = $this->service('NF\Tickets:Ticket\Editor', $ticket);
        $status = $this->getStatusRepo()->getActiveStatus($statusId);
        if ($status !== null)
        {
            $editor->setPerformValidations(false);
            $editor->setStatus($status->status_id, $status->alert_on_transition);
            if ($editor->validate($errors))
            {
                $editor->save();
            }
        }
    }

    /**
     * @version 2.2.6
     *
     * @return View
     *
     * @throws \XF\Mvc\Reply\Exception
     */
	public function actionResponse()
	{
		$responseId = $this->filter('response_id', 'uint');

		/** @var \NF\Tickets\Entity\Response $response */
		$response = $this->assertRecordExists('NF\Tickets:Response', $responseId);
		$message = $response->message;

		if ($ticketId = $this->filter('ticket_id', 'uint'))
		{
			/** @var \NF\Tickets\Entity\Ticket $ticket */
			$ticket = $this->assertRecordExists('NF\Tickets:Ticket', $ticketId);
			$replace = $this->getResponseRepo()->getReplacementValues($ticket);

			$message = \strtr($message, $replace);
		}
		else
		{
            /** @var \NF\Tickets\Entity\Ticket $ticket **/
			$ticket = $this->em()->create('NF\Tickets:Ticket');
		}

		$viewParams = [
			'ticket' => $ticket,
			'response' => $response,
			'message' => $message,
		];

		return $this->view('NF\Tickets:Response\Get', '', $viewParams);
	}

	/**
	 * @param \NF\Tickets\Entity\Ticket $ticket
	 * @param \XF\Entity\Forum $destination
	 *
	 * @return \NF\Tickets\Service\Ticket\Converter\ToThread
	 */
	protected function setupTicketThreadConverter(\NF\Tickets\Entity\Ticket $ticket, \XF\Entity\Forum $destination)
	{
		/** @var \NF\Tickets\Service\Ticket\Converter\ToThread $converter */
		$converter = $this->service('NF\Tickets:Ticket\Converter\ToThread', $ticket, $destination);

		return $converter;
	}

	public function actionConvertToThread(ParameterBag $params)
	{
		/** @noinspection PhpUndefinedFieldInspection */
		$ticket = $this->assertViewableTicket($params->ticket_id);
		if (!$ticket->canConvertToThread($error))
		{
			return $this->noPermission($error);
		}

		if ($this->isPost())
		{
			$nodeId = $this->filter('destination_node_id', 'uint');

			/** @var \XF\Entity\Forum $forum */
			$forum = $this->em()->find('XF:Forum', $nodeId);

			if (!$forum)
			{
				return $this->notFound();
			}

			if (!$forum->canCreateThread())
			{
				return $this->noPermission();
			}

			$converter = $this->setupTicketThreadConverter($ticket, $forum);

			if (!$converter->validate($errors))
			{
				return $this->error($errors);
			}

			$converter->save();
			$converter->sendNotifications();

			$converter->deleteTicket();

			$thread = $converter->getThread();

			return $this->redirect($this->buildLink('threads', $thread));
		}
		else
		{
		    /** @var \XF\Repository\Node $nodeRepo */
			$nodeRepo = $this->repository('XF:Node');
			$nodeList = $nodeRepo->getNodeOptionsData(false, ['Forum'], null, true);

			$viewParams = [
				'ticket' => $ticket,

				'nodeList' => $nodeList,
			];

			return $this->view('NF\Tickets:Ticket\ConvertToThread', 'nf_tickets_convert_to_thread', $viewParams);
		}
	}

	/**
	 * @param int   $ticketId
	 * @param array $extraWith
	 *
	 * @return \NF\Tickets\Entity\Ticket
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertTicketExists(int $ticketId, array $extraWith = []): \NF\Tickets\Entity\Ticket
    {
		$visitor = \XF::visitor();

		$extraWith[] = 'User';
		$extraWith[] = 'Category';
		$extraWith[] = 'Category.AnonymousUser';
		$extraWith[] = 'Category.Permissions|' . $visitor->permission_combination_id;
		$extraWith[] = 'Thread';
		$extraWith[] = 'Thread.Forum';
		$extraWith[] = 'Thread.Forum.Node';
		$extraWith[] = 'Thread.Forum.Node.Permissions|' . $visitor->permission_combination_id;

		$userId = $visitor->user_id;
		if ($userId)
		{
			$extraWith[] = 'Category.Watch|' . $userId;
			$extraWith[] = 'Category.Read|' . $userId;
			$extraWith[] = 'Watch|' . $userId;
			$extraWith[] = 'Read|' . $userId;
		}

        /** @noinspection PhpIncompatibleReturnTypeInspection */
        return $this->assertRecordExists('NF\Tickets:Ticket', $ticketId, $extraWith, 'nf_tickets_requested_ticket_not_found');
	}

	/**
	 * @param int  $ticketId
	 * @param array $extraWith
	 *
	 * @return \NF\Tickets\Entity\Ticket
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertViewableTicket(int $ticketId, array $extraWith = []): \NF\Tickets\Entity\Ticket
    {
		$ticket = $this->assertTicketExists($ticketId, $extraWith);

		if (!$ticket->canView($error))
		{
			if (!\XF::visitor()->user_id && $ticket->viewableToGuests())
			{
				// Guest - show guest password entry

				$viewParams = [
					'ticket' => $ticket,
				];

				throw $this->exception($this->view('NF\Tickets:Ticket\Password', 'nf_tickets_ticket_password_entry', $viewParams));
			}

			throw $this->exception($this->noPermission($error));
		}

		return $ticket;
	}

	/**
	 * @param \NF\Tickets\Entity\Ticket $ticket
	 * @param null $forceAttachmentHash
	 *
	 * @return array|null
	 */
	protected function getReplyAttachmentData(\NF\Tickets\Entity\Ticket $ticket, $forceAttachmentHash = null)
	{
		$category = $ticket->Category;

		if ($category && $category->canUploadAndManageAttachments())
		{
			if ($forceAttachmentHash !== null)
			{
				$attachmentHash = $forceAttachmentHash;
			}
			else
			{
				/** @noinspection PhpUndefinedFieldInspection */
				$attachmentHash = $ticket->draft_message->attachment_hash;
			}

			/** @var \XF\Repository\Attachment $attachmentRepo */
			$attachmentRepo = $this->repository('XF:Attachment');

			return $attachmentRepo->getEditorData('nf_tickets_message', $ticket, $attachmentHash);
		}

		return null;
	}

	public static function getActivityDetails(array $activities)
	{
		return self::getActivityDetailsForContent(
			$activities, \XF::phrase('nf_tickets_viewing_ticket'), 'ticket_id',
			function (array $ids)
			{
				$tickets = \XF::em()->findByIds(
					'NF\Tickets:Ticket',
					$ids,
					['Category', 'Category.Permissions|' . \XF::visitor()->permission_combination_id]
				);

				$router = \XF::app()->router('public');
				$data = [];

				foreach ($tickets->filterViewable() AS $id => $ticket)
				{
					$data[$id] = [
						'title' => $ticket->title,
						'url' => $router->buildLink('tickets', $ticket),
					];
				}

				return $data;
			},
			\XF::phrase('nf_tickets_viewing_tickets')
		);
	}
}
