<?php

namespace NF\Tickets\Report;

use XF\Entity\Report;
use XF\Mvc\Entity\Entity;
use XF\Report\AbstractHandler;

class Message extends AbstractHandler
{
    protected function canActionContent(Report $report)
    {
        $visitor = \XF::visitor();

        return $visitor->hasPermission('general', 'warn'); // TODO: update
    }

    public function setupReportEntityContent(Report $report, Entity $content)
    {
        /** @var \NF\Tickets\Entity\Message $content */
        $ticket = $content->Ticket;
        $report->content_user_id = $content->user_id;
        $report->content_info = [
            'message'        => $content->message,
            'message_id'     => $content->message_id,
            'category_id'    => $ticket->Category->ticket_category_id,
            'category_title' => $ticket->Category->title,
            'ticket_id'      => $content->ticket_id,
            'ticket_title'   => $ticket->title,
            'user_id'        => $content->user_id,
            'username'       => $content->username,
        ];
    }

    public function getContentTitle(Report $report)
    {
        $title = $report->content_info['ticket_title'];

        return \XF::phrase('nf_tickets_message_in_ticket_x', [
            'title' => \XF::app()->stringFormatter()->censorText($title),
        ]);
    }

    public function getContentMessage(Report $report)
    {
        return $report->content_info['message'];
    }

    public function getContentLink(Report $report)
    {
        return \XF::app()->router('public')->buildLink('canonical:ticket-message', $report->content_info);
    }

    public function getEntityWith()
    {
        return ['Ticket', 'Ticket.User', 'User'];
    }
}