<?php

namespace NF\Tickets\Repository;

use NF\Tickets\Finder\BanEmail as BanEmailFinder;
use XF\Mvc\Entity\Finder;
use XF\Mvc\Entity\Repository;

class Banning extends Repository
{
    /**
     * @return BanEmailFinder|Finder
     */
    public function findEmailBans(): BanEmailFinder
    {
        return $this->finder('NF\Tickets:BanEmail')
                    ->setDefaultOrder('banned_email', 'asc');
    }

    public function banEmail(string $email, bool $shadowBan = true, string $reason = '', \XF\Entity\User $user = null): bool
    {
        $user = $user ?: \XF::visitor();

        /** @var \NF\Tickets\Entity\BanEmail $emailBan */
        $emailBan = $this->em->create('NF\Tickets:BanEmail');
        $emailBan->banned_email = $email;
        $emailBan->reason = $reason;
        $emailBan->create_user_id = $user->user_id;
        $emailBan->shadow_ban = $shadowBan;

        return $emailBan->save();
    }

    public function getShadowBanReference(string $email, bool $isShadowBanned = null): array
    {
        $finder = $this->findEmailBans();
        if ($isShadowBanned !== null)
        {
            $finder->isShadowBanned($isShadowBanned);
        }
        $shadowBannedEmails = $finder->fetchColumns(['banned_email', 'shadow_ban']);
        foreach ($shadowBannedEmails as $row)
        {
            $bannedEmail = $rawValue = $row['banned_email'];
            $bannedEmail = \str_replace('\\*', '(.*)', \preg_quote($bannedEmail, '/'));
            if (\preg_match('/^' . $bannedEmail . '$/i', $email))
            {
                \XF::db()->query('UPDATE xf_nf_tickets_ban_email SET last_triggered_date = UNIX_TIMESTAMP() WHERE banned_email = ?', $rawValue);

                return [$rawValue, $row['shadow_ban']];
            }
        }

        return [];
    }

    public function isBanned(string $email, bool $isShadowBanned): bool
    {
        $result = $this->getShadowBanReference($email, $isShadowBanned);

        return (bool)$result;
    }
}