<?php

namespace NF\Tickets\Repository;

use XF\Repository\AbstractCategoryTree;

/**
 * Class Category
 *
 * @package NF\Tickets\Repository
 */
class Category extends AbstractCategoryTree
{
    /**
     * @return string
     */
    protected function getClassName()
    {
        return 'NF\Tickets:Category';
    }

    /**
     * @param array $extras
     * @param array $childExtras
     * @return array
     */
    public function mergeCategoryListExtras(array $extras, array $childExtras)
    {
        $output = \array_merge([
            'ticket_count'  => 0,
            'message_count' => 0,
            'childCount'    => 0,
        ], $extras);

        foreach ($childExtras AS $child)
        {
            if (!empty($child['ticket_count']))
            {
                $output['ticket_count'] += $child['ticket_count'];
            }

            if (!empty($child['message_count']))
            {
                $output['message_count'] += $child['message_count'];
            }

            $output['childCount'] += 1 + (!empty($child['childCount']) ? $child['childCount'] : 0);
        }

        return $output;
    }

    public function markCategoriesReadByVisitor($categories = null, $newRead = null)
    {
        if (!$categories)
        {
            $categories = $this->getViewableCategories();
        }

        foreach ($categories AS $category)
        {
            $this->markCategoryReadByVisitor($category, $newRead);
        }
    }

    /**
     * @param \NF\Tickets\Entity\Category $category
     * @param null                        $newRead
     * @return bool
     */
    public function markCategoryReadByVisitor(\NF\Tickets\Entity\Category $category, $newRead = null)
    {
        $userId = \XF::visitor()->user_id;
        if (!$userId)
        {
            return false;
        }

        if ($newRead === null)
        {
            $newRead = \max(\XF::$time, $category->last_ticket);
        }

        $cutOff = \XF::$time - $this->options()->readMarkingDataLifetime * 86400;
        if ($newRead <= $cutOff)
        {
            return false;
        }

        $read = $category->Read[$userId];
        if ($read && $newRead <= $read->category_read_date)
        {
            return false;
        }

        $this->db()->insert('xf_nf_tickets_category_read', [
            'ticket_category_id' => $category->ticket_category_id,
            'user_id'            => $userId,
            'category_read_date' => $newRead,
        ], false, 'category_read_date = VALUES(category_read_date)');

        return true;
    }

    public function getQueueCategories()
    {
        $categoryFinder = $this->finder('NF\Tickets:Category');

        $categories = $categoryFinder->fetch();
        $categories = $categories->filterViewable();

        return $categories;
    }

    public function getDefaultTicketSort()
    {
        // todo: make cat option
        return ['last_message_date', 'desc'];
    }
}