<?php

namespace NF\Tickets\Repository;

use XF\Mvc\Entity\Repository;

class CategoryWatch extends Repository
{
    public function setWatchState(\NF\Tickets\Entity\Category $category, \XF\Entity\User $user, $action, array $config = [])
    {
        if (!$category->ticket_category_id || !$user->user_id)
        {
            throw new \InvalidArgumentException("Invalid category or user");
        }
        try
        {
            /** @var \NF\Tickets\Entity\CategoryWatch $watch */
            $watch = $this->em->find('NF\Tickets:CategoryWatch', [
                'ticket_category_id' => $category->ticket_category_id,
                'user_id'            => $user->user_id,
            ]);

            switch ($action)
            {
                case 'delete':
                    if ($watch)
                    {
                        $watch->delete();
                    }
                    break;

                case 'watch':
                    if (!$watch)
                    {
                        /** @var \NF\Tickets\Entity\CategoryWatch $watch */
                        $watch = $this->em->create('NF\Tickets:CategoryWatch');
                        $watch->ticket_category_id = $category->ticket_category_id;
                        $watch->user_id = $user->user_id;
                    }
                    unset($config['ticket_category_id'], $config['user_id']);

                    $watch->bulkSet($config);
                    $watch->save();
                    break;

                case 'update':
                    if ($watch)
                    {
                        unset($config['ticket_category_id'], $config['user_id']);

                        $watch->bulkSet($config);
                        $watch->save();
                    }
                    break;

                default:
                    throw new \InvalidArgumentException("Unknown action '$action' (expected: delete/watch/update)");
            }
        }
            /** @noinspection PhpRedundantCatchClauseInspection */
        catch (\XF\Db\DuplicateKeyException $e)
        {
            return null;
        }
    }

    public function setWatchStateForAll(\XF\Entity\User $user, $action, array $updates = [])
    {
        if (!$user->user_id)
        {
            throw new \InvalidArgumentException("Invalid user");
        }

        $db = $this->db();

        switch ($action)
        {
            case 'update':
                unset($updates['ticket_category_id'], $updates['user_id']);

                return $db->update('xf_nf_tickets_category_watch', $updates, 'user_id = ?', $user->user_id);

            case 'delete':
                return $db->delete('xf_nf_tickets_category_watch', 'user_id = ?', $user->user_id);

            default:
                throw new \InvalidArgumentException("Unknown action '$action'");
        }
    }
}