<?php

namespace NF\Tickets\Repository;

use XF\Mvc\Entity\Repository;

/**
 * Class Message
 *
 * @package NF\Tickets\Repository
 */
class Message extends Repository
{
    /**
     * @param \NF\Tickets\Entity\Ticket $ticket
     * @param array                     $limits
     * @return \NF\Tickets\Finder\Message
     */
    public function findMessagesForTicketView(\NF\Tickets\Entity\Ticket $ticket, array $limits = [])
    {
        /** @var \NF\Tickets\Finder\Message $finder */
        $finder = $this->finder('NF\Tickets:Message');
        $finder
            ->inTicket($ticket, $limits)
            ->orderByDate()
            ->forFullView();

        return $finder;
    }

    /**
     * @param \NF\Tickets\Entity\Ticket $ticket
     * @param                           $newerThan
     * @param array                     $limits
     * @return \NF\Tickets\Finder\Message
     */
    public function findNewestMessagesInTicket(\NF\Tickets\Entity\Ticket $ticket, $newerThan, array $limits = [])
    {
        /** @var \NF\Tickets\Finder\Message $finder */
        $finder = $this->finder('NF\Tickets:Message');
        $finder
            ->orderByDate('DESC')
            ->inTicket($ticket, $limits)
            ->newerThan($newerThan);

        return $finder;
    }

    public function findNextMessagesInTicket(\NF\Tickets\Entity\Ticket $ticket, $newerThan, array $limits = [])
    {
        /** @var \NF\Tickets\Finder\Message $finder */
        $finder = $this->finder('NF\Tickets:Message');
        $finder
            ->inTicket($ticket, $limits)
            ->orderByDate()
            ->newerThan($newerThan);

        return $finder;
    }

    public function sendModeratorActionAlert(\NF\Tickets\Entity\Message $message, $action, $reason = '', array $extra = [], \XF\Entity\User $forceUser = null)
    {
        $ticket = $message->Ticket;
        if (!$forceUser)
        {
            if (!$message->user_id || !$message->User)
            {
                return false;
            }

            $forceUser = $message->User;
        }

        $extra = \array_merge([
            'title'  => $ticket->title,
            'link'   => $this->app()->router('public')->buildLink('nopath:tickets', $ticket),
            'reason' => $reason,
            ['depends_on_addon_id' => 'NF/Tickets'],
        ], $extra);

        /** @var \XF\Repository\UserAlert $alertRepo */
        $alertRepo = $this->repository('XF:UserAlert');
        $alertRepo->alert(
            $forceUser,
            0, '',
            'user', $forceUser->user_id,
            "nf_tickets_message_{$action}", $extra
        );

        return true;
    }
}