<?php

namespace NF\Tickets\Repository;

use XF\Mvc\Entity\Repository;

class QueueWatch extends Repository
{
	public function setWatchState(\XF\Entity\User $user, $action, array $config = [])
	{
		if (!$user->user_id)
		{
			throw new \InvalidArgumentException("Invalid user set");
		}

		try
		{
			/** @var \NF\Tickets\Entity\QueueWatch $watch */
			$watch = $this->em->find('NF\Tickets:QueueWatch', [
				'user_id' => $user->user_id,
			]);

			switch ($action)
			{
				case 'delete':
					if ($watch)
					{
						$watch->delete();
					}
					break;

				case 'watch':
					if (!$watch)
					{
						/** @var \NF\Tickets\Entity\QueueWatch $watch */
						$watch = $this->em->create('NF\Tickets:QueueWatch');
						$watch->user_id = $user->user_id;
					}

					unset($config['user_id']);

					$watch->bulkSet($config);
					$watch->save();
					break;

				case 'update':
					if ($watch)
					{
						unset($config['user_id']);

						$watch->bulkSet($config);
						$watch->save();
					}
					break;

				default:
					throw new \InvalidArgumentException("Unknown action '$action' (expected: delete/watch/update)");
			}
		}
		/** @noinspection PhpRedundantCatchClauseInspection */
		catch (\XF\Db\DuplicateKeyException $e)
		{
			return null;
		}
	}

	public function setWatchStateForAll(\XF\Entity\User $user, $action, array $updates = []): int
	{
		if (!$user->user_id)
		{
			throw new \InvalidArgumentException("Invalid user");
		}

		$db = $this->db();

		switch ($action)
		{
			case 'update':
				unset($updates['user_id']);

				return $db->update('xf_nf_tickets_queue_watch', $updates, 'user_id = ?', $user->user_id);

			case 'delete':
				return $db->delete('xf_nf_tickets_queue_watch', 'user_id = ?', $user->user_id);

			default:
				throw new \InvalidArgumentException("Unknown action '$action'");
		}
	}
}