<?php

namespace NF\Tickets\Repository;

use XF\Mvc\Entity\Repository;

use function count, is_array;

class Response extends Repository
{
    public function findResponsesCategorized($getDefault = false)
    {
        $categories = $this->finder('NF\Tickets:ResponseCategory')
                           ->order('display_order');

        if ($getDefault)
        {
            $categories = $categories->fetch();

            $defaultCategory = $this->getDefaultCategory();
            $cats = $categories->toArray();
            $cats = $cats + [$defaultCategory];
            $categories = $this->em->getBasicCollection($cats);
        }

        return $categories;
    }

    public function getDefaultCategory()
    {
        /** @var \NF\Tickets\Entity\ResponseCategory $responseCategory */
        $responseCategory = $this->em->create('NF\Tickets:ResponseCategory');
        $responseCategory->setTrusted('response_category_id', 0);
        $responseCategory->setTrusted('display_order', 0);
        $responseCategory->setReadOnly(true);

        return $responseCategory;
    }

    public function getResponseListData()
    {
        $responses = $this->finder('NF\Tickets:Response')->order('display_order')->fetch();
        $responseCategories = $this->findResponsesCategorized(true);

        return [
            'responseCategories'   => $responseCategories,
            'responsesCategorized' => $responses->groupBy('response_category_id'),
            'responseTotal'        => count($responses),
        ];
    }

    public function getReplacementValues(\NF\Tickets\Entity\Ticket $ticket)
    {
        $visitor = \XF::visitor();

        $replaceFields = [];

        if (!empty($fields = $ticket->getCustomFields()->getFieldValues()))
        {
            foreach ($fields AS $id => $value)
            {
                if (is_array($value))
                {
                    foreach ($value AS &$v)
                    {
                        $v = \XF::phrase('nf_tickets_ticket_field_choice.' . $id . '_' . $v);
                    }

                    $replaceFields["{field.$id}"] = \implode(', ', $value);
                }
                else
                {
                    $replaceFields["{field.$id}"] = $value;
                }
            }
        }

        /** @var \XF\Entity\UserGroup $displayGroup */
        $displayGroup = $this->app()->find('XF:UserGroup', $visitor->display_style_group_id);

        $link = $this->app()->router('public')->buildLink('tickets', $ticket);

        if ($ticket->Priority)
        {
            $replaceFields['{priority}'] = $ticket->Priority->title;
        }

        return \array_merge($replaceFields, [
            '{id}'           => $ticket->ticket_ref,
            '{title}'        => $ticket->title,
            '{state}'        => $ticket->Status->title,
            '{name}'         => $ticket->user_name,
            '{email}'        => $ticket->user_email,
            '{assignee}'     => $ticket->assigned_username,
            '{link}'         => $link,
            '{linkAbsolute}' => \XF::convertToAbsoluteUrl($link, $this->app()->options()->boardUrl),

            '{staffName}'  => $visitor->username,
            '{group}'      => $displayGroup ? ($displayGroup->user_title ?: $displayGroup->title) : '',
            '{groupTitle}' => $displayGroup ? $displayGroup->title : '',
        ]);
    }
}