<?php

namespace NF\Tickets\Repository;

use XF\Mvc\Entity\Repository;

class Status extends Repository
{
    public function getActiveStatus(int $statusId): ?\NF\Tickets\Entity\Status
    {
        if ($statusId === 0)
        {
            return null;
        }

        /** @var \NF\Tickets\Entity\Status $status */
        /** @noinspection PhpUnnecessaryLocalVariableInspection */
        $status = $this->findStatusesForList()
                         ->isActive()
                         ->whereId($statusId)
                         ->fetchOne();

        return $status;
    }

    /**
     * @param bool $activeOnly
     * @param \NF\Tickets\Entity\Status|int|null $forceIncludeStatus
     *
     * @return \NF\Tickets\Finder\Status
     */
    public function findStatusesForList($activeOnly = true, $forceIncludeStatus = null)
    {
        /** @var \NF\Tickets\Finder\Status $finder */
        $finder = $this->finder('NF\Tickets:Status')
                       ->setDefaultOrder('display_order');

        if ($activeOnly)
        {
            $finder->isActive(true, $forceIncludeStatus);
        }

        return $finder;
    }

    public function getStatusTitlePairs($includeEmpty = false)
    {
		$data = [];

		if ($includeEmpty)
		{
			$data[0] = '';
		}

		$statuses = $this->finder('NF\Tickets:Status')->order('display_order')->fetch();

		/**
		 * @var int $id
		 * @var \NF\Tickets\Entity\Status $status
		 */
	    foreach ($statuses AS $id => $status)
		{
			$data[$id] = $status->title;
		}

		return $data;
    }

    public function getDefaultDisplayStyles()
    {
        return [
            'label label--hidden',
            'label label--primary',
            'label label--accent',
            'label label--red',
            'label label--green',
            'label label--olive',
            'label label--lightGreen',
            'label label--blue',
            'label label--royalBlue',
            'label label--skyBlue',
            'label label--gray',
            'label label--silver',
            'label label--yellow',
            'label label--orange',
        ];
    }

    public function updateTicketsStatus(\NF\Tickets\Entity\Status $status, \NF\Tickets\Entity\Status $newStatus)
    {
        $this->db()
            ->update('xf_nf_tickets_ticket', ['status_id' => $newStatus->status_id], 'status_id = ?', $status->status_id);
    }
}