<?php

namespace NF\Tickets\Repository;

use XF\Mvc\Entity\Repository;

/**
 * Class TicketWatch
 *
 * @package NF\Tickets\Repository
 */
class TicketWatch extends Repository
{
    /**
     * @param \NF\Tickets\Entity\Ticket $ticket
     * @param \XF\Entity\User           $user
     * @param bool                      $onCreation
     * @return \NF\Tickets\Entity\TicketWatch|null|\XF\Mvc\Entity\Entity
     * @throws \XF\PrintableException
     */
    public function autoWatchTicket(\NF\Tickets\Entity\Ticket $ticket, \XF\Entity\User $user, $onCreation = false)
    {
        $userField = $onCreation ? 'creation_watch_state' : 'interaction_watch_state';

        if (!$ticket->ticket_id || !$user->user_id || !$user->Option->getValue($userField))
        {
            return null;
        }

        $watch = $this->em->find('NF\Tickets:TicketWatch', [
            'ticket_id' => $ticket->ticket_id,
            'user_id'   => $user->user_id,
        ]);
        if ($watch)
        {
            return null;
        }

        /** @var \NF\Tickets\Entity\TicketWatch $watch */
        $watch = $this->em->create('NF\Tickets:TicketWatch');
        $watch->ticket_id = $ticket->ticket_id;
        $watch->user_id = $user->user_id;
        $watch->email_subscribe = ($user->Option->getValue($userField) === 'watch_email');

        try
        {
            $watch->save();
        }
            /** @noinspection PhpRedundantCatchClauseInspection */
        catch (\XF\Db\DuplicateKeyException $e)
        {
            return null;
        }

        return $watch;
    }

    public function setWatchState(\NF\Tickets\Entity\Ticket $ticket, \XF\Entity\User $user, $action, array $config = [])
    {
        if (!$ticket->ticket_id || !$user->user_id)
        {
            throw new \InvalidArgumentException('Invalid ticket or user');
        }
        try
        {
            /** @var \NF\Tickets\Entity\TicketWatch $watch */
            $watch = $this->em->find('NF\Tickets:TicketWatch', [
                'ticket_id' => $ticket->ticket_id,
                'user_id'   => $user->user_id,
            ]);

            switch ($action)
            {
                case 'watch':
                    if (!$watch)
                    {
                        /** @var \NF\Tickets\Entity\TicketWatch $watch */
                        $watch = $this->em->create('NF\Tickets:TicketWatch');
                        $watch->ticket_id = $ticket->ticket_id;
                        $watch->user_id = $user->user_id;
                    }
                    unset($config['ticket_id'], $config['user_id']);

                    $watch->bulkSet($config);
                    $watch->save();
                    break;

                case 'update':
                    if ($watch)
                    {
                        unset($config['ticket_id'], $config['user_id']);

                        $watch->bulkSet($config);
                        $watch->save();
                    }
                    break;

                case 'delete':
                    if ($watch)
                    {
                        $watch->delete();
                    }
                    break;

                default:
                    throw new \InvalidArgumentException("Unknown action '$action' (expected: delete/watch)");
            }
        }
            /** @noinspection PhpRedundantCatchClauseInspection */
        catch (\XF\Db\DuplicateKeyException $e)
        {
            return null;
        }
    }

    public function setWatchStateForAll(\XF\Entity\User $user, $action, array $updates = [])
    {
        if (!$user->user_id)
        {
            throw new \InvalidArgumentException("Invalid user");
        }

        $db = $this->db();

        switch ($action)
        {
            case 'update':
                unset($updates['ticket_id'], $updates['user_id']);

                return $db->update('xf_nf_tickets_ticket_watch', $updates, 'user_id = ?', $user->user_id);

            case 'delete':
                return $db->delete('xf_nf_tickets_ticket_watch', 'user_id = ?', $user->user_id);

            default:
                throw new \InvalidArgumentException("Unknown action '$action'");
        }
    }

    public function isValidWatchState($state)
    {
        switch ($state)
        {
            case 'watch':
            case 'update':
            case 'delete':

            default:
                return false;
        }
    }
}