<?php

namespace NF\Tickets\Search\Data;

use NF\Tickets\XF\Entity\User;
use XF\Mvc\Entity\Entity;
use XF\Search\Data\AbstractData;
use XF\Search\IndexRecord;
use XF\Search\MetadataStructure;
use XF\Search\Query\MetadataConstraint;

class KBArticle extends AbstractData
{
    public function getEntityWith($forView = false)
    {
        $get = ['Category'];
//        if ($forView)
//        {
//            $visitor = \XF::visitor();
//            $get[] = 'Category.Permissions|' . $visitor->permission_combination_id;
//        }

        return $get;
    }

    public function getIndexData(Entity $entity)
    {
        /** @var \NF\Tickets\Entity\KBArticle $entity */
        if (!$entity->Category)
        {
            return null;
        }

        $index = IndexRecord::create('nf_tickets_kb_article', $entity->kb_article_id, [
            'title'         => $entity->title,
            'message'       => $entity->content,
            'date'          => $entity->created_at,
            'metadata'      => $this->getMetaData($entity),
        ]);

        if (!$entity->isVisible())
        {
            $index->setHidden();
        }

        return $index;
    }

    protected function getMetaData(\NF\Tickets\Entity\KBArticle $entity)
    {
        $metadata = [
            'kbcat' => $entity->Category->kb_category_id,
        ];

        return $metadata;
    }

    public function setupMetadataStructure(MetadataStructure $structure)
    {
        $structure->addField('kbcat', MetadataStructure::INT);
    }

    public function getResultDate(Entity $entity)
    {
        /** @var \NF\Tickets\Entity\KBArticle $entity */
        return $entity->created_at;
    }

    public function getTemplateData(Entity $entity, array $options = [])
    {
        /** @var \NF\Tickets\Entity\KBArticle $entity */
        return [
            'article' => $entity,
            'category'  => $entity->Category,
            'options' => $options,
        ];
    }

    public function canUseInlineModeration(Entity $entity, &$error = null)
    {
        return false;
        /** @var \NF\Tickets\Entity\KBArticle $entity */
        //return $entity->canUseInlineModeration($error);
    }

    public function getTypePermissionConstraints(\XF\Search\Query\Query $query, $isOnlyType)
    {
        $categoryRepo = $this->getCategoryRepo();
        $with = [];
        $with[] = 'Permissions|' . \XF::visitor()->permission_combination_id;
        $categories = $categoryRepo->findCategoryList(null, $with)->fetch();

        $skip = [];
        foreach ($categories AS $category)
        {
            /** @var \NF\Tickets\Entity\KBCategory $category */
            if (!$category->canView())
            {
                $skip[] = $category->kb_category_id;
            }
        }

        if ($skip)
        {
            return [
                new MetadataConstraint('kbcat', $skip, MetadataConstraint::MATCH_NONE)
            ];
        }
        else
        {
            return [];
        }
    }

    /**
     * @return array|null
     */
    public function getSearchFormTab()
    {
        /** @var User $visitor */
        $visitor = \XF::visitor();
        if (!\is_callable([$visitor, 'canViewKB']) || !$visitor->canViewKB())
        {
            return null;
        }

        return [
            'title' => \XF::phrase('nf_tickets_search_kb_articles'),
            'order' => 5000,
        ];
    }

    /**
     * @return string
     */
    public function getSectionContext()
    {
        return 'nfKnowledgeBase';
    }

    /**
     * @return \XF\Mvc\Entity\Repository|\NF\Tickets\Repository\KBCategory
     */
    protected function getCategoryRepo()
    {
        return \XF::repository('NF\Tickets:KBCategory');
    }
}