<?php

namespace NF\Tickets\Search\Data;

use NF\Tickets\XF\Entity\User;
use XF\Mvc\Entity\Entity;
use XF\Search\Data\AbstractData;
use XF\Search\IndexRecord;
use XF\Search\MetadataStructure;
use XF\Search\Query\MetadataConstraint;

class Ticket extends AbstractData
{
    public function getEntityWith($forView = false): array
    {
        $get = ['User', 'Category'];
        if ($forView)
        {
            $get[] = 'FirstMessage';
            $get[] = 'Status';

            $visitor = \XF::visitor();
            $get[] = 'Category.Permissions|' . $visitor->permission_combination_id;
        }

        return $get;
    }

    public function getIndexData(Entity $entity): ?IndexRecord
    {
        /** @var \NF\Tickets\Entity\Ticket $entity */
        if (!$entity->Category)
        {
            return null;
        }

        $firstMessage = $entity->FirstMessage;

        $index = IndexRecord::create('nf_tickets_ticket', $entity->ticket_id, [
            'title'         => $entity->title,
            'message'       => $firstMessage ? $firstMessage->message : '',
            'date'          => $entity->start_date,
            'user_id'       => $entity->getAnonymousUser(false)->user_id,
            'discussion_id' => $entity->ticket_id,
            'metadata'      => $this->getMetaData($entity),
        ]);

        if (!$entity->isVisible())
        {
            $index->setHidden();
        }

        return $index;
    }

    protected function getMetaData(\NF\Tickets\Entity\Ticket $ticket): array
    {
        $metadata = [
            'ticketcat' => $ticket->ticket_category_id,
            'ticket'    => $ticket->ticket_id,
            'ticket_status' => $ticket->status_id,
        ];

        if ($ticket->assigner_user_id)
        {
            $metadata['assigner_user'] = $ticket->assigner_user_id;
        }

        if ($ticket->assigned_user_id)
        {
            $metadata['assigned_user'] = $ticket->assigned_user_id;
        }

        return $metadata;
    }

    public function setupMetadataStructure(MetadataStructure $structure): void
    {
        $structure->addField('ticketcat', MetadataStructure::INT);
        $structure->addField('ticket', MetadataStructure::INT);
        $structure->addField('ticket_status', MetadataStructure::INT);
        $structure->addField('assigned_user', MetadataStructure::INT);
        $structure->addField('assigner_user', MetadataStructure::INT);
    }

    public function getResultDate(Entity $entity): int
    {
        /** @var \NF\Tickets\Entity\Ticket $entity */
        return $entity->start_date;
    }

    public function getTemplateData(Entity $entity, array $options = []): array
    {
        /** @var \NF\Tickets\Entity\Ticket $entity */
        return [
            'message' => $entity->FirstMessage,
            'ticket'  => $entity,
            'options' => $options,
        ];
    }

    public function canUseInlineModeration(Entity $entity, &$error = null): bool
    {
        /** @var \NF\Tickets\Entity\Ticket $entity */
        return $entity->canUseInlineModeration($error);
    }
}