<?php

namespace NF\Tickets\Service\KBArticle;

use NF\Tickets\Entity\KBArticle;

class Editor extends \XF\Service\AbstractService
{
	use \XF\Service\ValidateAndSavableTrait;

	/**
	 * @var KBArticle
	 */
	protected $article;

	/**
	 * @var Preparer
	 */
	protected $preparer;

	public function __construct(\XF\App $app, KBArticle $article)
	{
		parent::__construct($app);
		$this->setArticle($article);
	}

	protected function setArticle(KBArticle $article)
	{
		$this->article = $article;
		$this->preparer = $this->service('NF\Tickets:KBArticle\Preparer', $article);
	}

	public function getArticle()
	{
		return $this->article;
	}

	public function getPreparer()
	{
		return $this->preparer;
	}

	public function setContent($title, $content, $format = true)
	{
		$this->article->set('title', $title);

		return $this->preparer->setContent($content, $format);
	}

	public function setAttachmentHash($hash)
	{
		$this->preparer->setAttachmentHash($hash);
	}

	protected function finalSetup() { }

	protected function _validate()
	{
		$this->finalSetup();

		$this->article->preSave();

		return $this->article->getErrors();
	}

	protected function _save()
	{
		$db = $this->db();
		$db->beginTransaction();

		$content = $this->article;

		$content->save(true, false);

		$this->preparer->afterUpdate();

		$db->commit();

		return $content;
	}
}