<?php

namespace NF\Tickets\Service\KBArticle;

use XF\Service\AbstractService;
use NF\Tickets\Entity\KBArticle;

class Preparer extends AbstractService
{
	/**
	 * @var KBArticle
	 */
	protected $article;

	/**
	 * @var string
	 */
	protected $attachmentHash;

    /**
     * @var bool
     */
    protected $logIp = true;

    /**
     * @var int
     */
    protected $userId;

    public function __construct(\XF\App $app, KBArticle $article)
	{
		parent::__construct($app);

		$this->article = $article;
        $this->userId = \XF::visitor()->user_id;
	}

	/**
	 * @return KBArticle
	 */
	public function getArticle()
	{
		return $this->article;
	}

    public function logIp(bool $logIp)
    {
        $this->logIp = $logIp;
    }


    public function setContent($content, $format = true, $checkValidity = true)
	{
		$preparer = $this->getMessagePreparer($format);
		$this->article->content = $preparer->prepare($content, $checkValidity);
		$this->article->embed_metadata = $preparer->getEmbedMetadata();

		return $preparer->pushEntityErrorIfInvalid($this->article);
	}

	/**
	 * @param bool $format
	 * @return \XF\Service\Message\Preparer
	 */
	protected function getMessagePreparer($format = true)
	{
		$options = $this->app->options();

		/** @var \XF\Service\Message\Preparer $preparer */
		$preparer = $this->service('XF:Message\Preparer', 'nf_tickets_kb_article', $this->article);
		$preparer->setConstraint('maxLength', $options->messageMaxLength * 3);

		if (!$format)
		{
			$preparer->disableAllFilters();
		}

		return $preparer;
	}

	/**
	 * @param string $hash
	 */
	public function setAttachmentHash($hash)
	{
		$this->attachmentHash = $hash;
	}

	public function afterInsert()
	{
		if ($this->attachmentHash)
		{
			$this->associateAttachments($this->attachmentHash);
		}

		if ($this->logIp)
		{
			$ip = ($this->logIp === true ? $this->app->request()->getIp() : $this->logIp);
			$this->writeIpLog($ip);
		}
	}

	public function afterUpdate()
	{
		if ($this->attachmentHash)
		{
			$this->associateAttachments($this->attachmentHash);
		}
	}

	/**
	 * @param $hash
	 */
	protected function associateAttachments($hash)
	{
		$article = $this->article;

		/** @var \XF\Service\Attachment\Preparer $inserter */
		$inserter = $this->service('XF:Attachment\Preparer');
		$associated = $inserter->associateAttachmentsWithContent($hash, 'nf_tickets_kb_article', $article->kb_article_id);
		if ($associated)
		{
			$article->fastUpdate('attach_count', $article->attach_count + $associated);
		}
	}

    /**
     * @param $ip
     */
    protected function writeIpLog($ip)
    {
        $article = $this->article;

        /** @var \XF\Repository\IP $ipRepo */
        $ipRepo = $this->repository('XF:Ip');
        $ipEnt = $ipRepo->logIp($this->userId, $ip, 'nf_tickets_kb_article', $article->kb_article_id);
        //if ($ipEnt)
        //{
        //    $article->fastUpdate('ip_id', $ipEnt->ip_id);
        //}
    }
}