<?php

namespace NF\Tickets\Service\Message;

use NF\Tickets\Entity\Message;

class Approver extends \XF\Service\AbstractService
{
    /** @var Message */
    protected $message;

    /**  @var Preparer */
    protected $messagePreparer;

    protected $notifyRunTime = 1;

    public function __construct(\XF\App $app, Message $message)
    {
        parent::__construct($app);
        $this->message = $message;
        $this->messagePreparer = $this->service('NF\Tickets:Message\Preparer', $message);
    }

    public function getMessage()
    {
        return $this->message;
    }

    public function setNotifyRunTime($time)
    {
        $this->notifyRunTime = $time;
    }

    public function approve()
    {
        if ($this->message->message_state === 'hidden')
        {
            $this->message->message_state = 'visible';
            $this->message->save();

            $this->onApprove();

            return true;
        }
        else
        {
            return false;
        }
    }

    protected function onApprove()
    {
        $message = $this->message;

        if ($message)
        {
            $this->messagePreparer->setMessage($message->message, false, false);

            // Standard notifications

            /** @var Notifier $notifier */
            $notifier = $this->service('NF\Tickets:Message\Notifier', $message, 'ticket');
            $notifier->setMentionedUserIds($this->messagePreparer->getMentionedUserIds());
            $notifier->setQuotedUserIds($this->messagePreparer->getQuotedUserIds());
            $notifier->notifyAndEnqueue($this->notifyRunTime);
        }
    }

    protected function getTicketRepo(): \NF\Tickets\Repository\Ticket
    {
        /** @noinspection PhpIncompatibleReturnTypeInspection */
        return $this->repository('NF\Tickets:Ticket');
    }
}