<?php

namespace NF\Tickets\Service\Message;

use NF\Tickets\Entity\Message;

class Deleter extends \XF\Service\AbstractService
{
    /**
     * @var Message
     */
    protected $message;

    protected $user;

    protected $alert       = false;
    protected $alertReason = '';
    protected $ticketDeleted;

    public function __construct(\XF\App $app, Message $message)
    {
        parent::__construct($app);
        $this->setMessage($message);
    }

    public function setMessage(Message $message)
    {
        $this->message = $message;
    }

    public function getMessage()
    {
        return $this->message;
    }

    public function setUser(\XF\Entity\User $user = null)
    {
        $this->user = $user;
    }

    public function getUser()
    {
        return $this->user;
    }

    public function setSendAlert($alert, $reason = null)
    {
        $this->alert = (bool)$alert;
        if ($reason !== null)
        {
            $this->alertReason = $reason;
        }
    }

    /**
     * @param string $type
     * @param string $reason
     * @return bool|null
     * @throws \XF\PrintableException
     */
    public function delete($type, $reason = '')
    {
        $deleteTicket = $this->isTicketDeleteRequired($type);
        $user = $this->user ?: \XF::visitor();

        $result = null;
        $this->ticketDeleted = $deleteTicket;
        if ($deleteTicket)
        {
            /** @var \NF\Tickets\Service\Ticket\Deleter $deleter */
            $deleter = $this->service('NF\Tickets:Ticket\Deleter', $this->message->Ticket);
            $deleter->setSendAlert($this->alert, $this->alertReason);

            return $deleter->delete($type, $reason);
        }

        $wasVisible = $this->message->message_state === 'visible';

        if ($type === 'soft')
        {
            $result = $this->message->softDelete($reason, $user);
        }
        else
        {
            $result = $this->message->delete();
        }

        if ($result && $wasVisible && $this->alert && $this->message->user_id !== $user->user_id)
        {
            /** @var \NF\Tickets\Repository\Message $messageRepo */
            $messageRepo = $this->repository('NF\Tickets:Message');
            $messageRepo->sendModeratorActionAlert($this->message, 'delete', $this->alertReason);
        }

        return $result;
    }

    public function wasTicketDeleted()
    {
        return $this->ticketDeleted;
    }

    /**
     * @param string $type
     * @return bool
     */
    protected function isTicketDeleteRequired($type)
    {
        return !$this->message->Ticket->reply_count;
    }
}