<?php

namespace NF\Tickets\Service\Message;

use NF\Tickets\Entity\Message;

class Editor extends \XF\Service\AbstractService
{
    use \XF\Service\ValidateAndSavableTrait;

    /**
     * @var Message
     */
    protected $message;

    /**
     * @var Preparer
     */
    protected $preparer;

    protected $alert       = false;
    protected $alertReason = '';

    public function __construct(\XF\App $app, Message $message)
    {
        parent::__construct($app);
        $this->setMessage($message);
    }

    /**
     * @param bool $anonymize
     */
    public function setAnonymize($anonymize)
    {
        $this->message->anonymized = (bool)$anonymize;
    }

    protected function setMessage(Message $message)
    {
        $this->message = $message;
        $this->preparer = $this->service('NF\Tickets:Message\Preparer', $message);
    }

    public function getMessage()
    {
        return $this->message;
    }

    public function getPreparer()
    {
        return $this->preparer;
    }

    public function setMessageText($message, $format = true)
    {
        return $this->preparer->setMessage($message, $format);
    }

    public function setAttachmentHash($hash)
    {
        $this->preparer->setAttachmentHash($hash);
    }

    public function setSendAlert($alert, $reason = null)
    {
        $this->alert = (bool)$alert;
        if ($reason !== null)
        {
            $this->alertReason = $reason;
        }
    }

    public function checkForSpam()
    {
        $this->preparer->checkForShadowBan();
        if ($this->message->message_state === 'visible' && \XF::visitor()->isSpamCheckRequired())
        {
            $this->preparer->checkForSpam();
        }
    }

    protected function finalSetup() { }

    protected function _validate()
    {
        $this->finalSetup();

        $this->message->preSave();

        return $this->message->getErrors();
    }

    protected function _save()
    {
        $db = $this->db();
        $db->beginTransaction();

        $message = $this->message;
        $changeAnonymize = $message->isChanged('anonymized');

        $message->save(true, false);

        if ($changeAnonymize && $message->isLastMessage())
        {
            $message->Ticket->rebuildLastMessageInfo();
            $message->Ticket->saveIfChanged();
        }

        $this->preparer->afterUpdate();

        $db->commit();

        if ($this->alert && $message->canSendModeratorActionAlert())
        {
            /** @var \NF\Tickets\Repository\Message $messageRepo */
            $messageRepo = $this->repository('NF\Tickets:Message');
            //$messageRepo->sendModeratorActionAlert($message, 'edit', $this->alertReason);
        }

        return $message;
    }
}