<?php

namespace NF\Tickets\Service\Ticket;

use NF\Tickets\Entity\Ticket;

class Approver extends \XF\Service\AbstractService
{
    /** @var Ticket */
    protected $ticket;

    /**  @var \NF\Tickets\Service\Message\Preparer */
    protected $messagePreparer;

    protected $notifyRunTime = 1;

    public function __construct(\XF\App $app, Ticket $ticket)
    {
        parent::__construct($app);
        $this->ticket = $ticket;
        $this->messagePreparer = $this->service('NF\Tickets:Message\Preparer', $ticket->FirstMessage);
    }

    public function getTicket()
    {
        return $this->ticket;
    }

    public function setNotifyRunTime($time)
    {
        $this->notifyRunTime = $time;
    }

    public function approve()
    {
        if ($this->ticket->discussion_state === 'moderated')
        {
            $this->ticket->discussion_state = 'visible';
            $this->ticket->save();

            $this->onApprove();

            return true;
        }
        else
        {
            return false;
        }
    }

    protected function onApprove()
    {
        $message = $this->ticket->FirstMessage;

        if ($message)
        {
            $this->messagePreparer->setMessage($message->message, false, false);

            // Standard notifications

            /** @var \NF\Tickets\Service\Message\Notifier $notifier */
            $notifier = $this->service('NF\Tickets:Message\Notifier', $message, 'ticket');
            $notifier->setMentionedUserIds($this->messagePreparer->getMentionedUserIds());
            $notifier->setQuotedUserIds($this->messagePreparer->getQuotedUserIds());
            $notifier->notifyAndEnqueue($this->notifyRunTime);
        }
    }

    protected function getTicketRepo(): \NF\Tickets\Repository\Ticket
    {
        /** @noinspection PhpIncompatibleReturnTypeInspection */
        return $this->repository('NF\Tickets:Ticket');
    }
}