<?php

namespace NF\Tickets\Service\Ticket;

use NF\Tickets\Entity\Ticket;

class ChangeDiscussion extends \XF\Service\AbstractService
{
    /**
     * @var Ticket
     */
    protected $ticket;

    public function __construct(\XF\App $app, Ticket $ticket)
    {
        parent::__construct($app);
        $this->ticket = $ticket;
    }

    public function getTicket()
    {
        return $this->ticket;
    }

    public function disconnectDiscussion()
    {
        $this->ticket->thread_id = 0;
        $this->ticket->save();

        return true;
    }

    public function changeThreadByUrl($threadUrl, $checkPermissions = true, &$error = null)
    {
        /** @var \XF\Repository\Thread $threadRepo */
        $threadRepo = $this->repository('XF:Thread');
        $thread = $threadRepo->getThreadFromUrl($threadUrl, null, $threadFetchError);
        if (!$thread)
        {
            $error = $threadFetchError;

            return false;
        }

        return $this->changeThreadTo($thread, $checkPermissions, $error);
    }

    public function changeThreadTo(\XF\Entity\Thread $thread, $checkPermissions = true, &$error = null)
    {
        if ($checkPermissions && !$thread->canView($viewError))
        {
            $error = $viewError ?: \XF::phrase('do_not_have_permission');

            return false;
        }

        if ($thread->thread_id === $this->ticket->thread_id)
        {
            return true;
        }

        $this->ticket->thread_id = $thread->thread_id;
        $this->ticket->save();

        return true;
    }
}