<?php

namespace NF\Tickets\Service\Ticket;

use NF\Tickets\Entity\Ticket;

class Deleter extends \XF\Service\AbstractService
{
    /**
     * @var Ticket
     */
    protected $ticket;

    protected $user;

    protected $alert       = false;
    protected $alertReason = '';

    public function __construct(\XF\App $app, Ticket $ticket)
    {
        parent::__construct($app);
        $this->setMessage($ticket);
        $this->setUser(\XF::visitor());
    }

    public function setMessage(Ticket $ticket)
    {
        $this->ticket = $ticket;
    }

    public function getMessage()
    {
        return $this->ticket;
    }

    public function setUser(\XF\Entity\User $user = null)
    {
        $this->user = $user;
    }

    public function getUser()
    {
        return $this->user;
    }

    public function setSendAlert($alert, $reason = null)
    {
        $this->alert = (bool)$alert;
        if ($reason !== null)
        {
            $this->alertReason = $reason;
        }
    }

    public function delete($type, $reason = '')
    {
        $user = $this->user;

        $result = null;

        $wasVisible = $this->ticket->discussion_state === 'visible';

        if ($type === 'soft')
        {
            $result = $this->ticket->softDelete($reason, $user);
        }
        else
        {
            $result = $this->ticket->delete();
        }

        if ($result && $wasVisible && $this->alert && $this->ticket->user_id !== $user->user_id && !$this->ticket->isShadowBanned())
        {
            /** @var \NF\Tickets\Repository\Ticket $ticketRepo */
            $ticketRepo = $this->repository('NF\Tickets:Ticket');
            $ticketRepo->sendModeratorActionAlert($this->ticket, 'delete', $this->alertReason);
        }

        return $result;
    }
}