<?php

namespace NF\Tickets\Service\Ticket;

use NF\Tickets\Entity\Ticket;

use function call_user_func;

class Move extends \XF\Service\AbstractService
{
    /**
     * @var Ticket
     */
    protected $ticket;

    protected $alert       = false;
    protected $alertReason = '';

    protected $notifyWatchers = false;

    protected $extraSetup = [];

    public function __construct(\XF\App $app, Ticket $ticket)
    {
        parent::__construct($app);
        $this->ticket = $ticket;
    }

    public function getTicket()
    {
        return $this->ticket;
    }

    public function setSendAlert($alert, $reason = null)
    {
        $this->alert = (bool)$alert;
        if ($reason !== null)
        {
            $this->alertReason = $reason;
        }
    }

    public function setNotifyWatchers($value = true)
    {
        $this->notifyWatchers = (bool)$value;
    }

    public function addExtraSetup(callable $extra)
    {
        $this->extraSetup[] = $extra;
    }

    public function move(\NF\Tickets\Entity\Category $category)
    {
        $user = \XF::visitor();

        $ticket = $this->ticket;
        $oldCategory = $ticket->Category;

        $moved = ($ticket->ticket_category_id !== $category->ticket_category_id);

        foreach ($this->extraSetup AS $extra)
        {
            call_user_func($extra, $ticket, $category);
        }

        $ticket->ticket_category_id = $category->ticket_category_id;

        if (!$ticket->preSave())
        {
            throw new \XF\PrintableException($ticket->getErrors());
        }

        $db = $this->db();
        $db->beginTransaction();

        $ticket->save(true, false);

        $db->commit();

        $isShadowBanned = $this->ticket->isShadowBanned();

        if (!$isShadowBanned && $moved && $this->alert && $ticket->canSendModeratorActionAlert())
        {
            /** @var \NF\Tickets\Repository\Ticket $ticketRepo */
            $ticketRepo = $this->repository('NF\Tickets:Ticket');
            $ticketRepo->sendModeratorActionAlert($this->ticket, 'move', $this->alertReason);
        }

        if (!$isShadowBanned && $moved && $this->notifyWatchers)
        {
            /** @var \NF\Tickets\Service\Message\Notifier $notifier */
            $notifier = $this->service('NF\Tickets:Message\Notifier', $this->ticket->FirstMessage, 'ticket');
            if ($oldCategory)
            {
                $notifier->skipUsersWatchingCategory($oldCategory);
            }
            $notifier->notifyAndEnqueue(3);
        }

        return $moved;
    }
}