<?php

namespace NF\Tickets\Service\Ticket;

use NF\Tickets\Entity\Ticket;
use XF\Entity\User;
use XF\Service\AbstractNotifier;

class Notifier extends AbstractNotifier
{
    /** @var Ticket */
    protected $ticket;

    /** @var User */
    protected $from;

    /** @var bool */
    protected $anonymized;

    /** @var string */
    protected $actionType;

    public function __construct(\XF\App $app, Ticket $ticket, User $from, $anonymized, $actionType)
    {
        parent::__construct($app);

        $this->ticket = $ticket;
        $this->from = $from;
        $this->anonymized = $anonymized;
        $this->actionType = $actionType;
    }

    /**
     * @param array $extraData
     * @return null|\XF\Service\AbstractService
     */
    public static function createForJob(array $extraData)
    {
        $app = \XF::app();
        /** @var Ticket $ticket */
        $ticket = $app->find('NF\Tickets:Ticket', $extraData['ticketId'], ['Category']);
        if (!$ticket)
        {
            return null;
        }

        $from = null;
        /** @var \XF\Repository\User $userRepo */
        $userRepo = $app->repository('XF:User');
        if (!empty($extraData['fromUserId']))
        {
            $from = $app->find('XF:User', $extraData['fromUserId']);
        }
        if (!$from && !empty($extraData['fromUsername']))
        {
            $from = $userRepo->getGuestUser($extraData['fromUsername']);
        }
        if (!$from)
        {
            $from = $userRepo->getGuestUser();
        }

        $anonymized = !empty($extraData['anonymized']);

        return \XF::service('NF\Tickets:Ticket\Notifier', $ticket, $from, $anonymized, $extraData['actionType']);
    }

    /**
     * @return array
     */
    protected function getExtraJobData()
    {
        return [
            'fromUserId' => $this->from->user_id,
            'fromUsername' => $this->from->username,
            'ticketId'   => $this->ticket->ticket_id,
            'actionType' => $this->actionType,
        ];
    }

    protected function loadNotifiers()
    {
        $notifiers = [];

        switch ($this->actionType) {
            case 'create':
                $notifiers['create'] = $this->app->notifier('NF\Tickets:Ticket\Create', $this->ticket);
                break;
            case 'invite':
                $notifiers['invite'] = $this->app->notifier('NF\Tickets:Ticket\Invite', $this->ticket);
                break;
        }

        return $notifiers;
    }

    /**
     * @param array $users
     */
    protected function loadExtraUserData(array $users)
    {
        $permCombinationIds = [];
        foreach ($users AS $user)
        {
            $id = $user->permission_combination_id;
            $permCombinationIds[$id] = $id;
        }

        $this->app->permissionCache()->cacheMultipleContentPermsForContent(
            $permCombinationIds,
            'nf_tickets_category',
            $this->ticket->Category->ticket_category_id
        );
    }

    /**
     * @param User $user
     * @return mixed
     * @throws \Exception
     */
    protected function canUserViewContent(User $user)
    {
        // This'll have to do until we think of a better way :(
        $this->em()->clearEntityCache('NF\Tickets:Ticket');
        $this->ticket = $this->em()->find('NF\Tickets:Ticket', $this->ticket->ticket_id);

        return \XF::asVisitor(
            $user,
            function () { return $this->ticket->canView(); }
        );
    }
}