<?php

namespace NF\Tickets\Service\Ticket;

use NF\Tickets\Entity\Ticket;

class Reassign extends \XF\Service\AbstractService
{
    /**
     * @var Ticket
     */
    protected $ticket;

    /**
     * @var \XF\Entity\User
     */
    private $user;

    public function __construct(\XF\App $app, Ticket $ticket)
    {
        parent::__construct($app);
        $this->ticket = $ticket;
        $this->user = \XF::visitor();
    }

    public function getTicket()
    {
        return $this->ticket;
    }

    public function reassignTo(\XF\Entity\User $newUser)
    {
        $ticket = $this->ticket;
        $oldUser = $ticket->Assignee;
        $reassigned = ($oldUser && $oldUser->user_id !== $newUser->user_id);

        $ticket->assigner_user_id = $this->user->user_id;
        $ticket->assigner_username = $this->user->user_id;
        $ticket->assigned_user_id = $newUser->user_id;
        $ticket->assigned_username = $newUser->username;
        $ticket->save();

        if ($reassigned && $ticket->isVisible())
        {
            if (\XF::visitor()->user_id !== $oldUser->user_id)
            {
                /** @var \NF\Tickets\Repository\Ticket $ticketRepo */
                $ticketRepo = $this->repository('NF\Tickets:Ticket');
                $ticketRepo->sendModeratorActionAlert(
                    $this->ticket, 'reassign_from', '', ['to' => $newUser->username], $oldUser
                );
            }

            if (\XF::visitor()->user_id !== $newUser->user_id)
            {
                /** @var \NF\Tickets\Repository\Ticket $ticketRepo */
                $ticketRepo = $this->repository('NF\Tickets:Ticket');
                $ticketRepo->sendModeratorActionAlert(
                    $this->ticket, 'reassign_to', '', [], $newUser
                );
            }
        }

        return $reassigned;
    }

    public function unassign()
    {
        $ticket = $this->ticket;

        $ticket->assigner_user_id = 0;
        $ticket->assigner_username = '';
        $ticket->assigned_user_id = 0;
        $ticket->assigned_username = '';
        $ticket->save();
    }
}