<?php

namespace NF\Tickets\Spam\Cleaner;

class Ticket extends \XF\Spam\Cleaner\AbstractHandler
{
    public function canCleanUp(array $options = []): bool
    {
        return !empty($options['action_threads']);
    }

    public function cleanUp(array &$log, &$error = null): bool
    {
        $app = \XF::app();

        $ticketFinder = $app->finder('NF\Tickets:Ticket');
        $tickets = $ticketFinder->where('user_id', $this->user->user_id)->fetch();

        if ($tickets->count())
        {
            $submitter = $app->container('spam.contentSubmitter');
            $submitter->submitSpam('nf_tickets_ticket', $tickets->keys());

            $deleteType = $app->options()->spamMessageAction == 'delete' ? 'hard' : 'soft';

            $log['nf_tickets_ticket'] = [
                'action'     => 'deleted',
                'deleteType' => $deleteType,
                'ticketIds'  => []
            ];

            foreach ($tickets as $ticketId => $ticket)
            {
                $log['nf_tickets_ticket']['ticketIds'][] = $ticketId;

                /** @var \NF\Tickets\Entity\Ticket $ticket */
                $ticket->setOption('log_moderator', false);
                if ($deleteType === 'soft')
                {
                    $ticket->softDelete();
                }
                else
                {
                    $ticket->delete();
                }
            }
        }


        return true;
    }

    public function restore(array $log, &$error = null): bool
    {
        $app = \XF::app();

        if ($log['deleteType'] === 'soft')
        {
            $ticketFinder = $app->finder('NF\Tickets:Ticket');
            $tickets = $ticketFinder->where('ticket_id', $log['ticketIds'])->fetch();
            foreach ($tickets as $ticket)
            {
                /** @var \NF\Tickets\Entity\Ticket $ticket */
                $ticket->setOption('log_moderator', false);
                $ticket->discussion_state = 'visible';
                $ticket->save();
            }
        }

        return true;
    }
}