<?php

namespace NF\Tickets\Spam\Cleaner;

class TicketMessage extends \XF\Spam\Cleaner\AbstractHandler
{
    public function canCleanUp(array $options = []): bool
    {
        return !empty($options['action_threads']);
    }

    public function cleanUp(array &$log, &$error = null): bool
    {
        $app = \XF::app();

        /** @var \NF\Tickets\Finder\Message $messageFinder */
        $messageFinder = $app->finder('NF\Tickets:Message');
        $messages = $messageFinder
            ->with('Ticket', true)
            ->where('user_id', $this->user->user_id)
            ->isNotFirstMessage()
            ->fetch();

        if ($messages->count())
        {
            $messageIds = $messages->pluckNamed('message_id');
            $submitter = $app->container('spam.contentSubmitter');
            $submitter->submitSpam('nf_tickets_message', $messageIds);

            $deleteType = $app->options()->spamMessageAction == 'delete' ? 'hard' : 'soft';

            $log['nf_tickets_message'] = [
                'deleteType'       => $deleteType,
                'ticketMessageIds' => []
            ];

            foreach ($messages as $messageId => $message)
            {
                $log['nf_tickets_message']['ticketMessageIds'][] = $messageId;

                /** @var \NF\Tickets\Entity\Message $message */
                $message->setOption('log_moderator', false);
                if ($deleteType === 'soft')
                {
                    $message->softDelete();
                }
                else
                {
                    $message->delete();
                }
            }
        }

        return true;
    }

    public function restore(array $log, &$error = null): bool
    {
        /** @var \NF\Tickets\Finder\Message $messageFinder */
        $messageFinder = \XF::app()->finder('NF\Tickets:Message');

        if ($log['deleteType'] === 'soft')
        {
            $messages = $messageFinder->where('message_id', $log['ticketMessageIds'])->fetch();
            foreach ($messages as $message)
            {
                /** @var \NF\Tickets\Entity\Message $message */
                $message->setOption('log_moderator', false);
                $message->message_state = 'visible';
                $message->save();
            }
        }

        return true;
    }
}