<?php

namespace NF\Tickets\SubContainer;

use NF\Tickets\EmailTickets\Parser22;
use NF\Tickets\EmailTickets\Processor22;
use NF\Tickets\Entity\Category;
use XF\Container;
use XF\SubContainer\AbstractSubContainer;

class TicketEmail extends AbstractSubContainer
{
    public function initialize()
    {
        $container = $this->container;

        $container->factory('storage', function($categoryId) use ($container)
        {
            return $this->processor($categoryId)->getTicketCategoryEmailHandlerStorage();
        }, false);

        $container->factory('parser', function($categoryId)
        {
            /** @var Category $category */
            $category = $this->app->em()->find('NF\Tickets:Category', $categoryId);

            $class = $this->app->extendClass('NF\Tickets\EmailTickets\Parser22');

            return new $class(
                $category->incoming_ticket_email_settings['address'],
                $this->app->config('globalSalt')
            );
        });

        $container->factory('processor', function($categoryId, array $params, Container $c)
        {
            /** @var Category $category */
            $category = $this->app->em()->find('NF\Tickets:Category', $categoryId);

            $class = $this->app->extendClass('NF\Tickets\EmailTickets\Processor22');

            return new $class($this->app, $c->create('parser', $categoryId), $category);
        });
    }

    /**
     * @param $categoryId
     * @return \Laminas\Mail\Storage\AbstractStorage
     */
    public function storage($categoryId)
    {
        if ($categoryId instanceof Category)
        {
            $categoryId = $categoryId->ticket_category_id;
        }

        return $this->container->create('storage', $categoryId);
    }

    /**
     * @param $categoryId
     * @return Parser22
     */
    public function parser($categoryId)
    {
        if ($categoryId instanceof Category)
        {
            $categoryId = $categoryId->ticket_category_id;
        }

        return $this->container->create('parser', $categoryId);
    }

    /**
     * @param $categoryId
     * @return Processor22
     */
    public function processor($categoryId)
    {
        if ($categoryId instanceof Category)
        {
            $categoryId = $categoryId->ticket_category_id;
        }

        return $this->container->create('processor', $categoryId);
    }
}