<?php

namespace NF\Tickets\Tag;

use XF\Mvc\Entity\Entity;
use XF\Tag\AbstractHandler;

class Ticket extends AbstractHandler
{
    public function getPermissionsFromContext(Entity $entity)
    {
        if ($entity instanceof \NF\Tickets\Entity\Ticket)
        {
            $ticket = $entity;
            $category = $ticket->Category;
        }
        else if ($entity instanceof \NF\Tickets\Entity\Category)
        {
            $ticket = null;
            $category = $entity;
        }
        else
        {
            throw new \InvalidArgumentException("Entity must be a ticket or category");
        }

        $visitor = \XF::visitor();


        if ($ticket)
        {
            if ($ticket->user_id === $visitor->user_id && $ticket->hasPermission('manageOthersTagsOwnTicket'))
            {
                $removeOthers = true;
            }
            else
            {
                $removeOthers = (bool)$ticket->hasPermission('manageAnyTag');
            }

            $edit = $ticket->canEditTags();
        }
        else
        {
            $removeOthers = false;
            $edit = $category->canEditTags();
        }

        return [
            'edit'         => $edit,
            'removeOthers' => $removeOthers,
            'minTotal'     => $category->min_tags,
        ];
    }

    public function getContentDate(Entity $entity)
    {
        /** @var \NF\Tickets\Entity\Ticket $entity */
        return $entity->start_date;
    }

    public function getContentVisibility(Entity $entity)
    {
        /** @var \NF\Tickets\Entity\Ticket $entity */
        return $entity->discussion_state === 'visible';
    }

    public function getTemplateData(Entity $entity, array $options = [])
    {
        return [
            'ticket'  => $entity,
            'options' => $options,
        ];
    }

    public function getEntityWith($forView = false)
    {
        $get = ['Category'];
        if ($forView)
        {
            $get[] = 'User';

            $visitor = \XF::visitor();
            $get[] = 'Category.Permissions|' . $visitor->permission_combination_id;
        }

        return $get;
    }

    public function canUseInlineModeration(Entity $entity, &$error = null)
    {
        /** @var \NF\Tickets\Entity\Ticket $entity */
        return $entity->canUseInlineModeration($error);
    }
}