<?php

namespace NF\Tickets\Warning;

use XF\Entity\Warning;
use XF\Mvc\Entity\Entity;
use XF\Warning\AbstractHandler;

use function is_string, strlen;

class Message extends AbstractHandler
{
    public function getStoredTitle(Entity $entity)
    {
        /** @var \NF\Tickets\Entity\Message $entity */
        return $entity->Ticket ? $entity->Ticket->title : '';
    }

    public function getDisplayTitle($title)
    {
        return \XF::phrase('nf_tickets_message_in_ticket_x', ['title' => $title]);
    }

    public function getContentForConversation(Entity $entity)
    {
        /** @var \NF\Tickets\Entity\Message $entity */
        return $entity->message;
    }

    public function getContentUrl(Entity $entity, $canonical = false)
    {
        return \XF::app()->router('public')->buildLink(($canonical ? 'canonical:' : '') . 'tickets/messages', $entity);
    }

    public function getContentUser(Entity $entity)
    {
        /** @var \NF\Tickets\Entity\Message $entity */
        return $entity->User;
    }

    public function canViewContent(Entity $entity, &$error = null)
    {
        /** @var \NF\Tickets\Entity\Message $entity */
        return $entity->canView();
    }

    public function onWarning(Entity $entity, Warning $warning)
    {
        /** @var \NF\Tickets\Entity\Message $entity */
        $entity->warning_id = $warning->warning_id;
        $entity->save();
    }

    public function onWarningRemoval(Entity $entity, Warning $warning)
    {
        /** @var \NF\Tickets\Entity\Message $entity */
        $entity->warning_id = 0;
        $entity->warning_message = '';
        $entity->save();
    }

    public function takeContentAction(Entity $entity, $action, array $options)
    {
        /** @var \NF\Tickets\Entity\Message $entity */
        if ($action === 'public')
        {
            $message = $options['message'] ?? '';
            if (is_string($message) && strlen($message))
            {
                $entity->warning_message = $message;
                $entity->save();
            }
        }
        else if ($action === 'delete')
        {
            $reason = $options['reason'] ?? '';
            if (!is_string($reason))
            {
                $reason = '';
            }

            /** @var \NF\Tickets\Service\Message\Deleter $deleter */
            $deleter = \XF::app()->service('NF\Tickets:Message\Deleter', $entity);
            $deleter->delete('soft', $reason);
        }
    }

    protected function canWarnPublicly(Entity $entity)
    {
        return true;
    }

    protected function canDeleteContent(Entity $entity)
    {
        /** @var \NF\Tickets\Entity\Message $entity */
        return $entity->canDelete('soft');
    }

    public function getEntityWith()
    {
        $visitor = \XF::visitor();

        return ['Ticket', 'Ticket.Category', 'Ticket.Category.Permissions|' . $visitor->permission_combination_id];
    }
}