<?php
/**
 * @noinspection PhpMultipleClassDeclarationsInspection
 */

namespace NF\Tickets\XF\Entity;

use NF\Tickets\Repository\Category as TicketCategory;
use XF\Entity\LinkableInterface;
use function is_array;

/**
 * Extends \XF\Entity\Search
 */
class Search extends XFCP_Search
{
    protected function setupConstraintFields(): void
    {
        parent::setupConstraintFields();

        $this->svIgnoreConstraint[] = 'nfTickets_child_categories';
    }

    protected function expandStructuredSearchConstraint(array &$query, string $key, $value): bool
    {
        if ($key === 'nfTickets_categories' && is_array($value))
        {
            /** @var TicketCategory $categoryRepo */
            $categoryRepo = \XF::repository('NF\Tickets:Category');
            $categories = $categoryRepo->getViewableCategories();

            // This can be a ticket category or XFRM/etc :(
            // for now assume tickets
            foreach ($value as $subKey => $id)
            {
                $id = (int)$id;
                if ($id === 0)
                {
                    continue;
                }

                $category = $categories[$id] ?? null;
                if ($category instanceof LinkableInterface)
                {
                    $query[$key . '_' . $subKey] = \XF::phrase('svSearchConstraint.ticket_categories', [
                        'url'  => $category->getContentUrl(),
                        'category' => $category->getContentTitle(),
                    ]);
                }
            }
        }

        return parent::expandStructuredSearchConstraint($query, $key, $value);
    }
}