<?php

namespace NF\Tickets\XF\Entity;

use NF\Tickets\Entity\Ticket;
use XF\Mvc\Entity\Structure;

/**
 * Extends \XF\Entity\Thread
 *
 * GETTER
 * @property-read Ticket $Ticket
 *
 * RELATIONSHIP
 * @property-read Ticket $Ticket_
 */
class Thread extends XFCP_Thread
{
    public function canConvertToTicket(&$error = null)
    {
        return \XF::visitor()->hasNodePermission($this->node_id, 'convertToTicket');
    }

    protected function getTicket()
    {
        if ($this->discussion_type !== 'nf_tickets_ticket')
        {
            return null;
        }

        return $this->Ticket_;
    }

    protected function _preSave()
    {
        $ticket = $this->Ticket;

        parent::_preSave();

        // if we have a ticket, force the thread type back to ticket
        // ideally this would be using a proper Forum Thread Type handler, but this is a quick fix
        if ($ticket && $this->discussion_type === 'discussion')
        {
            $this->discussion_type = 'nf_tickets_ticket';
        }
    }

    /**
     * @param Structure $structure
     * @return Structure
     */
    public static function getStructure(Structure $structure)
    {
        $structure = parent::getStructure($structure);

        $structure->relations['Ticket'] = [
            'entity' => 'NF\Tickets:Ticket',
            'type' => self::TO_ONE,
            'conditions' => 'thread_id',
        ];
        $structure->getters['Ticket'] = true;
        $structure->options['nfTickets'] = true;

        return $structure;
    }
}