<?php

namespace NF\Tickets\XF\Pub\Controller;

use NF\Tickets\Entity\Category;
use NF\Tickets\Service\Ticket\ConversationConverter;
use XF\Entity\ConversationMaster;
use XF\Mvc\ParameterBag;

/**
 * Extends \XF\Pub\Controller\Conversation
 */
class Conversation extends XFCP_Conversation
{
    public function actionConvertToTicket(ParameterBag $params)
    {
        $conversation = $this->assertViewableUserConversation($params->conversation_id);

        if (!\XF::visitor()->hasPermission('conversation', 'convertToTicket'))
        {
            return $this->noPermission();
        }

        if ($this->isPost())
        {
            $categoryId = $this->filter('category_id', 'uint');
            /** @var Category $ticketCategory */
            $ticketCategory = $this->em()->find('NF\Tickets:Category', $categoryId);

            if (!$ticketCategory)
            {
                return $this->notFound();
            }

            if (!$ticketCategory->canCreateTicket($error))
            {
                return $this->noPermission($error);
            }

            $converter = $this->setupConversationTicketConverter($conversation->Master, $ticketCategory);

            if (!$converter->validate($errors))
            {
                return $this->error($errors);
            }

            $converter->save();
            $converter->sendNotifications();

            $converter->deleteConversation();

            $ticket = $converter->getTicket();

            return $this->redirect($this->buildLink('tickets', $ticket));
        }
        else
        {
            /** @var \NF\Tickets\Repository\Category $categoryRepo */
            $categoryRepo = $this->repository('NF\Tickets:Category');

            $categories = $categoryRepo->getViewableCategories();
            $canAdd = false;
            $error = null;

            /** @var Category $category */
            foreach ($categories AS $category)
            {
                if ($category->canCreateTicket($error))
                {
                    $canAdd = true;
                    break;
                }
            }

            if (!$canAdd)
            {
                return $this->noPermission($error);
            }

            $categoryTree = $categoryRepo->createCategoryTree($categories);
            $categoryTree = $categoryTree->filter(null, function ($id, Category $category, $depth, $children) {
                if ($children)
                {
                    return true;
                }

                if ($category->canCreateTicket())
                {
                    return true;
                }

                return false;
            });

            $categoryExtras = $categoryRepo->getCategoryListExtras($categoryTree);

            $viewParams = [
                'conversation' => $conversation->Master,

                'categoryTree'   => $categoryTree,
                'categoryExtras' => $categoryExtras,
            ];

            return $this->view('NF\Tickets:Conversation\Convert', 'nf_tickets_conversation_convert_category', $viewParams);
        }
    }

    /**
     * @param ConversationMaster $conversation
     * @param Category           $category
     * @return ConversationConverter
     */
    protected function setupConversationTicketConverter(ConversationMaster $conversation, Category $category)
    {
        /** @var ConversationConverter $converter */
        $converter = $this->service('NF\Tickets:Ticket\ConversationConverter', $conversation, $category);

        return $converter;
    }
}