<?php

namespace NF\Tickets\XF\Pub\Controller;


use NF\Tickets\XF\Entity\User;
use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\ParameterBag;

/**
 * Extends \XF\Pub\Controller\Misc
 */
class Misc extends XFCP_Misc
{
    protected function preDispatchType($action, ParameterBag $params)
    {
        $allowBanned = \XF::options()->nfTicketsAllowBannedTicketUse ?? false;
        if ($allowBanned && \XF::visitor()->is_banned)
        {
            $actionL = \preg_replace('#[^a-z0-9]#i', ' ', $action);
            $actionL = \str_replace(' ', '', ucwords($actionL));

            if ($actionL === 'Captcha')
            {
                // possible banned user using tickets
                $request = $this->request();
                $redirect = $request->filter('_xfRequestUri', 'str');
                if ($redirect)
                {
                    $routePath = $this->app()->request()->getRoutePathFromUrl($redirect);
                    $routeMatch = $this->app()->router('public')->routeToController($routePath);

                    if ($routeMatch && \strpos($routeMatch->getController(), 'NF\Tickets:') === 0)
                    {
                        /** @var \NF\Tickets\ControllerPlugin\Ticket $plugin */
                        $plugin = $this->plugin('NF\Tickets:Ticket');
                        $plugin->preDispatchForTicket($this->isDiscouraged(), $action, $params);
                    }
                }
            }
        }

        parent::preDispatchType($action, $params);
    }

    public function actionContact()
    {
        $options = \XF::options();
        if ($options->nfTicketsContactUs)
        {
            /** @var User $visitor */
            $visitor = \XF::visitor();
            if ($visitor->canViewTickets())
            {
                /** @var \NF\Tickets\ControllerPlugin\TicketList $queueListPlugin */
                $queueListPlugin = $this->plugin('NF\Tickets:TicketList');

                $categoryParams = $queueListPlugin->getCategoryListData();
                /** @var AbstractCollection $categories */
                $categories = $categoryParams['categories'];
                $categories = $categories->filter(function (\NF\Tickets\Entity\Category $category) {
                    return $category->canCreateTicket();
                });

                if ($categories->count() > 0)
                {
                	return $this->rerouteController('NF\Tickets:Ticket', 'create');
                }
            }
        }

        return parent::actionContact();
    }
}