<?php

namespace NF\Tickets\XF\Pub\Controller;

use NF\Tickets\Entity\Category;
use NF\Tickets\Service\Ticket\ThreadConverter;
use XF\Mvc\ParameterBag;

/**
 * Extends \XF\Pub\Controller\Thread
 */
class Thread extends XFCP_Thread
{
    public function actionConvertToTicket(ParameterBag $params)
    {
        /** @var \NF\Tickets\XF\Entity\Thread $thread */
        $thread = $this->assertViewableThread($params->thread_id);

        if (!$thread->canConvertToTicket($error))
        {
            return $this->noPermission($error);
        }

        if ($this->isPost())
        {
            $categoryId = $this->filter('category_id', 'uint');
            /** @var Category $ticketCategory */
            $ticketCategory = $this->em()->find('NF\Tickets:Category', $categoryId);

            if (!$ticketCategory)
            {
                return $this->notFound();
            }

            if (!$ticketCategory->canCreateTicket($error))
            {
                return $this->noPermission($error);
            }

            $converter = $this->setupThreadTicketConverter($thread, $ticketCategory);

            if (!$converter->validate($errors))
            {
                return $this->error($errors);
            }

            $converter->save();
            $converter->sendNotifications();

            $converter->deleteThread();

            $ticket = $converter->getTicket();

            return $this->redirect($this->buildLink('tickets', $ticket));
        }
        else
        {
            /** @var \NF\Tickets\Repository\Category $categoryRepo */
            $categoryRepo = $this->repository('NF\Tickets:Category');

            $categories = $categoryRepo->getViewableCategories();
            $canAdd = false;

            /** @var Category $category */
            foreach ($categories AS $category)
            {
                if ($category->canCreateTicket($error))
                {
                    $canAdd = true;
                    break;
                }
            }

            if (!$canAdd)
            {
                return $this->noPermission($error);
            }

            $categoryTree = $categoryRepo->createCategoryTree($categories);
            $categoryTree = $categoryTree->filter(null, function ($id, Category $category, $depth, $children) {
                if ($children)
                {
                    return true;
                }

                if ($category->canCreateTicket())
                {
                    return true;
                }

                return false;
            });

            $categoryExtras = $categoryRepo->getCategoryListExtras($categoryTree);

            $viewParams = [
                'thread' => $thread,

                'categoryTree'   => $categoryTree,
                'categoryExtras' => $categoryExtras,
            ];

            return $this->view('NF\Tickets:Thread\Convert', 'nf_tickets_thread_convert_category', $viewParams);
        }
    }

    /**
     * @param \XF\Entity\Thread $thread
     * @param Category          $category
     * @return ThreadConverter
     */
    protected function setupThreadTicketConverter(\XF\Entity\Thread $thread, Category $category)
    {
	    $options = $this->filter([
		    'starter_alert' => 'bool',
		    'starter_alert_reason' => 'str',
		    'prefix_id' => 'uint'
	    ]);

        /** @var ThreadConverter $converter */
        $converter = $this->service('NF\Tickets:Ticket\ThreadConverter', $thread, $category);

	    if ($options['starter_alert'])
	    {
		    $converter->setSendAlert(true, $options['starter_alert_reason']);
	    }

        return $converter;
    }
}